import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/map";
import * as tsplus_module_3 from "@effect/core/io/Cause/operations/fold";
import { Both, Die, Empty, Fail, Interrupt, Stackless, Then } from "@effect/core/io/Cause/definition";
/**
 * Converts the specified `Cause<Maybe<E>>` to an `Maybe[Cause[E]]` by
 * recursively stripping out any failures with the error `None`.
 *
 * @tsplus static effect/core/io/Cause.Ops flipCauseMaybe
 */

export function flipCauseMaybe(cause) {
  return tsplus_module_3.fold(tsplus_module_1.some(new Empty()), failureMaybe => tsplus_module_2.map(e => new Fail(e))(failureMaybe), defect => tsplus_module_1.some(new Die(defect)), fiberId => tsplus_module_1.some(new Interrupt(fiberId)), (left, right) => {
    if (left._tag === "Some" && right._tag === "Some") {
      return tsplus_module_1.some(new Then(left.value, right.value));
    }

    if (left._tag === "None" && right._tag === "Some") {
      return tsplus_module_1.some(right.value);
    }

    if (left._tag === "Some" && right._tag === "None") {
      return tsplus_module_1.some(left.value);
    }

    return tsplus_module_1.none;
  }, (left, right) => {
    if (left._tag === "Some" && right._tag === "Some") {
      return tsplus_module_1.some(new Both(left.value, right.value));
    }

    if (left._tag === "None" && right._tag === "Some") {
      return tsplus_module_1.some(right.value);
    }

    if (left._tag === "Some" && right._tag === "None") {
      return tsplus_module_1.some(left.value);
    }

    return tsplus_module_1.none;
  }, (causeMaybe, stackless) => tsplus_module_2.map(cause => new Stackless(cause, stackless))(causeMaybe))(cause);
}
//# sourceMappingURL=flipCauseMaybe.mjs.map