import * as tsplus_module_1 from "@effect/printer/Doc/definition";
import * as tsplus_module_2 from "@effect/core/io/FiberId/operations/ids";
import * as tsplus_module_3 from "@effect/printer/Doc/operations/spaces";
import * as tsplus_module_4 from "@effect/printer/Doc/operations/hcat";
import * as tsplus_module_5 from "@tsplus/stdlib/io/Eval/zipWith";
import * as tsplus_module_6 from "@tsplus/stdlib/io/Eval/map";
import * as tsplus_module_7 from "@tsplus/stdlib/io/Eval/succeed";
import * as tsplus_module_8 from "@tsplus/stdlib/io/Eval/suspend";
import * as tsplus_module_9 from "@effect/printer/Optimize/definition";
import * as tsplus_module_10 from "@effect/printer/Doc/operations/appendWithLineBreak";
import * as tsplus_module_11 from "@effect/printer/Doc/operations/concatWith";
import * as tsplus_module_12 from "@effect/printer/Optimize/optimize";
import * as tsplus_module_13 from "@tsplus/stdlib/io/Eval/run";
import { realCause } from "@effect/core/io/Cause/definition";
import { renderPretty } from "@effect/printer/Render";
export function Failure(lines) {
  return {
    _tag: "Failure",
    lines
  };
}
export function Sequential(all) {
  return {
    _tag: "Sequential",
    all
  };
}
export function Parallel(all) {
  return {
    _tag: "Parallel",
    all
  };
}
const box = {
  horizontal: {
    light: /*#__PURE__*/tsplus_module_1.char("─"),
    heavy: /*#__PURE__*/tsplus_module_1.char("═")
  },
  vertical: {
    heavy: /*#__PURE__*/tsplus_module_1.char("║")
  },
  branch: {
    right: {
      heavy: /*#__PURE__*/tsplus_module_1.char("╠")
    },
    down: {
      light: /*#__PURE__*/tsplus_module_1.char("╥"),
      heavy: /*#__PURE__*/tsplus_module_1.char("╦")
    }
  },
  terminal: {
    down: {
      heavy: /*#__PURE__*/tsplus_module_1.char("╗")
    }
  },
  arrow: {
    down: /*#__PURE__*/tsplus_module_1.char("▼")
  }
};

function isNonEmptyArray(array) {
  return array.length > 0;
}

function headTail(a) {
  const x = [...a]; // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

  const head = x.shift();
  return [head, x];
}

function lines(s) {
  return s.split("\n").map(s => s.replace("\r", ""));
}

function renderToString(u) {
  if (typeof u === "object" && u != null && "toString" in u && typeof u["toString"] === "function" && u["toString"] !== Object.prototype.toString) {
    return u["toString"]();
  }

  return JSON.stringify(u, null, 2);
}

function times(value, n) {
  const array = [];

  for (let i = 0; i < n; i = i + 1) {
    array.push(value);
  }

  return array;
}

function renderFail(error // trace: O.Option<Trace>,
// traceRenderer: TraceRenderer
) {
  return Sequential([Failure([tsplus_module_1.text("A checked error was not handled."), tsplus_module_1.empty, ...error // ...renderTrace(trace, traceRenderer)
  ])]);
}

function renderDie(error // trace: O.Option<Trace>,
// traceRenderer: TraceRenderer
) {
  return Sequential([Failure([tsplus_module_1.text("An unchecked error was produced."), tsplus_module_1.empty, ...error // ...renderTrace(trace, traceRenderer)
  ])]);
}

function renderInterrupt(fiberId // trace: O.Option<Trace>,
// traceRenderer: TraceRenderer
) {
  const ids = Array.from(tsplus_module_2.ids(fiberId)).map(id => `#${id}`).join(", ");
  return Sequential([Failure([tsplus_module_1.text(`An interrupt was produced by ${ids}.`) // "",
  // ...renderTrace(trace, traceRenderer)
  ])]);
}

function renderError(error) {
  return lines(error.stack ? error.stack : String(error));
}

function prefixBlock(values, prefix1, prefix2) {
  if (isNonEmptyArray(values)) {
    const [head, tail] = headTail(values);
    const init = tsplus_module_1.cat(prefix1, head);
    const rest = tail.map(value => tsplus_module_1.cat(prefix2, value));
    return [init, ...rest];
  }

  return [];
}

function format(segment) {
  switch (segment._tag) {
    case "Failure":
      {
        return prefixBlock(segment.lines, box.horizontal.light, tsplus_module_1.char(" "));
      }

    case "Parallel":
      {
        const spaces = tsplus_module_3.spaces(2);
        const horizontalLines = tsplus_module_1.cat(box.horizontal.heavy, box.horizontal.heavy);
        const verticalSeparator = tsplus_module_1.cat(spaces, box.vertical.heavy);
        const junction = tsplus_module_1.cat(horizontalLines, box.branch.down.heavy);
        const busTerminal = tsplus_module_1.cat(horizontalLines, box.terminal.down.heavy);
        const fiberBus = tsplus_module_4.hcat([...times(junction, segment.all.length - 1), busTerminal]);
        const segments = segment.all.reduceRight((acc, curr) => [...prefixBlock(acc, verticalSeparator, verticalSeparator), ...prefixBlock(format(curr), spaces, spaces)], []);
        return [fiberBus, ...segments];
      }

    case "Sequential":
      {
        return segment.all.flatMap(step => [box.vertical.heavy, ...prefixBlock(format(step), box.branch.right.heavy, box.vertical.heavy), box.arrow.down]);
      }
  }
}

function linearSegments(cause, renderer) {
  void 0;

  switch (cause._tag) {
    case "Then":
      {
        return tsplus_module_5.zipWith(linearSegments(cause.right, renderer), (left, right) => [...left, ...right])(linearSegments(cause.left, renderer));
      }

    default:
      {
        return tsplus_module_6.map(sequential => sequential.all)(causeToSequential(cause, renderer));
      }
  }
}

function parallelSegments(cause, renderer) {
  void 0;

  switch (cause._tag) {
    case "Both":
      {
        return tsplus_module_5.zipWith(parallelSegments(cause.right, renderer), (left, right) => [...left, ...right])(parallelSegments(cause.left, renderer));
      }

    default:
      {
        return tsplus_module_6.map(sequential => [sequential])(causeToSequential(cause, renderer));
      }
  }
}

function causeToSequential(cause, renderer) {
  void 0;

  switch (cause._tag) {
    case "Empty":
      {
        return tsplus_module_7.succeed(() => Sequential([]));
      }

    case "Fail":
      {
        return tsplus_module_7.succeed(() => renderFail(renderer.renderError(cause.value).map(line => tsplus_module_1.text(line))));
      }

    case "Die":
      {
        return tsplus_module_7.succeed(() => renderDie(renderer.renderUnknown(cause.value).map(line => tsplus_module_1.text(line))));
      }

    case "Interrupt":
      {
        return tsplus_module_7.succeed(() => renderInterrupt(cause.fiberId));
      }

    case "Then":
      {
        return tsplus_module_6.map(segments => Sequential(segments))(linearSegments(cause, renderer));
      }

    case "Both":
      {
        return tsplus_module_6.map(segments => Sequential([Parallel(segments)]))(parallelSegments(cause, renderer));
      }

    case "Stackless":
      {
        // TODO: determine if this is correct for `Stackless` cause
        return tsplus_module_8.suspend(() => causeToSequential(cause.cause, renderer));
      }
  }
}

function defaultErrorToLines(error) {
  return error instanceof Error ? renderError(error) : lines(renderToString(error));
}

export const defaultRenderer = {
  lineWidth: 80,
  ribbonFraction: 1,
  renderError: defaultErrorToLines,
  renderUnknown: defaultErrorToLines
};

function prettyDocuments(cause, renderer) {
  return tsplus_module_6.map(sequential => {
    if (sequential.all.length === 1 && sequential.all[0] && sequential.all[0]._tag === "Failure") {
      return sequential.all[0].lines;
    }

    const documents = format(sequential);
    return documents.length > 0 ? [box.branch.down.light, ...documents] : documents;
  })(causeToSequential(cause, renderer));
}

function prettySafe(cause, renderer) {
  return tsplus_module_6.map(docs => {
    const document = tsplus_module_12.optimize(tsplus_module_9.Deep)(tsplus_module_1.cat(tsplus_module_1.lineBreak, tsplus_module_11.concatWith(docs, (x, y) => tsplus_module_10.appendWithLineBreak(y)(x))));
    return renderPretty(renderer.lineWidth, renderer.ribbonFraction)(document);
  })(prettyDocuments(cause, renderer));
}
/**
 * Returns a `String` with the cause pretty-printed.
 *
 * @tsplus static effect/core/io/Cause.Aspects pretty
 * @tsplus pipeable effect/core/io/Cause pretty
 */


export function pretty(renderer = defaultRenderer) {
  return self => tsplus_module_13.run(prettySafe(self, renderer));
}
//# sourceMappingURL=pretty.mjs.map