import * as tsplus_module_1 from "@effect/core/io/Effect/operations/failCause";
import * as tsplus_module_2 from "@effect/core/io/Cause/operations/failureOrCause";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Either/fold";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/foldCauseEffect";
/**
 * Recovers from errors by accepting one effect to execute for the case of an
 * error, and one effect to execute for the case of success.
 *
 * This method has better performance than `either` since no intermediate
 * value is allocated and does not require subsequent calls to `chain` to
 * define the next effect.
 *
 * The error parameter of the returned `IO` may be chosen arbitrarily, since
 * it will depend on the `IO`s returned by the given continuations.
 *
 * @tsplus static effect/core/io/Effect.Aspects foldEffect
 * @tsplus pipeable effect/core/io/Effect foldEffect
 */

export function foldEffect(failure, success) {
  return self => tsplus_module_4.foldCauseEffect(cause => tsplus_module_3.fold(failure, tsplus_module_1.failCause)(tsplus_module_2.failureOrCause(cause)), success)(self);
}
//# sourceMappingURL=foldEffect.mjs.map