import * as tsplus_module_1 from "@tsplus/stdlib/data/Either/definition";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/fromEither";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/failSync";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/getOrFail";
import * as tsplus_module_7 from "@tsplus/stdlib/service/Tag";
import * as tsplus_module_8 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/suspendSucceed";
import * as tsplus_module_11 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_12 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_13 from "@effect/core/io/Scope/operations/close";
import * as tsplus_module_14 from "@effect/core/io/Scope/operations/make";
/**
 * inspired by https://github.com/tusharmath/qio/pull/22 (revised)
 */

export const _GenR = /*#__PURE__*/Symbol.for("@effect/core/io/Effect/Gen/R");
export const _GenE = /*#__PURE__*/Symbol.for("@effect/core/io/Effect/Gen/E");
export const _GenA = /*#__PURE__*/Symbol.for("@effect/core/io/Effect/Gen/A");
export class GenEffect {
  constructor(effect) {
    this.effect = effect;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}

function adapter(_, __) {
  if (tsplus_module_1.isEither(_)) {
    return new GenEffect(tsplus_module_2.fromEither(_));
  }

  if (tsplus_module_3.isMaybe(_)) {
    if (__ && typeof __ === "function") {
      return new GenEffect(_._tag === "None" ? tsplus_module_4.failSync(() => __()) : tsplus_module_5.sync(() => _.value));
    }

    return new GenEffect(tsplus_module_6.getOrFail(_));
  }

  if (tsplus_module_7.Tag.is(_)) {
    return new GenEffect(tsplus_module_8.service(_));
  }

  return new GenEffect(_);
}
/**
 * @tsplus static effect/core/io/Effect.Ops genWithManaged
 */


export function genScoped(f) {
  return tsplus_module_10.suspendSucceed(() => {
    const iterator = f(adapter);
    const state = iterator.next();

    function run(scope, state) {
      if (state.done) {
        return tsplus_module_9.succeed(state.value);
      }

      return tsplus_module_11.flatMap(val => {
        const next = iterator.next(val);
        return run(scope, next);
      })(tsplus_module_10.suspendSucceed(() => state.value.effect));
    }

    return tsplus_module_11.flatMap(scope => tsplus_module_8.acquireUseReleaseExit(tsplus_module_12.unit, () => run(scope, state), (_, exit) => tsplus_module_13.close(exit)(scope)))(tsplus_module_14.make);
  });
}
/**
 * @tsplus static effect/core/io/Effect.Ops gen
 */

export function gen(f) {
  return tsplus_module_10.suspendSucceed(() => {
    const iterator = f(adapter);
    const state = iterator.next();

    function run(state) {
      if (state.done) {
        return tsplus_module_9.succeed(state.value);
      }

      return tsplus_module_11.flatMap(val => run(iterator.next(val)))(tsplus_module_10.suspendSucceed(() => state.value["effect"]));
    }

    return run(state);
  });
}
//# sourceMappingURL=gen.mjs.map