import * as tsplus_module_1 from "@effect/core/io/LogLevel/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_3 from "@effect/core/io/Cause/definition";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/withFiberRuntime";
import * as tsplus_module_6 from "@effect/core/io/LogSpan/definition";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_8 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/suspendSucceed";
import * as tsplus_module_10 from "@effect/core/io/Clock/operations/currentTime";
import * as tsplus_module_11 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_12 from "@tsplus/stdlib/collections/ImmutableMap/set";
const someFatal = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Fatal);
const someError = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Error);
const someWarning = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Warning);
const someTrace = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Trace);
const someInfo = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Info);
const someDebug = /*#__PURE__*/tsplus_module_2.some(tsplus_module_1.Debug);
/**
 * Logs the specified message at the current log level.
 *
 * @tsplus static effect/core/io/Effect.Ops log
 */

export function log(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, tsplus_module_2.none);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the debug log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logDebug
 */

export function logDebug(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someDebug);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the debug log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logDebugCause
 */

export function logDebugCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someDebug);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the debug log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logDebugCauseMessage
 */

export function logDebugCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someDebug);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logError
 */

export function logError(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someError);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the error log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logErrorCause
 */

export function logErrorCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someError);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the error log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logErrorCauseMessage
 */

export function logErrorCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someError);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the fatal log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logFatal
 */

export function logFatal(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someFatal);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the fatal log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logFatalCause
 */

export function logFatalCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someFatal);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the fatal log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logFatalCauseMessage
 */

export function logFatalCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someFatal);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the informational log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logInfo
 */

export function logInfo(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someInfo);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the informational log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logInfoCause
 */

export function logInfoCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someInfo);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the informational log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logInfoCauseMessage
 */

export function logInfoCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someInfo);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the warning log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logWarning
 */

export function logWarning(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someWarning);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the warning log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logWarningCause
 */

export function logWarningCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someWarning);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the warning log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logWarningCauseMessage
 */

export function logWarningCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someWarning);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message at the trace log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logTrace
 */

export function logTrace(message) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, tsplus_module_3.empty, someTrace);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified cause at the trace log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logTraceCause
 */

export function logTraceCause(cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log("", cause, someTrace);
    return tsplus_module_4.unit;
  });
}
/**
 * Logs the specified message and cause at the trace log level.
 *
 * @tsplus static effect/core/io/Effect.Ops logTraceCauseMessage
 */

export function logTraceCauseMessage(message, cause) {
  return tsplus_module_5.withFiberRuntime(fiberState => {
    fiberState.log(message, cause, someTrace);
    return tsplus_module_4.unit;
  });
}
/**
 * Adjusts the label for the current logging span.
 *
 * @tsplus static effect/core/io/Effect.Ops logSpan
 */

export function logSpan(label) {
  return effect => tsplus_module_11.flatMap(stack => tsplus_module_11.flatMap(now => tsplus_module_9.suspendSucceed(() => {
    const logSpan = tsplus_module_6.apply(label, now);
    return tsplus_module_8.currentLogSpan.locally(tsplus_module_7.prepend(logSpan)(stack))(effect);
  }))(tsplus_module_10.currentTime))(tsplus_module_8.currentLogSpan.get);
}
/**
 * Annotates each log in this effect with the specified log annotation.
 *
 * @tsplus static effect/core/io/Effect.Ops logAnnotate
 */

export function logAnnotate(key, value) {
  return effect => tsplus_module_11.flatMap(annotations => tsplus_module_9.suspendSucceed(() => tsplus_module_8.currentLogAnnotations.locally(tsplus_module_12.set(key, value)(annotations))(effect)))(tsplus_module_8.currentLogAnnotations.get);
}
/**
 * Retrieves the log annotations associated with the current scope.
 *
 * @tsplus static effect/core/io/Effect.Ops logAnnotations
 */

export function logAnnotations() {
  return tsplus_module_8.currentLogAnnotations.get;
}
//# sourceMappingURL=logging.mjs.map