import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/definition";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/mutable/ListBuffer";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/suspendSucceed";
/**
 * Loops with the specified effectual function, collecting the results into a
 * list. The moral equivalent of:
 *
 * ```typescript
 * let s  = initial
 * let as = [] as readonly A[]
 *
 * while (cont(s)) {
 *   as = [body(s), ...as]
 *   s  = inc(s)
 * }
 *
 * A.reverse(as)
 * ```
 *
 * @tsplus static effect/core/io/Effect.Ops loop
 */

export function loop(initial, cont, inc) {
  return body => {
    return tsplus_module_2.map(listBuffer => tsplus_module_1.from(listBuffer))(loopInternal(initial, cont, inc, body));
  };
}

function loopInternal(initial, cont, inc, body) {
  return tsplus_module_6.suspendSucceed(() => {
    return cont(initial) ? tsplus_module_3.flatMap(a => tsplus_module_2.map(as => {
      as.prepend(a);
      return as;
    })(loopInternal(inc(initial), cont, inc, body)))(body(initial)) : tsplus_module_5.sync(() => tsplus_module_4.ListBuffer.empty());
  });
}
//# sourceMappingURL=loop.mjs.map