import * as tsplus_module_1 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/suspendSucceed";
/**
 * Retrieves the maximum number of fibers for parallel operators or `None` if
 * it is unbounded.
 *
 * @tsplus static effect/core/io/Effect.Ops parallelism
 */

export function parallelism() {
  return tsplus_module_1.currentParallelism.get;
}
/**
 * Retrieves the current maximum number of fibers for parallel operators and
 * uses it to run the specified effect.
 *
 * @tsplus static effect/core/io/Effect.Ops parallelismWith
 */

export function parallelismWith(f) {
  return tsplus_module_1.currentParallelism.getWith(f);
}
/**
 * Runs the specified effect with the specified maximum number of fibers for
 * parallel operators.
 *
 * @tsplus static effect/core/io/Effect.Aspects withParallelism
 * @tsplus pipeable effect/core/io/Effect withParallelism
 */

export function withParallelism(n) {
  return self => tsplus_module_3.suspendSucceed(() => tsplus_module_1.currentParallelism.locally(tsplus_module_2.some(n))(self));
}
/**
 * Runs the specified effect with an unbounded maximum number of fibers for
 * parallel operators.
 *
 * @tsplus getter effect/core/io/Effect withParallelismUnbounded
 */

export function withParallelismUnbounded(self) {
  return tsplus_module_3.suspendSucceed(() => tsplus_module_1.currentParallelism.locally(tsplus_module_2.none)(self));
}
//# sourceMappingURL=parallelism.mjs.map