import * as tsplus_module_1 from "@effect/core/io/Effect/operations/fail";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/async";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/tryCatch";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/attempt";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/die";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/sync";
/**
 * Create an `Effect` that when executed will construct `promise` and wait for
 * its result, errors will be handled using `onReject`.
 *
 * @tsplus static effect/core/io/Effect.Ops tryCatchPromise
 */

export function tryCatchPromise(promise, onReject) {
  return tsplus_module_5.flatMap(promise => tsplus_module_3._async(resolve => {
    promise.then(a => resolve(tsplus_module_2.succeed(a))).catch(e => resolve(tsplus_module_1.fail(onReject(e))));
  }))(tsplus_module_4.tryCatch(promise, onReject));
}
/**
 * Create an `Effect` that when executed will construct `promise` and wait for
 * its result, errors will produce failure as `unknown`.
 *
 * @tsplus static effect/core/io/Effect.Ops tryPromise
 */

export function tryPromise(promise) {
  return tsplus_module_5.flatMap(promise => tsplus_module_3._async(resolve => {
    promise.then(a => resolve(tsplus_module_2.succeed(a))).catch(e => resolve(tsplus_module_1.fail(e)));
  }))(tsplus_module_6.attempt(promise));
}
/**
 * Like `tryPromise` but produces a defect in case of errors.
 *
 * @tsplus static effect/core/io/Effect.Ops promise
 */

export function promise(promise) {
  return tsplus_module_5.flatMap(promise => tsplus_module_3._async(resolve => {
    promise.then(a => resolve(tsplus_module_2.succeed(a))).catch(e => resolve(tsplus_module_7.die(e)));
  }))(tsplus_module_8.sync(promise));
}
//# sourceMappingURL=promise.mjs.map