import * as tsplus_module_1 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_2 from "@effect/core/io/Cause/definition";
import * as tsplus_module_3 from "@effect/core/io/Fiber/operations/join";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/mapErrorCause";
import * as tsplus_module_5 from "@effect/core/io/Fiber/operations/interruptAs";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/as";
import * as tsplus_module_7 from "@effect/core/io/Exit/operations/foldEffect";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/raceWith";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/withFiberRuntime";
import * as tsplus_module_10 from "@tsplus/stdlib/data/Either/definition";
import * as tsplus_module_11 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_12 from "@effect/core/io/Effect/operations/exit";
import * as tsplus_module_13 from "@effect/core/io/Effect/operations/flatten";
export const race = race_1;
export const raceAwait = raceAwait_1;
/**
 * Returns an effect that races this effect with the specified effect,
 * returning the first successful `A` from the faster side. If one effect
 * succeeds, the other will be interrupted. If neither succeeds, then the
 * effect will fail with some error.
 *
 * Note that both effects are disconnected before being raced. This means that
 * interruption of the loser will always be performed in the background. This
 * is a change in behavior compared to ZIO 2.0. If this behavior is not
 * desired, you can use [[Effect#raceWith]], which will not disconnect or
 * interrupt losers.
 *
 * @tsplus static effect/core/io/Effect.Aspects race
 * @tsplus pipeable effect/core/io/Effect race
 */

function race_1(that) {
  return self => raceAwait_1(tsplus_module_1.disconnect(that))(tsplus_module_1.disconnect(self));
}
/**
 * Returns an effect that races this effect with the specified effect,
 * returning the first successful `A` from the faster side. If one effect
 * succeeds, the other will be interrupted. If neither succeeds, then the
 * effect will fail with some error.
 *
 * @tsplus static effect/core/io/Effect.Aspects raceAwait
 * @tsplus pipeable effect/core/io/Effect raceAwait
 */


function raceAwait_1(that) {
  return self => tsplus_module_9.withFiberRuntime(state => tsplus_module_8.raceWith(that, (exit, right) => tsplus_module_7.foldEffect(cause => tsplus_module_4.mapErrorCause(_ => tsplus_module_2.combinePar(cause, _))(tsplus_module_3.join(right)), a => tsplus_module_6.as(a)(tsplus_module_5.interruptAsNow(state.id)(right)))(exit), (exit, left) => tsplus_module_7.foldEffect(cause => tsplus_module_4.mapErrorCause(_ => tsplus_module_2.combinePar(_, cause))(tsplus_module_3.join(left)), a => tsplus_module_6.as(a)(tsplus_module_5.interruptAsNow(state.id)(left)))(exit))(self));
}
/**
 * Returns an effect that races this effect with the specified effect,
 * yielding the first result to succeed. If neither effect succeeds, then the
 * composed effect will fail with some error.
 *
 * WARNING: The raced effect will safely interrupt the "loser", but will not
 * resume until the loser has been cleanly terminated.
 *
 * @tsplus static effect/core/io/Effect.Aspects raceEither
 * @tsplus pipeable effect/core/io/Effect raceEither
 */


export function raceEither(that) {
  return self => race_1(tsplus_module_11.map(tsplus_module_10.right)(that))(tsplus_module_11.map(tsplus_module_10.left)(self));
}
/**
 * Returns an effect that races this effect with the specified effect,
 * yielding the first result to complete, whether by success or failure. If
 * neither effect completes, then the composed effect will not complete.
 *
 * WARNING: The raced effect will safely interrupt the "loser", but will not
 * resume until the loser has been cleanly terminated. If early return is
 * desired, then instead of performing `l raceFirst r`, perform
 * `l.disconnect raceFirst r.disconnect`, which disconnects left and right
 * interrupt signal, allowing a fast return, with interruption performed
 * in the background.
 *
 * @tsplus static effect/core/io/Effect.Aspects raceFirst
 * @tsplus pipeable effect/core/io/Effect raceFirst
 */

export function raceFirst(that) {
  return self => tsplus_module_13.flatten(race_1(tsplus_module_12.exit(that))(tsplus_module_12.exit(self)));
}
//# sourceMappingURL=race.mjs.map