import * as tsplus_module_1 from "@tsplus/stdlib/data/AtomicBoolean";
import * as tsplus_module_2 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/HashSet/operations/from";
import * as tsplus_module_5 from "@effect/core/io/FiberId/operations/combineAll";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/async";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/withFiberRuntime";
import { unsafeForkUnstarted } from "@effect/core/io/Fiber/_internal/runtime";
/**
 * Forks this effect and the specified effect into their own fibers, and races
 * them, calling one of two specified callbacks depending on which fiber wins
 * the race. This method does not interrupt, join, or otherwise do anything
 * with the fibers. It can be considered a low-level building block for
 * higher-level operators like `race`.
 *
 * @tsplus static effect/core/io/Effect.Aspects raceFibersWith
 * @tsplus pipeable effect/core/io/Effect raceFibersWith
 */

export function raceFibersWith(that, selfWins, thatWins) {
  return self => tsplus_module_7.withFiberRuntime((parentState, parentStatus) => {
    const parentRuntimeFlags = parentStatus.runtimeFlags;
    const raceIndicator = new tsplus_module_1.AtomicBoolean(true);
    const leftFiber = unsafeForkUnstarted(self, parentState, parentRuntimeFlags);
    const rightFiber = unsafeForkUnstarted(that, parentState, parentRuntimeFlags);
    leftFiber.setFiberRef(tsplus_module_2.forkScopeOverride, tsplus_module_3.some(parentState.scope));
    rightFiber.setFiberRef(tsplus_module_2.forkScopeOverride, tsplus_module_3.some(parentState.scope));
    return tsplus_module_6.asyncBlockingOn(cb => {
      leftFiber.addObserver(() => complete(leftFiber, rightFiber, selfWins, raceIndicator, cb));
      rightFiber.addObserver(() => complete(rightFiber, leftFiber, thatWins, raceIndicator, cb));
      leftFiber.startFork(self);
      rightFiber.startFork(that);
    }, tsplus_module_5.combineAll(tsplus_module_4.from([leftFiber.id, rightFiber.id])));
  });
}

function complete(winner, loser, cont, ab, cb) {
  if (ab.compareAndSet(true, false)) {
    cb(cont(winner, loser));
  }
}
//# sourceMappingURL=raceFibersWith.mjs.map