import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/definition";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/suspendSucceed";
import { concreteChunkId } from "@tsplus/stdlib/collections/Chunk";
/**
 * Folds over the elements in this chunk from the left.
 * Stops the fold early when the condition is not fulfilled.
 *
 * @tsplus static effect/core/io/Effect.Ops reduceWhile
 */

export function reduceWhile(self, s, p, f) {
  const chunk = tsplus_module_1.from(self);

  const iterator = chunk._arrayLikeIterator();

  const next = iterator.next();

  if (next.done) {
    return tsplus_module_2.succeed(s);
  } else {
    const array = next.value;
    const length = array.length;
    return reduceWhileLoop(s, iterator, array, 0, length, p, f);
  }
}

function reduceWhileLoop(s, iterator, array, i, length, p, f) {
  if (i < length) {
    if (p(s)) {
      return tsplus_module_3.flatMap(s1 => reduceWhileLoop(s1, iterator, array, i + 1, length, p, f))(f(s, array[i]));
    } else {
      return tsplus_module_2.succeed(s);
    }
  } else {
    const next = iterator.next();

    if (next.done) {
      return tsplus_module_2.succeed(s);
    } else {
      const arr = next.value;
      return tsplus_module_4.suspendSucceed(() => reduceWhileLoop(s, iterator, arr, 0, arr.length, p, f));
    }
  }
}
//# sourceMappingURL=reduceWhile.mjs.map