import * as tsplus_module_1 from "@effect/core/io/Exit/operations/interrupt";
import * as tsplus_module_2 from "@effect/core/io/Fiber/operations/done";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/flatMap";
import { realFiber } from "@effect/core/io/Fiber/definition";
/**
 * A fiber that is already interrupted.
 *
 * @tsplus static effect/core/io/Fiber.Ops interruptAs
 */

export function interruptAs(fiberId) {
  return tsplus_module_2.done(tsplus_module_1.interrupt(fiberId));
}
/**
 * Interrupts the fiber as if interrupted from the specified fiber. If the
 * fiber has already exited, the returned effect will resume immediately.
 * Otherwise, the effect will resume when the fiber exits.
 *
 * @tsplus static effect/core/io/Fiber.Aspects interruptAs
 * @tsplus static effect/core/io/RuntimeFiber.Aspects interruptAs
 * @tsplus pipeable effect/core/io/Fiber interruptAs
 * @tsplus pipeable effect/core/io/RuntimeFiber interruptAs
 */

export function interruptAsNow(fiberId) {
  return self => {
    void 0;
    return tsplus_module_3.flatMap(() => self.await)(self.interruptAsFork(fiberId));
  };
}
/**
 * Interrupts the fiber as if interrupted from the specified fiber. If the
 * fiber has already exited, the returned effect will resume immediately.
 * Otherwise, the effect will resume when the fiber exits.
 *
 * @tsplus static effect/core/io/Fiber.Aspects interruptAsFork
 * @tsplus static effect/core/io/RuntimeFiber.Aspects interruptAsFork
 * @tsplus pipeable effect/core/io/Fiber interruptAsFork
 * @tsplus pipeable effect/core/io/RuntimeFiber interruptAsFork
 */

export function interruptAsFork(fiberId) {
  return self => {
    void 0;
    return self.interruptAsFork(fiberId);
  };
}
//# sourceMappingURL=interruptAs.mjs.map