import * as tsplus_module_1 from "@tsplus/stdlib/collections/mutable/MutableHashSet/empty";
import * as tsplus_module_2 from "@tsplus/stdlib/data/AtomicBoolean";
import * as tsplus_module_3 from "@effect/core/io/Deferred/operations/make";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_5 from "@effect/core/io/Scope/operations/make";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_7 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/Chunk/operations/single";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/suspendSucceed";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/Chunk/operations/isEmpty";
import * as tsplus_module_12 from "@effect/core/io/Scope/operations/addFinalizer";
import * as tsplus_module_13 from "@effect/core/io/Effect/operations/tap";
import * as tsplus_module_14 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_15 from "@effect/core/io/Exit/operations/interrupt";
import * as tsplus_module_16 from "@effect/core/io/Scope/operations/close";
import * as tsplus_module_17 from "@effect/core/io/Effect/operations/zipRight";
import * as tsplus_module_18 from "@effect/core/io/Effect/operations/whenEffect";
import * as tsplus_module_19 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_20 from "@effect/core/io/Effect/operations/withFiberRuntime";
import { HubSym } from "@effect/core/io/Hub/definition";
import { makeSubscription } from "@effect/core/io/Hub/operations/_internal/makeSubscription";
import { unsafePublishAll } from "@effect/core/io/Hub/operations/_internal/unsafePublishAll";
import { _In, QueueSym } from "@effect/core/io/Queue/definition";
/**
 * Creates a hub with the specified strategy.
 */

export function makeHub(hub, strategy) {
  return tsplus_module_6.flatMap(scope => tsplus_module_4.map(deferred => unsafeMakeHub(hub, tsplus_module_1.empty(), scope, deferred, new tsplus_module_2.AtomicBoolean(false), strategy))(tsplus_module_3.make()))(tsplus_module_5.make);
}

class HubImpl {
  get [_In]() {
    return a => a;
  }

  get [HubSym]() {
    return HubSym;
  }

  get [QueueSym]() {
    return QueueSym;
  }

  constructor(hub, subscribers, scope, shutdownHook, shutdownFlag, strategy) {
    this.hub = hub;
    this.subscribers = subscribers;
    this.scope = scope;
    this.shutdownHook = shutdownHook;
    this.shutdownFlag = shutdownFlag;
    this.strategy = strategy;
  }

  publish(a) {
    return tsplus_module_10.suspendSucceed(() => {
      if (this.shutdownFlag.get) {
        return tsplus_module_7.interrupt;
      }

      if (this.hub.publish(a)) {
        this.strategy.unsafeCompleteSubscribers(this.hub, this.subscribers);
        return tsplus_module_8.succeed(true);
      }

      return this.strategy.handleSurplus(this.hub, this.subscribers, tsplus_module_9.single(a), this.shutdownFlag);
    });
  }

  publishAll(as) {
    return tsplus_module_10.suspendSucceed(() => {
      if (this.shutdownFlag.get) {
        return tsplus_module_7.interrupt;
      }

      const surplus = unsafePublishAll(this.hub, as);
      this.strategy.unsafeCompleteSubscribers(this.hub, this.subscribers);

      if (tsplus_module_11.isEmpty(surplus)) {
        return tsplus_module_8.succeed(true);
      }

      return this.strategy.handleSurplus(this.hub, this.subscribers, surplus, this.shutdownFlag);
    });
  }

  get subscribe() {
    return tsplus_module_7.acquireRelease(tsplus_module_13.tap(dequeue => tsplus_module_12.addFinalizer(dequeue.shutdown)(this.scope))(makeSubscription(this.hub, this.subscribers, this.strategy)), dequeue => dequeue.shutdown);
  }

  offer(a) {
    return this.publish(a);
  }

  offerAll(as) {
    return this.publishAll(as);
  }

  get capacity() {
    return this.hub.capacity;
  }

  get size() {
    return tsplus_module_10.suspendSucceed(() => this.shutdownFlag.get ? tsplus_module_7.interrupt : tsplus_module_14.sync(() => this.hub.size));
  }

  get awaitShutdown() {
    return this.shutdownHook.await;
  }

  get isShutdown() {
    return tsplus_module_14.sync(() => this.shutdownFlag.get);
  }

  get shutdown() {
    return tsplus_module_7.uninterruptible(tsplus_module_20.withFiberRuntime(state => {
      this.shutdownFlag.set(true);
      return tsplus_module_19.asUnit(tsplus_module_18.whenEffect(this.shutdownHook.succeed(undefined), tsplus_module_17.zipRight(this.strategy.shutdown)(tsplus_module_16.close(tsplus_module_15.interrupt(state.id))(this.scope))));
    }));
  }

  get isFull() {
    return tsplus_module_4.map(size => size === this.capacity)(this.size);
  }

  get isEmpty() {
    return tsplus_module_4.map(size => size === 0)(this.size);
  }

}
/**
 * Unsafely creates a hub with the specified strategy.
 */


export function unsafeMakeHub(hub, subscribers, scope, shutdownHook, shutdownFlag, strategy) {
  return new HubImpl(hub, subscribers, scope, shutdownHook, shutdownFlag, strategy);
}
//# sourceMappingURL=makeHub.mjs.map