import * as tsplus_module_1 from "@tsplus/stdlib/collections/mutable/MutableQueue/unbounded";
import * as tsplus_module_2 from "@tsplus/stdlib/data/AtomicBoolean";
import * as tsplus_module_3 from "@effect/core/io/Deferred/operations/make";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_5 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/mutable/MutableQueue";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/suspendSucceed";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/withFiberRuntime";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_12 from "@tsplus/stdlib/collections/Chunk/operations/concat";
import * as tsplus_module_13 from "@tsplus/stdlib/collections/Chunk/operations/append";
import * as tsplus_module_14 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_15 from "@tsplus/stdlib/collections/Chunk/operations/head";
import * as tsplus_module_16 from "@effect/core/io/Effect/operations/excl-forEach";
import * as tsplus_module_17 from "@effect/core/io/Effect/operations/zipRight";
import * as tsplus_module_18 from "@effect/core/io/Effect/operations/whenEffect";
import * as tsplus_module_19 from "@effect/core/io/Effect/operations/unit";
import { unsafePollAllSubscription } from "@effect/core/io/Hub/operations/_internal/unsafePollAllSubscription";
import { unsafePollN } from "@effect/core/io/Hub/operations/_internal/unsafePollN";
import { unsafeRemove } from "@effect/core/io/Hub/operations/_internal/unsafeRemove";
import { _Out, QueueSym } from "@effect/core/io/Queue/definition";
import { unsafePollAll } from "@effect/core/io/Queue/operations/_internal/unsafePollAll";
/**
 * Creates a subscription with the specified strategy.
 */

export function makeSubscription(hub, subscribers, strategy) {
  return tsplus_module_4.map(deferred => unsafeMakeSubscription(hub, subscribers, hub.subscribe(), tsplus_module_1.unbounded(), deferred, new tsplus_module_2.AtomicBoolean(false), strategy))(tsplus_module_3.make());
}

class SubscriptionImpl {
  get [QueueSym]() {
    return QueueSym;
  }

  get [_Out]() {
    return a => a;
  }

  constructor(hub, subscribers, subscription, pollers, shutdownHook, shutdownFlag, strategy) {
    this.hub = hub;
    this.subscribers = subscribers;
    this.subscription = subscription;
    this.pollers = pollers;
    this.shutdownHook = shutdownHook;
    this.shutdownFlag = shutdownFlag;
    this.strategy = strategy;
  }

  get take() {
    return tsplus_module_10.withFiberRuntime(state => {
      if (this.shutdownFlag.get) {
        return tsplus_module_5.interrupt;
      }

      const message = this.pollers.isEmpty ? this.subscription.poll(tsplus_module_6.EmptyMutableQueue) : tsplus_module_6.EmptyMutableQueue;

      if (message === tsplus_module_6.EmptyMutableQueue) {
        const deferred = tsplus_module_5.unsafeMakeDeferred(state.id);
        return tsplus_module_5.onInterrupt(() => tsplus_module_7.sync(() => unsafeRemove(this.pollers, deferred)))(tsplus_module_8.suspendSucceed(() => {
          this.pollers.offer(deferred);
          this.subscribers.add([this.subscription, this.pollers]);
          this.strategy.unsafeCompletePollers(this.hub, this.subscribers, this.subscription, this.pollers);
          return this.shutdownFlag.get ? tsplus_module_5.interrupt : deferred.await;
        }));
      } else {
        this.strategy.unsafeOnHubEmptySpace(this.hub, this.subscribers);
        return tsplus_module_9.succeed(message);
      }
    });
  }

  get takeAll() {
    return tsplus_module_8.suspendSucceed(() => {
      if (this.shutdownFlag.get) {
        return tsplus_module_5.interrupt;
      }

      const as = this.pollers.isEmpty ? unsafePollAllSubscription(this.subscription) : tsplus_module_11.empty();
      this.strategy.unsafeOnHubEmptySpace(this.hub, this.subscribers);
      return tsplus_module_9.succeed(as);
    });
  }

  takeUpTo(max) {
    return tsplus_module_8.suspendSucceed(() => {
      if (this.shutdownFlag.get) {
        return tsplus_module_5.interrupt;
      }

      const as = this.pollers.isEmpty ? unsafePollN(this.subscription, max) : tsplus_module_11.empty();
      this.strategy.unsafeOnHubEmptySpace(this.hub, this.subscribers);
      return tsplus_module_9.succeed(as);
    });
  }

  takeRemainderLoop(self, min, max, acc) {
    if (max < min) {
      return tsplus_module_9.succeed(acc);
    }

    return tsplus_module_14.flatMap(bs => {
      const remaining = min - bs.length;

      if (remaining === 1) {
        return tsplus_module_4.map(b => tsplus_module_13.append(b)(tsplus_module_12.concat(bs)(acc)))(self.take);
      }

      if (remaining > 1) {
        return tsplus_module_14.flatMap(b => this.takeRemainderLoop(self, remaining - 1, max - bs.length - 1, tsplus_module_13.append(b)(tsplus_module_12.concat(bs)(acc))))(self.take);
      }

      return tsplus_module_9.succeed(tsplus_module_12.concat(bs)(acc));
    })(self.takeUpTo(max));
  }

  takeBetween(min, max) {
    return tsplus_module_8.suspendSucceed(() => this.takeRemainderLoop(this, min, max, tsplus_module_11.empty()));
  }

  takeN(n) {
    return this.takeBetween(n, n);
  }

  get poll() {
    return tsplus_module_4.map(chunk => tsplus_module_15.head(chunk))(this.takeUpTo(1));
  }

  get capacity() {
    return this.hub.capacity;
  }

  get size() {
    return tsplus_module_8.suspendSucceed(() => this.shutdownFlag.get ? tsplus_module_5.interrupt : tsplus_module_9.succeed(this.subscription.size));
  }

  get awaitShutdown() {
    return this.shutdownHook.await;
  }

  get isShutdown() {
    return tsplus_module_7.sync(() => this.shutdownFlag.get);
  }

  get shutdown() {
    return tsplus_module_5.uninterruptible(tsplus_module_10.withFiberRuntime(state => {
      this.shutdownFlag.set(true);
      return tsplus_module_19.asUnit(tsplus_module_18.whenEffect(this.shutdownHook.succeed(undefined), tsplus_module_17.zipRight(tsplus_module_7.sync(() => this.strategy.unsafeOnHubEmptySpace(this.hub, this.subscribers)))(tsplus_module_17.zipRight(tsplus_module_7.sync(() => this.subscription.unsubscribe()))(tsplus_module_16.forEachPar(unsafePollAll(this.pollers), deferred => deferred.interruptAs(state.id))))));
    }));
  }

  get isFull() {
    return tsplus_module_4.map(size => size === this.capacity)(this.size);
  }

  get isEmpty() {
    return tsplus_module_4.map(size => size === 0)(this.size);
  }

}
/**
 * Unsafely creates a subscription with the specified strategy.
 */


export function unsafeMakeSubscription(hub, subscribers, subscription, pollers, shutdownHook, shutdownFlag, strategy) {
  return new SubscriptionImpl(hub, subscribers, subscription, pollers, shutdownHook, shutdownFlag, strategy);
}
//# sourceMappingURL=makeSubscription.mjs.map