import * as tsplus_module_1 from "@tsplus/stdlib/collections/mutable/MutableQueue";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/prepend";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/mutable/MutableQueue/unbounded";
import * as tsplus_module_4 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/suspendSucceed";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/withFiberRuntime";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/excl-forEach";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/fiberId";
import * as tsplus_module_11 from "@effect/core/io/Effect/operations/do";
import * as tsplus_module_12 from "@effect/core/io/Effect/operations/tap";
import * as tsplus_module_13 from "@tsplus/stdlib/collections/Chunk/operations/filter";
import * as tsplus_module_14 from "@effect/core/io/Effect/operations/succeed";
import { unsafeCompleteDeferred } from "@effect/core/io/Hub/operations/_internal/unsafeCompleteDeferred";
import { unsafeOfferAll } from "@effect/core/io/Hub/operations/_internal/unsafeOfferAll";
import { unsafePollAllQueue } from "@effect/core/io/Hub/operations/_internal/unsafePollAllQueue";
export const Strategy = {};

class BaseStrategy {
  unsafeCompletePollers(hub, subscribers, subscription, pollers) {
    let keepPolling = true;

    while (keepPolling && !subscription.isEmpty) {
      const poller = pollers.poll(tsplus_module_1.EmptyMutableQueue);

      if (poller === tsplus_module_1.EmptyMutableQueue) {
        const subPollerPair = [subscription, pollers];
        subscribers.remove(subPollerPair);

        if (pollers.isEmpty) {
          keepPolling = false;
        } else {
          subscribers.add(subPollerPair);
        }
      } else {
        const pollResult = subscription.poll(tsplus_module_1.EmptyMutableQueue);

        if (pollResult == tsplus_module_1.EmptyMutableQueue) {
          unsafeOfferAll(pollers, tsplus_module_2.prepend(poller)(unsafePollAllQueue(pollers)));
        } else {
          unsafeCompleteDeferred(poller, pollResult);
          this.unsafeOnHubEmptySpace(hub, subscribers);
        }
      }
    }
  }

  unsafeCompleteSubscribers(hub, subscribers) {
    for (const [subscription, pollers] of subscribers) {
      this.unsafeCompletePollers(hub, subscribers, subscription, pollers);
    }
  }

}
/**
 * A strategy that applies back pressure to publishers when the hub is at
 * capacity. This guarantees that all subscribers will receive all messages
 * published to the hub while they are subscribed. However, it creates the
 * risk that a slow subscriber will slow down the rate at which messages
 * are published and received by other subscribers.
 */


export class BackPressure extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.publishers = tsplus_module_3.unbounded();
  }

  handleSurplus(hub, subscribers, as, isShutdown) {
    return tsplus_module_7.withFiberRuntime(state => {
      const deferred = tsplus_module_4.unsafeMakeDeferred(state.id);
      return tsplus_module_4.onInterrupt(() => tsplus_module_5.sync(() => this.unsafeRemove(deferred)))(tsplus_module_6.suspendSucceed(() => {
        this.unsafeOffer(as, deferred);
        this.unsafeOnHubEmptySpace(hub, subscribers);
        this.unsafeCompleteSubscribers(hub, subscribers);
        return isShutdown.get ? tsplus_module_4.interrupt : deferred.await;
      }));
    });
  }

  get shutdown() {
    return tsplus_module_8.asUnit(tsplus_module_12.tap(({
      fiberId,
      publishers
    }) => tsplus_module_9.forEachParDiscard(publishers, ([_, deferred, last]) => last ? deferred.interruptAs(fiberId) : tsplus_module_8.unit))(tsplus_module_11.bind("publishers", () => tsplus_module_5.sync(() => unsafePollAllQueue(this.publishers)))(tsplus_module_11.bind("fiberId", () => tsplus_module_10.fiberId)(tsplus_module_11.Do()))));
  }

  unsafeOnHubEmptySpace(hub, subscribers) {
    let keepPolling = true;

    while (keepPolling && !hub.isFull) {
      const publisher = this.publishers.poll(tsplus_module_1.EmptyMutableQueue);

      if (publisher === tsplus_module_1.EmptyMutableQueue) {
        keepPolling = false;
      } else {
        const published = hub.publish(publisher[0]);

        if (published && publisher[2]) {
          unsafeCompleteDeferred(publisher[1], true);
        } else if (!published) {
          unsafeOfferAll(this.publishers, tsplus_module_2.prepend(publisher)(unsafePollAllQueue(this.publishers)));
        }

        this.unsafeCompleteSubscribers(hub, subscribers);
      }
    }
  }

  unsafeOffer(as, deferred) {
    const it = as[Symbol.iterator]();
    let curr = it.next();

    if (!curr.done) {
      let next;

      while ((next = it.next()) && !next.done) {
        this.publishers.offer([curr.value, deferred, false]);
        curr = next;
      }

      this.publishers.offer([curr.value, deferred, true]);
    }
  }

  unsafeRemove(deferred) {
    unsafeOfferAll(this.publishers, tsplus_module_13.filter(([_, a]) => a !== deferred)(unsafePollAllQueue(this.publishers)));
  }

}
/**
 * A strategy that drops new messages when the hub is at capacity. This
 * guarantees that a slow subscriber will not slow down the rate at which
 * messages are published. However, it creates the risk that a slow
 * subscriber will slow down the rate at which messages are received by
 * other subscribers and that subscribers may not receive all messages
 * published to the hub while they are subscribed.
 */

export class Dropping extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.shutdown = tsplus_module_8.unit;
  }

  handleSurplus(_hub, _subscribers, _as, _isShutdown) {
    return tsplus_module_14.succeed(false);
  }

  unsafeOnHubEmptySpace(_hub, _subscribers) {//
  }

}
/**
 * A strategy that adds new messages and drops old messages when the hub is
 * at capacity. This guarantees that a slow subscriber will not slow down
 * the rate at which messages are published and received by other
 * subscribers. However, it creates the risk that a slow subscriber will
 * not receive some messages published to the hub while it is subscribed.
 */

export class Sliding extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.shutdown = tsplus_module_8.unit;
  }

  unsafeSlidingPublish(hub, as) {
    const it = as[Symbol.iterator]();
    let next = it.next();

    if (!next.done && hub.capacity > 0) {
      let a = next.value;
      let loop = true;

      while (loop) {
        hub.slide();
        const pub = hub.publish(a);

        if (pub && (next = it.next()) && !next.done) {
          a = next.value;
        } else if (pub) {
          loop = false;
        }
      }
    }
  }

  handleSurplus(hub, subscribers, as, _isShutdown) {
    return tsplus_module_5.sync(() => {
      this.unsafeSlidingPublish(hub, as);
      this.unsafeCompleteSubscribers(hub, subscribers);
      return true;
    });
  }

  unsafeOnHubEmptySpace(_hub, _subscribers) {//
  }

}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops BackPressure
 */

export function backPressureStrategy() {
  return new BackPressure();
}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops Dropping
 */

export function droppingStrategy() {
  return new Dropping();
}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops Sliding
 */

export function slidingStrategy() {
  return new Sliding();
}
//# sourceMappingURL=strategy.mjs.map