import * as tsplus_module_1 from "@effect/core/io/Metrics/MetricState";
import * as tsplus_module_2 from "@effect/core/io/Metrics/MetricHook";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/HashMap/operations/make";
import * as tsplus_module_4 from "@tsplus/stdlib/prelude/Ord/common";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/Chunk/operations/sort";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Chunk/operations/mapWithIndex";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/Chunk/operations/builder";
import * as tsplus_module_8 from "@tsplus/stdlib/data/Duration";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/Chunk/operations/isEmpty";
import * as tsplus_module_10 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/Chunk/operations/unsafeHead";
import * as tsplus_module_12 from "@tsplus/stdlib/collections/Chunk/operations/unsafeTail";
import * as tsplus_module_13 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_14 from "@tsplus/stdlib/collections/Chunk/operations/single";
import * as tsplus_module_15 from "@tsplus/stdlib/collections/Chunk/operations/append";
import * as tsplus_module_16 from "@tsplus/stdlib/collections/Chunk/operations/reduce";
import * as tsplus_module_17 from "@tsplus/stdlib/collections/Chunk/operations/map";
import * as tsplus_module_18 from "@tsplus/stdlib/collections/Chunk/operations/unsafeLast";
import * as tsplus_module_19 from "@tsplus/stdlib/collections/Chunk/operations/splitWhere";
import * as tsplus_module_20 from "@tsplus/stdlib/collections/Chunk/operations/head";
import { DurationInternal } from "@tsplus/stdlib/data/Duration";
export const MetricHooks = {};
/**
 * @tsplus static effect/core/io/Metrics/MetricHooks.Ops counter
 */

export function counter(_key) {
  let sum = 0;
  return tsplus_module_2.make(value => {
    sum = sum + value;
  }, () => tsplus_module_1.counter(sum));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricHooks.Ops gauge
 */

export function gauge(_key, startAt) {
  let value = startAt;
  return tsplus_module_2.make(v => {
    value = v;
  }, () => tsplus_module_1.gauge(value));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricHooks.Ops frequency
 */

export function frequency(_key) {
  let count = 0;
  const values = new Map();

  const update = word => {
    count = count + 1;
    const slotCount = values.get(word) ?? 0;
    values.set(word, slotCount + 1);
  };

  const snapshot = () => tsplus_module_3.make(...Array.from(values.entries()).map(([k, v]) => [k, v]));

  return tsplus_module_2.make(update, () => tsplus_module_1.frequency(snapshot()));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricHooks.Ops histogram
 */

export function histogram(key) {
  const bounds = key.keyType.boundaries.values;
  const size = bounds.length;
  const values = Array(size + 1);
  const boundaries = Array(size);
  let count = 0;
  let sum = 0;
  let min = Number.MAX_VALUE;
  let max = Number.MIN_VALUE;
  tsplus_module_6.mapWithIndex((i, n) => {
    boundaries[i] = n;
  })(tsplus_module_5.sort(tsplus_module_4.number)(bounds)); // Insert the value into the right bucket with a binary search

  const update = value => {
    let from = 0;
    let to = size;

    while (from !== to) {
      const mid = Math.floor(from + (to - from) / 2);
      const boundary = boundaries[mid];

      if (value <= boundary) {
        to = mid;
      } else {
        from = mid;
      } // The special case when to / from have a distance of one


      if (to === from + 1) {
        if (value <= boundaries[from]) {
          to = from;
        } else {
          from = to;
        }
      }
    }

    values[from] = values[from] + 1;
    count = count + 1;
    sum = sum + value;

    if (value < min) {
      min = value;
    }

    if (value > max) {
      max = value;
    }
  };

  const getBuckets = () => {
    const builder = tsplus_module_7.builder();
    let i = 0;
    let cumulated = 0;

    while (i != size) {
      const boundary = boundaries[i];
      const value = values[i];
      cumulated = cumulated + value;
      builder.append([boundary, cumulated]);
      i = i + 1;
    }

    return builder.build();
  };

  return tsplus_module_2.make(update, () => tsplus_module_1.histogram(getBuckets(), count, min, max, sum));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricHooks.Ops summary
 */

export function summary(key) {
  const {
    error,
    maxAge,
    maxSize,
    quantiles
  } = key.keyType;
  const sortedQuantiles = tsplus_module_5.sort(tsplus_module_4.number)(quantiles);
  const values = Array(maxSize);
  let head = 0;
  let count = 0;
  let sum = 0;
  let min = Number.MAX_VALUE;
  let max = Number.MIN_VALUE; // Just before the snapshot we filter out all values older than maxAge

  const snapshot = now => {
    const builder = tsplus_module_7.builder(); // If the buffer is not full yet it contains valid items at the 0..last
    // indices and null values at the rest of the positions.
    //
    // If the buffer is already full then all elements contains a valid
    // measurement with timestamp.
    //
    // At any given point in time we can enumerate all the non-null elements in
    // the buffer and filter them by timestamp to get a valid view of a time
    // window.
    //
    // The order does not matter because it gets sorted before passing to
    // `calculateQuantiles`.

    let i = 0;

    while (i !== maxSize - 1) {
      const item = values[i];

      if (item != null) {
        const [t, v] = item;
        const age = new DurationInternal(now - t);

        if (age.millis >= 0 && tsplus_module_8.lowerThenOrEqual(maxAge)(age)) {
          builder.append(v);
        }
      }

      i = i + 1;
    }

    return calculateQuantiles(error, sortedQuantiles, tsplus_module_5.sort(tsplus_module_4.number)(builder.build()));
  };

  const observe = (value, timestamp) => {
    if (maxSize > 0) {
      head = head + 1;
      const target = head % maxSize;
      values[target] = [timestamp, value];
    }

    count = count + 1;
    sum = sum + value;

    if (value < min) {
      min = value;
    }

    if (value > max) {
      max = value;
    }
  };

  return tsplus_module_2.make(([value, timestamp]) => observe(value, timestamp), () => tsplus_module_1.summary(error, snapshot(Date.now()), count, min, max, sum));
}
export class ResolvedQuantile {
  constructor(
  /**
   * The quantile that shall be resolved.
   */
  quantile,
  /**
   * `Some<number>` if a value for the quantile could be found, otherwise
   * `None`.
   */
  value,
  /**
   * How many samples have been consumed prior to this quantile.
   */
  consumed,
  /**
   * The rest of the samples after the quantile has been resolved.
   */
  rest) {
    this.quantile = quantile;
    this.value = value;
    this.consumed = consumed;
    this.rest = rest;
  }

}

function calculateQuantiles(error, sortedQuantiles, sortedSamples) {
  // The number of samples examined
  const sampleCount = sortedSamples.length;

  if (tsplus_module_9.isEmpty(sortedQuantiles)) {
    return tsplus_module_10.empty();
  }

  const head = tsplus_module_11.unsafeHead(sortedQuantiles);
  const tail = tsplus_module_12.unsafeTail(sortedQuantiles);
  const resolved = tsplus_module_16.reduce(tsplus_module_14.single(resolveQuantile(error, sampleCount, tsplus_module_13.none, 0, head, sortedSamples)), (acc, quantile) => {
    const h = tsplus_module_11.unsafeHead(acc);
    return tsplus_module_15.append(resolveQuantile(error, sampleCount, h.value, h.consumed, quantile, h.rest))(acc);
  })(tail);
  return tsplus_module_17.map(rq => [rq.quantile, rq.value])(resolved);
}
/**
 * @tsplus tailRec
 */


function resolveQuantile(error, sampleCount, current, consumed, quantile, rest) {
  var error_1 = error,
      sampleCount_1 = sampleCount,
      current_1 = current,
      consumed_1 = consumed,
      quantile_1 = quantile,
      rest_1 = rest;
  var error_2 = error,
      sampleCount_2 = sampleCount,
      current_2 = current,
      consumed_2 = consumed,
      quantile_2 = quantile,
      rest_2 = rest;

  while (1) {
    // If the remaining list of samples is empty, there is nothing more to resolve
    if (tsplus_module_9.isEmpty(rest_1)) {
      return new ResolvedQuantile(quantile_1, tsplus_module_13.none, consumed_1, tsplus_module_10.empty());
    } // If the quantile is the 100% quantile, we can take the maximum of all the
    // remaining values as the result


    if (quantile_1 === 1) {
      return new ResolvedQuantile(quantile_1, tsplus_module_13.some(tsplus_module_18.unsafeLast(rest_1)), consumed_1 + rest_1.length, tsplus_module_10.empty());
    } // Split into two chunks - the first chunk contains all elements of the same
    // value as the chunk head


    const sameHead = tsplus_module_19.splitWhere(n => n > tsplus_module_11.unsafeHead(rest_1))(rest_1); // How many elements do we want to accept for this quantile

    const desired = quantile_1 * sampleCount_1; // The error margin

    const allowedError = error_1 / 2 * desired; // Taking into account the elements consumed from the samples so far and the
    // number of same elements at the beginning of the chunk, calculate the number
    // of elements we would have if we selected the current head as result

    const candConsumed = consumed_1 + sameHead[0].length;
    const candError = Math.abs(candConsumed - desired); // If we haven't got enough elements yet, recurse

    if (candConsumed < desired - allowedError) {
      error_2 = error_1;
      sampleCount_2 = sampleCount_1;
      current_2 = tsplus_module_20.head(rest_1);
      consumed_2 = candConsumed;
      quantile_2 = quantile_1;
      rest_2 = sameHead[1];
      error_1 = error_2;
      sampleCount_1 = sampleCount_2;
      current_1 = current_2;
      consumed_1 = consumed_2;
      quantile_1 = quantile_2;
      rest_1 = rest_2;
      continue;
    } // If we have too many elements, select the previous value and hand back the
    // the rest as leftover


    if (candConsumed > desired + allowedError) {
      return new ResolvedQuantile(quantile_1, current_1, consumed_1, rest_1);
    } // If we are in the target interval, select the current head and hand back the leftover after dropping all elements
    // from the sample chunk that are equal to the current head


    switch (current_1._tag) {
      case "None":
        {
          error_2 = error_1;
          sampleCount_2 = sampleCount_1;
          current_2 = tsplus_module_20.head(rest_1);
          consumed_2 = candConsumed;
          quantile_2 = quantile_1;
          rest_2 = sameHead[1];
          error_1 = error_2;
          sampleCount_1 = sampleCount_2;
          current_1 = current_2;
          consumed_1 = consumed_2;
          quantile_1 = quantile_2;
          rest_1 = rest_2;
          continue;
        }

      case "Some":
        {
          const prevError = Math.abs(desired - current_1.value);

          if (candError < prevError) {
            error_2 = error_1;
            sampleCount_2 = sampleCount_1;
            current_2 = tsplus_module_20.head(rest_1);
            consumed_2 = candConsumed;
            quantile_2 = quantile_1;
            rest_2 = sameHead[1];
            error_1 = error_2;
            sampleCount_1 = sampleCount_2;
            current_1 = current_2;
            consumed_1 = consumed_2;
            quantile_1 = quantile_2;
            rest_1 = rest_2;
            continue;
          }

          return new ResolvedQuantile(quantile_1, tsplus_module_13.some(current_1.value), consumed_1, rest_1);
        }
    }
  }
}
//# sourceMappingURL=MetricHooks.mjs.map