var _a;

import * as tsplus_module_1 from "@tsplus/stdlib/collections/HashSet/operations/empty";
import * as tsplus_module_2 from "@tsplus/stdlib/structure/Hash";
import * as tsplus_module_3 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/HashSet/operations/equals";
import * as tsplus_module_5 from "@effect/core/io/Metrics/MetricKeyType";
import * as tsplus_module_6 from "@effect/core/io/Metrics/definition";
import * as tsplus_module_7 from "@effect/core/io/Metrics/MetricLabel";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/HashSet/operations/make";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/HashSet/operations/from";
import * as tsplus_module_10 from "@tsplus/stdlib/collections/HashSet/operations/size";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/HashSet/operations/union";
export const taggedWithLabelSet = taggedWithLabelSet_1;
export const isMetricKey = isMetricKey_1;
export const MetricKeySym = /*#__PURE__*/Symbol.for("@effect/core/io/Metric/MetricKey");
/**
 * A `MetricKey` is a unique key associated with each metric. The key is based
 * on a combination of the metric type, the name and tags associated with the
 * metric, and any other information to describe a a metric, such as the
 * boundaries of a histogram. In this way, it is impossible to ever create
 * different metrics with conflicting keys.
 *
 * @tsplus type effect/core/io/Metrics/MetricKey
 * @tsplus companion effect/core/io/Metrics/MetricKey.Ops
 */

export class MetricKey {
  constructor(name, keyType, tags = tsplus_module_1.empty()) {
    this.name = name;
    this.keyType = keyType;
    this.tags = tags;
    this[_a] = MetricKeySym;
  }

  [(_a = MetricKeySym, tsplus_module_2.Hash.sym)]() {
    return tsplus_module_2.combine(tsplus_module_2.hashString(this.name), tsplus_module_2.combine(tsplus_module_2.hashUnknown(this.keyType), tsplus_module_2.hashUnknown(this.tags)));
  }

  [tsplus_module_3.Equals.sym](u) {
    return isMetricKey_1(u) && this.name === u.name && tsplus_module_2.hashUnknown(this.keyType) === tsplus_module_2.hashUnknown(u.keyType) && tsplus_module_4.equals(u.tags)(this.tags);
  }

}
/**
 * Creates a metric key for a counter, with the specified name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Counter
 */

export function counter(name) {
  return new MetricKey(name, tsplus_module_5.counter);
}
/**
 * Creates a metric key for a gauge, with the specified name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Gauge
 */

export function gauge(name) {
  return new MetricKey(name, tsplus_module_5.gauge);
}
/**
 * Creates a metric key for a categorical frequency table, with the specified
 * name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Frequency
 */

export function frequency(name) {
  return new MetricKey(name, tsplus_module_5.frequency);
}
/**
 * Creates a metric key for a histogram, with the specified name and boundaries.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Histogram
 */

export function histogram(name, boundaries) {
  return new MetricKey(name, tsplus_module_5.histogram(boundaries));
}
/**
 * Creates a metric key for a histogram, with the specified name, maxAge,
 * maxSize, error, and quantiles.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Summary
 */

export function summary(name, maxAge, maxSize, error, quantiles) {
  return new MetricKey(name, tsplus_module_5.summary(maxAge, maxSize, error, quantiles));
}
/**
 * Returns a new `MetricKey` with the specified tag appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects tagged
 * @tsplus pipeable effect/core/io/Metrics/MetricKey tagged
 */

export function tagged(key, value) {
  return self => taggedWithLabelSet_1(tsplus_module_8.make(tsplus_module_7.make(key, value)))(self);
}
/**
 * Returns a new `MetricKey` with the specified tags appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects taggedWithLabels
 * @tsplus pipeable effect/core/io/Metrics/MetricKey taggedWithLabels
 */

export function taggedWithLabels(extraTags) {
  return self => taggedWithLabelSet_1(tsplus_module_9.from(extraTags))(self);
}
/**
 * Returns a new `MetricKey` with the specified tags appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects taggedWithLabelSet
 * @tsplus pipeable effect/core/io/Metrics/MetricKey taggedWithLabelSet
 */

function taggedWithLabelSet_1(extraTags) {
  return self => tsplus_module_10.size(extraTags) === 0 ? self : new MetricKey(self.name, self.keyType, tsplus_module_11.union(extraTags)(self.tags));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops isMetricKey
 */


function isMetricKey_1(u) {
  return typeof u === "object" && u != null && MetricKeySym in u;
}
//# sourceMappingURL=MetricKey.mjs.map