import * as tsplus_module_1 from "@tsplus/stdlib/collections/HashSet/operations/empty";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/suspendSucceed";
import { DurationInternal } from "@tsplus/stdlib/data/Duration";
/**
 * Returns an aspect that will update this metric with the duration that the
 * effect takes to execute. To call this method, you must supply a function
 * that can convert the `Duration` to the input type of this metric.
 *
 * @tsplus static effect/core/io/Metrics/Metric.Aspects trackDurationWith
 * @tsplus pipeable effect/core/io/Metrics/Metric trackDurationWith
 */

export function trackDurationWith(f) {
  return self => {
    return effect => tsplus_module_3.suspendSucceed(() => {
      const startTime = Date.now();
      return tsplus_module_2.map(a => {
        const endTime = Date.now();
        const duration = new DurationInternal(endTime - startTime);
        self.unsafeUpdate(f(duration), tsplus_module_1.empty());
        return a;
      })(effect);
    });
  };
}
//# sourceMappingURL=trackDurationWith.mjs.map