import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/getOrElse";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/sync";
import { _A, RefSym } from "@effect/core/io/Ref/definition";
export class UnsafeAPI {
  constructor(value) {
    this.value = value;
  }

  get get() {
    return this.value.get;
  }

  getAndSet(a) {
    const current = this.value.get;
    this.value.set(a);
    return current;
  }

  getAndUpdate(f) {
    const current = this.value.get;
    this.value.set(f(current));
    return current;
  }

  getAndUpdateSome(pf) {
    const current = this.value.get;
    const opt = pf(current);

    if (tsplus_module_1.isSome(opt)) {
      this.value.set(opt.value);
    }

    return current;
  }

  modify(f) {
    const current = this.value.get;
    const [b, a] = f(current);
    this.value.set(a);
    return b;
  }

  modifySome(fallback, pf) {
    const current = this.value.get;
    const tuple = tsplus_module_2.getOrElse_(pf(current), () => [fallback, current]);
    this.value.set(tuple[1]);
    return tuple[0];
  }

  set(a) {
    return this.value.set(a);
  }

  update(f) {
    const current = this.value.get;
    this.value.set(f(current));
  }

  updateAndGet(f) {
    const current = this.value.get;
    const next = f(current);
    this.value.set(next);
    return next;
  }

  updateSome(pf) {
    const current = this.value.get;
    const opt = pf(current);

    if (tsplus_module_1.isSome(opt)) {
      this.value.set(opt.value);
    }
  }

  updateSomeAndGet(pf) {
    const current = this.value.get;
    const next = pf(current);

    if (tsplus_module_1.isSome(next)) {
      this.value.set(next.value);
      return next.value;
    }

    return current;
  }

}
export class AtomicInternal {
  constructor(unsafe) {
    this.unsafe = unsafe;
  }

  get [_A]() {
    return a => a;
  }
  /**
   * Internal Discriminator
   */


  get [RefSym]() {
    return RefSym;
  }

  get get() {
    return tsplus_module_3.sync(() => this.unsafe.get);
  }

  modify(f) {
    return tsplus_module_3.sync(() => this.unsafe.modify(f));
  }

  set(a) {
    return tsplus_module_3.sync(() => this.unsafe.set(a));
  }

  getAndSet(a) {
    return this.modify(v => [v, a]);
  }

  getAndUpdate(f) {
    return this.modify(v => [v, f(v)]);
  }

  getAndUpdateSome(pf) {
    return this.modify(v => [v, tsplus_module_2.getOrElse_(pf(v), () => v)]);
  }

  modifySome(fallback, pf) {
    return this.modify(v => tsplus_module_2.getOrElse_(pf(v), () => [fallback, v]));
  }

  update(f) {
    return this.modify(v => [undefined, f(v)]);
  }

  updateAndGet(f) {
    return this.modify(v => {
      const result = f(v);
      return [result, result];
    });
  }

  updateSome(pf) {
    return this.modify(v => [undefined, tsplus_module_2.getOrElse_(pf(v), () => v)]);
  }

  updateSomeAndGet(pf) {
    return this.modify(v => {
      const result = tsplus_module_2.getOrElse_(pf(v), () => v);
      return [result, result];
    });
  }

}
//# sourceMappingURL=AtomicInternal.mjs.map