import * as tsplus_module_1 from "@effect/core/io/FiberId/operations/make";
import * as tsplus_module_2 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_3 from "@effect/core/io/Supervisor/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_5 from "@effect/core/io/FiberScope/definition";
import * as tsplus_module_6 from "@effect/core/io/Fiber/operations/interruptAs";
import * as tsplus_module_7 from "@effect/core/io/Exit/operations/flatten";
import * as tsplus_module_8 from "@tsplus/stdlib/data/Function";
import * as tsplus_module_9 from "@effect/core/io/Cause/operations/squashWith";
import * as tsplus_module_10 from "@effect/core/io/Exit/operations/die";
import { FiberRuntime } from "@effect/core/io/Fiber/_internal/runtime";
import { StagedScheduler } from "@effect/core/support/Scheduler";
import { constVoid } from "@tsplus/stdlib/data/Function";
export class AsyncFiber {
  constructor(fiber) {
    this.fiber = fiber;
    this._tag = "AsyncFiber";
  }

}
export class Runtime {
  constructor(environment, runtimeFlags, fiberRefs) {
    this.environment = environment;
    this.runtimeFlags = runtimeFlags;
    this.fiberRefs = fiberRefs;

    this.unsafeFork = effect => {
      const fiberId = tsplus_module_1.unsafeMake();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment);
      const context = new FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      return context;
    };

    this.unsafeRunWith = (effect, k) => {
      const fiberId = tsplus_module_1.unsafeMake();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment);
      const context = new FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      context.addObserver(exit => {
        k(exit);
      });
      return id => k => this.unsafeRunAsyncWith(tsplus_module_6.interruptAsNow(id)(context), exit => k(tsplus_module_7.flatten(exit)));
    };

    this.unsafeRunSync = effect => {
      const exit = this.unsafeRunSyncExit(effect);

      if (exit._tag === "Failure") {
        throw tsplus_module_9.squashWith(tsplus_module_8.identity)(exit.cause);
      }

      return exit.value;
    };

    this.unsafeRunSyncExit = effect => {
      const fiberId = tsplus_module_1.unsafeMake();
      const scheduler = new StagedScheduler();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment).updateAs(fiberId, tsplus_module_2.currentScheduler, scheduler);
      const context = new FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      scheduler.flush();
      const result = context.unsafePoll;

      if (result) {
        return result;
      }

      return tsplus_module_10.die(new AsyncFiber(context));
    };
    /**
     * Executes the effect asynchronously, discarding the result of execution.
     *
     * This method is effectful and should only be invoked at the edges of your
     * program.
     */


    this.unsafeRunAsync = effect => {
      return this.unsafeRunAsyncWith(effect, constVoid);
    };
    /**
     * Executes the effect asynchronously, eventually passing the exit value to
     * the specified callback.
     *
     * This method is effectful and should only be invoked at the edges of your
     * program.
     */


    this.unsafeRunAsyncWith = (effect, k) => {
      this.unsafeRunWith(effect, k);
    };
    /**
     * Runs the `Effect`, returning a JavaScript `Promise` that will be resolved
     * with the value of the effect once the effect has been executed, or will be
     * rejected with the first error or exception throw by the effect.
     *
     * This method is effectful and should only be used at the edges of your
     * program.
     */


    this.unsafeRunPromise = effect => {
      return new Promise((resolve, reject) => {
        this.unsafeRunAsyncWith(effect, exit => {
          switch (exit._tag) {
            case "Success":
              {
                resolve(exit.value);
                break;
              }

            case "Failure":
              {
                reject(tsplus_module_9.squashWith(tsplus_module_8.identity)(exit.cause));
                break;
              }
          }
        });
      });
    };
    /**
     * Runs the `Effect`, returning a JavaScript `Promise` that will be resolved
     * with the `Exit` state of the effect once the effect has been executed.
     *
     * This method is effectful and should only be used at the edges of your
     * program.
     */


    this.unsafeRunPromiseExit = effect => {
      return new Promise(resolve => {
        this.unsafeRunAsyncWith(effect, exit => {
          resolve(exit);
        });
      });
    };
  }

}
//# sourceMappingURL=definition.mjs.map