import * as tsplus_module_1 from "@tsplus/stdlib/prelude/Ord/common";
import * as tsplus_module_2 from "@tsplus/stdlib/prelude/Ord/operations";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Maybe/definition";
export const fromStartEndMillis = fromStartEndMillis_1;
export const min = min_1;
export const isEmpty = isEmpty_1;
import { DurationInternal } from "@tsplus/stdlib/data/Duration";
export const IntervalSym = /*#__PURE__*/Symbol.for("@effect/core/io/Schedule/Interval");
export const Interval = {
  $: {}
};
/**
 * An `Interval` of zero-width.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops empty
 */

const empty_1 = /*#__PURE__*/fromStartEndMillis_1(0, 0);
export const empty = empty_1;
/**
 * Constructs a new interval from the two specified endpoints. If the start
 * endpoint greater than the end endpoint, then a zero size interval will be
 * returned.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops __call
 */

function fromStartEndMillis_1(startMilliseconds, endMilliseconds) {
  return startMilliseconds > endMilliseconds ? empty_1 : {
    [IntervalSym]: IntervalSym,
    startMillis: startMilliseconds,
    endMillis: endMilliseconds
  };
}
/**
 * @tsplus pipeable-operator effect/core/io/Schedule/Interval <
 * @tsplus static effect/core/io/Schedule/Interval.Aspects lessThan
 * @tsplus pipeable effect/core/io/Schedule/Interval lessThan
 */


export function lessThan(that) {
  return self => min_1(that)(self) === self;
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects min
 * @tsplus pipeable effect/core/io/Schedule/Interval min
 */

function min_1(that) {
  return self => {
    if (self.endMillis <= that.startMillis) return self;
    if (that.endMillis <= self.startMillis) return that;
    if (self.startMillis < that.startMillis) return self;
    if (that.startMillis < self.startMillis) return that;
    if (self.endMillis <= that.endMillis) return self;
    return that;
  };
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects max
 * @tsplus pipeable effect/core/io/Schedule/Interval max
 */


export function max(that) {
  return self => min_1(that)(self) === self ? that : self;
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval isEmpty
 */

function isEmpty_1(self) {
  return self.startMillis >= self.endMillis;
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval isNonEmpty
 */


export function isNonEmpty(self) {
  return !isEmpty_1(self);
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects intersect
 * @tsplus pipeable effect/core/io/Schedule/Interval intersect
 */

export function intersect(that) {
  return self => {
    const start = Math.max(self.startMillis, that.startMillis);
    const end = Math.min(self.endMillis, that.endMillis);
    return fromStartEndMillis_1(start, end);
  };
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval size
 */

export function size(self) {
  return new DurationInternal(self.endMillis - self.startMillis);
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects union
 * @tsplus pipeable effect/core/io/Schedule/Interval union
 */

export function union(that) {
  return self => {
    const start = tsplus_module_2.max(tsplus_module_1.number)(self.startMillis, that.startMillis);
    const end = tsplus_module_2.min(tsplus_module_1.number)(self.endMillis, that.endMillis);
    return start < end ? tsplus_module_3.none : tsplus_module_3.some(fromStartEndMillis_1(start, end));
  };
}
/**
 * Construct an `Interval` that includes all time equal to and after the
 * specified start time.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops after
 */

export function after(startMilliseconds) {
  return fromStartEndMillis_1(startMilliseconds, Number.MAX_SAFE_INTEGER);
}
/**
 * Construct an `Interval` that includes all time equal to and before the
 * specified end time.
 *
 * @tsplus static effect/core/io/Schedule/Interval before
 */

export function before(endMilliseconds) {
  return fromStartEndMillis_1(Number.MIN_SAFE_INTEGER, endMilliseconds);
}
//# sourceMappingURL=Interval.mjs.map