import * as tsplus_module_1 from "@effect/core/io/Effect/operations/as";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/flatMap";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * Returns a new schedule that will run the specified finalizer as soon as the
 * schedule is complete. Note that unlike `Effect.ensuring`, this method does not
 * guarantee the finalizer will be run. The `Schedule` may not initialize or
 * the driver of the schedule may not run to completion. However, if the
 * `Schedule` ever decides not to continue, then the finalizer will be run.
 *
 * @tsplus static effect/core/io/Schedule.Aspects ensuring
 * @tsplus pipeable effect/core/io/Schedule ensuring
 */

export function ensuring(finalizer) {
  return self => makeWithState(self.initial, (now, input, state) => tsplus_module_3.flatMap(([state, out, decision]) => decision._tag === "Done" ? tsplus_module_1.as([state, out, decision])(finalizer) : tsplus_module_2.succeed([state, out, decision]))(self.step(now, input, state)));
}
//# sourceMappingURL=ensuring.mjs.map