import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@effect/core/io/Schedule/Interval";
import * as tsplus_module_3 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_4 from "@tsplus/stdlib/data/Duration";
import * as tsplus_module_5 from "@tsplus/stdlib/data/Maybe/fold";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/sync";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
import { DurationInternal } from "@tsplus/stdlib/data/Duration";
/**
 * A schedule that recurs on a fixed interval. Returns the number of
 * repetitions of the schedule so far.
 *
 * If the action run between updates takes longer than the interval, then the
 * action will be run immediately, but re-runs will not "pile up".
 *
 * ```text
 * |-----interval-----|-----interval-----|-----interval-----|
 * |---------action--------||action|-----|action|-----------|
 * ```
 *
 * @tsplus static effect/core/io/Schedule.Ops fixed
 */

export function fixed(interval) {
  return makeWithState([tsplus_module_1.empty(), 0], (now, _, [option, n]) => tsplus_module_6.sync(() => {
    const intervalMillis = interval.millis;
    return tsplus_module_5.fold(() => [[tsplus_module_1.some([now, now + intervalMillis]), n + 1], n, tsplus_module_3.continueWith(tsplus_module_2.after(now + intervalMillis))], ([startMillis, lastRun]) => {
      const runningBehind = now > lastRun + intervalMillis;
      const boundary = tsplus_module_4.equals(tsplus_module_4.millis(0))(interval) ? interval : new DurationInternal(intervalMillis - (now - startMillis) % intervalMillis);
      const sleepTime = tsplus_module_4.equals(tsplus_module_4.millis(0))(boundary) ? interval : boundary;
      const nextRun = runningBehind ? now : now + sleepTime.millis;
      return [[tsplus_module_1.some([startMillis, nextRun]), n + 1], n, tsplus_module_3.continueWith(tsplus_module_2.after(nextRun))];
    })(option);
  }));
}
//# sourceMappingURL=fixed.mjs.map