import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/unsafeGet";
import * as tsplus_module_3 from "@effect/core/io/Schedule/Interval";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/Chunk/operations/drop";
import * as tsplus_module_5 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Chunk/operations/prepend";
import * as tsplus_module_7 from "@tsplus/stdlib/data/Duration";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/sync";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * A schedule that recurs once for each of the specified durations, delaying
 * each time for the length of the specified duration. Returns the length of
 * the current duration between recurrences.
 *
 * @tsplus static effect/core/io/Schedule.Ops fromDurations
 */

export function fromDurations(duration, ...durations) {
  return makeWithState([tsplus_module_1.from([duration, ...durations]), true], (now, _, [durations, cont]) => tsplus_module_8.sync(() => {
    if (cont) {
      const x = tsplus_module_2.unsafeGet(0)(durations);
      const interval = tsplus_module_3.after(now + x.millis);

      if (durations.length >= 2) {
        return [[tsplus_module_4.drop(1)(durations), true], x, tsplus_module_5.continueWith(interval)];
      }

      const y = tsplus_module_4.drop(1)(durations);
      return [[tsplus_module_6.prepend(x)(y), false], x, tsplus_module_5.continueWith(interval)];
    }

    return [[durations, false], tsplus_module_7.millis(0), tsplus_module_5.done];
  }));
}
//# sourceMappingURL=fromDurations.mjs.map