import * as tsplus_module_1 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/zipWith";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_5 from "@effect/core/io/Schedule/Intervals";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * Returns a new schedule that combines this schedule with the specified
 * schedule, continuing as long as both schedules want to continue and merging
 * the next intervals according to the specified merge function.
 *
 * @tsplus static effect/core/io/Schedule.Aspects intersectWith
 * @tsplus pipeable effect/core/io/Schedule intersectWith
 */

export function intersectWith(that, f) {
  return self => makeWithState([self.initial, that.initial], (now, input, state) => {
    const left = self.step(now, input, state[0]);
    const right = that.step(now, input, state[1]);
    return tsplus_module_4.flatMap(([[lState, out, lDecision], [rState, out2, rDecision]]) => {
      if (lDecision._tag === "Continue" && rDecision._tag === "Continue") {
        return intersectWithLoop(self, that, input, lState, out, lDecision.intervals, rState, out2, rDecision.intervals, f);
      }

      return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
    })(tsplus_module_3.zipWith(right, (a, b) => [a, b])(left));
  });
}

function intersectWithLoop(self, that, input, lState, out, lInterval, rState, out2, rInterval, f) {
  const combined = f(lInterval, rInterval);

  if (tsplus_module_5.isNonEmpty(combined)) {
    return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1._continue(combined)]);
  }

  if (tsplus_module_5.lessThan(rInterval)(lInterval)) {
    return tsplus_module_4.flatMap(([lState, out, decision]) => {
      switch (decision._tag) {
        case "Done":
          {
            return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
          }

        case "Continue":
          {
            return intersectWithLoop(self, that, input, lState, out, decision.intervals, rState, out2, rInterval, f);
          }
      }
    })(self.step(tsplus_module_5.end(lInterval), input, lState));
  }

  return tsplus_module_4.flatMap(([rState, out2, decision]) => {
    switch (decision._tag) {
      case "Done":
        {
          return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
        }

      case "Continue":
        {
          return intersectWithLoop(self, that, input, lState, out, lInterval, rState, out2, decision.intervals, f);
        }
    }
  })(that.step(tsplus_module_5.end(rInterval), input, rState));
}
//# sourceMappingURL=intersectWith.mjs.map