import * as tsplus_module_1 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_2 from "@effect/core/io/Schedule/Intervals";
import * as tsplus_module_3 from "@effect/core/io/Schedule/Interval";
import * as tsplus_module_4 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/flatMap";
import { Interval } from "@effect/core/io/Schedule/Interval";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * Returns a new schedule that modifies the delay using the specified
 * effectual function.
 *
 * @tsplus static effect/core/io/Schedule.Aspects modifyDelayEffect
 * @tsplus pipeable effect/core/io/Schedule modifyDelayEffect
 */

export function modifyDelayEffect(f) {
  return self => makeWithState(self.initial, (now, input, state) => tsplus_module_6.flatMap(([state, out, decision]) => {
    switch (decision._tag) {
      case "Done":
        {
          return tsplus_module_1.succeed([state, out, decision]);
        }

      case "Continue":
        {
          const intervals = decision.intervals;
          const delay = tsplus_module_3.size(tsplus_module_3.fromStartEndMillis(now, tsplus_module_2.start(intervals)));
          return tsplus_module_5.map(duration => {
            const oldStart = tsplus_module_2.start(intervals);
            const newStart = now + duration.millis;
            const delta = newStart - oldStart;
            const newEnd = Math.min(Math.max(0, tsplus_module_2.end(intervals) + delta), Number.MAX_SAFE_INTEGER);
            const newInterval = tsplus_module_3.fromStartEndMillis(newStart, newEnd);
            return [state, out, tsplus_module_4.continueWith(newInterval)];
          })(f(out, delay));
        }
    }
  })(self.step(now, input, state)));
}
//# sourceMappingURL=modifyDelayEffect.mjs.map