import * as tsplus_module_1 from "@effect/core/io/Cause/errors";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/dieSync";
import * as tsplus_module_3 from "@effect/core/io/Schedule/Interval";
import * as tsplus_module_4 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/succeed";
import { Interval } from "@effect/core/io/Schedule/Interval";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
import { beginningOfSecond, endOfSecond, nextSecond } from "@effect/core/io/Schedule/operations/_internal/time";
/**
 * Cron-like schedule that recurs every specified `second` of each minute. It
 * triggers at zero nanosecond of the second. Producing a count of repeats: 0,
 * 1, 2.
 *
 * NOTE: `second` parameter is validated lazily. Must be in range 0...59.
 *
 * @tsplus static effect/core/io/Schedule.Ops secondOfMinute
 */

export function secondOfMinute(second) {
  return makeWithState([Number.MIN_SAFE_INTEGER, 0], (now, _, state) => {
    if (!Number.isInteger(second) || second < 0 || 59 < second) {
      return tsplus_module_2.dieSync(() => new tsplus_module_1.IllegalArgumentException(`Invalid argument in: secondOfMinute(${second}). Must be in range 0...59`));
    }

    const [end0, n] = state;
    const now0 = Math.max(end0, now);
    const second0 = nextSecond(now0, second);
    const start = Math.max(beginningOfSecond(second0), now0);
    const end = endOfSecond(second0);
    const interval = tsplus_module_3.fromStartEndMillis(start, end);
    return tsplus_module_5.succeed([[end, n + 1], n, tsplus_module_4.continueWith(interval)]);
  });
}
//# sourceMappingURL=secondOfMinute.mjs.map