import * as tsplus_module_1 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/zipWith";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * Returns a new schedule that combines this schedule with the specified
 * schedule, continuing as long as either schedule wants to continue and
 * merging the next intervals according to the specified merge function.
 *
 * @tsplus static effect/core/io/Schedule.Aspects unionWith
 * @tsplus pipeable effect/core/io/Schedule unionWith
 */

export function unionWith(that, f) {
  return self => makeWithState([self.initial, that.initial], (now, input, state) => {
    const left = self.step(now, input, state[0]);
    const right = that.step(now, input, state[1]);
    return tsplus_module_2.zipWith(right, ([lState, l, lDecision], [rState, r, rDecision]) => {
      if (lDecision._tag === "Done" && rDecision._tag === "Done") {
        return [[lState, rState], [l, r], tsplus_module_1.done];
      }

      if (lDecision._tag === "Done" && rDecision._tag === "Continue") {
        return [[lState, rState], [l, r], tsplus_module_1._continue(rDecision.intervals)];
      }

      if (lDecision._tag === "Continue" && rDecision._tag === "Done") {
        return [[lState, rState], [l, r], tsplus_module_1._continue(lDecision.intervals)];
      }

      if (lDecision._tag === "Continue" && rDecision._tag === "Continue") {
        const combined = f(lDecision.intervals, rDecision.intervals);
        return [[lState, rState], [l, r], tsplus_module_1._continue(combined)];
      }

      throw new Error("bug");
    })(left);
  });
}
//# sourceMappingURL=unionWith.mjs.map