import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@effect/core/io/Schedule/Interval";
import * as tsplus_module_3 from "@effect/core/io/Schedule/Decision";
import * as tsplus_module_4 from "@tsplus/stdlib/data/Maybe/fold";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/succeed";
import { makeWithState } from "@effect/core/io/Schedule/operations/_internal/makeWithState";
/**
 * A schedule that divides the timeline to `interval`-long windows, and sleeps
 * until the nearest window boundary every time it recurs.
 *
 * For example, `windowed(Duration.seconds(10))` would produce a schedule as
 * follows:
 *
 * ```text
 *      10s        10s        10s       10s
 * |----------|----------|----------|----------|
 * |action------|sleep---|act|-sleep|action----|
 * ```
 *
 * @tsplus static effect/core/io/Schedule.Ops windowed
 */

export function windowed(interval) {
  const millis = interval.millis;
  return makeWithState([tsplus_module_1.empty(), 0], (now, _, [option, n]) => tsplus_module_5.succeed(tsplus_module_4.fold(() => [[tsplus_module_1.some(now), n + 1], n, tsplus_module_3.continueWith(tsplus_module_2.after(now + millis))], startMillis => [[tsplus_module_1.some(startMillis), n + 1], n, tsplus_module_3.continueWith(tsplus_module_2.after(now + (millis - (now - startMillis) % millis)))])(option)));
}
//# sourceMappingURL=windowed.mjs.map