import * as tsplus_module_1 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/fromNullable";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Maybe/fold";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/flatten";
import { Running } from "@effect/core/io/Scope/ReleaseMap/_internal/State";
/**
 * Runs the specified finalizer and removes it from the finalizers associated
 * with this scope.
 *
 * @tsplus static effect/core/io/ReleaseMap.Aspects release
 * @tsplus pipeable effect/core/io/ReleaseMap release
 */

export function release(key, exit) {
  return self => tsplus_module_4.flatten(self.ref.modify(s => {
    switch (s._tag) {
      case "Exited":
        {
          return [tsplus_module_1.unit, s];
        }

      case "Running":
        {
          const finalizers = s.finalizers();
          const finalizer = tsplus_module_2.fromNullable(finalizers.get(key));
          finalizers.delete(key);
          return [tsplus_module_3.fold(() => tsplus_module_1.unit, fin => s.update(fin)(exit))(finalizer), new Running(s.nextKey, finalizers, s.update)];
        }
    }
  }));
}
//# sourceMappingURL=release.mjs.map