import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/data/Maybe/fromNullable";
import { Exited, Running } from "@effect/core/io/Scope/ReleaseMap/_internal/State";
/**
 * Removes the finalizer associated with this key and returns it.
 *
 * @tsplus static effect/core/io/ReleaseMap.Aspects remove
 * @tsplus pipeable effect/core/io/ReleaseMap remove
 */

export function remove(key) {
  return self => self.ref.modify(s => {
    switch (s._tag) {
      case "Exited":
        {
          return [tsplus_module_1.none, new Exited(s.nextKey, s.exit, s.update)];
        }

      case "Running":
        {
          const finalizers = s.finalizers();
          const finalizer = tsplus_module_2.fromNullable(finalizers.get(key));
          finalizers.delete(key);
          return [finalizer, new Running(s.nextKey, finalizers, s.update)];
        }
    }
  });
}
//# sourceMappingURL=remove.mjs.map