import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Maybe/fromNullable";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/flatten";
import { Exited, Running } from "@effect/core/io/Scope/ReleaseMap/_internal/State";
/**
 * Replaces the finalizer associated with this key and returns it. If the
 * finalizers associated with this scope have already been run this
 * finalizer will be run immediately.
 *
 * @tsplus static effect/core/io/ReleaseMap.Aspects replace
 * @tsplus pipeable effect/core/io/ReleaseMap replace
 */

export function replace(key, finalizer) {
  return self => tsplus_module_5.flatten(self.ref.modify(s => {
    switch (s._tag) {
      case "Exited":
        {
          return [tsplus_module_2.map(() => tsplus_module_1.none)(finalizer(s.exit)), new Exited(s.nextKey, s.exit, s.update)];
        }

      case "Running":
        {
          const finalizers = s.finalizers();
          const oldFinalizer = tsplus_module_3.fromNullable(finalizers.get(key));
          const newFinalizers = finalizers.set(key, finalizer);
          return [tsplus_module_4.succeed(oldFinalizer), new Running(s.nextKey, newFinalizers, s.update)];
        }
    }
  }));
}
//# sourceMappingURL=replace.mjs.map