import * as tsplus_module_1 from "@tsplus/stdlib/collections/List/definition";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/List/from";
import * as tsplus_module_3 from "@effect/core/io/Supervisor/definition";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/HashSet/operations/empty";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/HashSet/operations/union";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/HashSet/operations/make";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/HashSet/operations/difference";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/HashSet/operations/reduce";
import * as tsplus_module_9 from "@tsplus/stdlib/data/Differ/operations/make";
export const combinePatch = combinePatch_1;
export const applyPatch = applyPatch_1;
import { Zip } from "@effect/core/io/Supervisor/definition";
import { equals } from "@tsplus/stdlib/structure/Equals";
export class AddSupervisor {
  constructor(supervisor) {
    this.supervisor = supervisor;
    this._tag = "AddSupervisor";
  }

}
export class Empty {
  constructor() {
    this._tag = "Empty";
  }

}
export class RemoveSupervisor {
  constructor(supervisor) {
    this.supervisor = supervisor;
    this._tag = "RemoveSupervisor";
  }

}
export class AndThen {
  constructor(first, second) {
    this.first = first;
    this.second = second;
    this._tag = "AndThen";
  }

}
/**
 * Combines two patches to produce a new patch that describes applying the
 * updates from this patch and then the updates from the specified patch.
 *
 * @tsplus static effect/core/io/Supervisor.Ops combinePatch
 */

function combinePatch_1(self, that) {
  return new AndThen(self, that);
}
/**
 * An empty patch
 *
 * @tsplus static effect/core/io/Supervisor.Ops emptyPatch
 */


const emptyPatch_1 = /*#__PURE__*/new Empty();
export const emptyPatch = emptyPatch_1;
/**
 * Applies an update to the supervisor to produce a new supervisor.
 *
 * @tsplus static effect/core/io/Supervisor.Ops applyPatch
 */

function applyPatch_1(self, supervisor) {
  /**
   * @tsplus tailRec
   */
  function loop(supervisor, patches) {
    if (tsplus_module_1.isCons(patches)) {
      switch (patches.head._tag) {
        case "AddSupervisor":
          {
            return loop(supervisor.zip(patches.head.supervisor), patches.tail);
          }

        case "RemoveSupervisor":
          {
            return loop(removeSupervisor(supervisor, patches.head.supervisor), patches.tail);
          }

        case "AndThen":
          {
            return loop(supervisor, tsplus_module_1.cons(patches.head.first, tsplus_module_1.cons(patches.head.second, patches.tail)));
          }

        case "Empty":
          {
            return loop(supervisor, patches.tail);
          }
      }
    }

    return supervisor;
  }

  return loop(supervisor, tsplus_module_2.make(self));
}

function removeSupervisor(self, that) {
  if (equals(self, that)) {
    return tsplus_module_3.none;
  } else {
    if (self instanceof Zip) {
      return removeSupervisor(self.left, that).zip(removeSupervisor(self.right, that));
    } else {
      return self;
    }
  }
}

function toSet(self) {
  if (equals(self, tsplus_module_3.none)) {
    return tsplus_module_4.empty();
  } else {
    if (self instanceof Zip) {
      return tsplus_module_5.union(toSet(self.right))(toSet(self.left));
    } else {
      return tsplus_module_6.make(self);
    }
  }
}
/**
 * @tsplus static effect/core/io/Supervisor.Ops diff
 */


export function diff(oldValue, newValue) {
  if (equals(oldValue, newValue)) {
    return emptyPatch_1;
  }

  const oldSupervisors = toSet(oldValue);
  const newSupervisors = toSet(newValue);
  const added = tsplus_module_8.reduce(emptyPatch_1, (p, s) => combinePatch_1(p, new AddSupervisor(s)))(tsplus_module_7.difference(oldSupervisors)(newSupervisors));
  const removed = tsplus_module_8.reduce(emptyPatch_1, (p, s) => combinePatch_1(p, new RemoveSupervisor(s)))(tsplus_module_7.difference(newSupervisors)(oldSupervisors));
  return combinePatch_1(added, removed);
}
/**
 * @tsplus static effect/core/io/Supervisor.Ops differ
 */

export const differ = /*#__PURE__*/tsplus_module_9.make({
  empty: emptyPatch_1,
  patch: applyPatch_1,
  combine: combinePatch_1,
  diff
});
//# sourceMappingURL=patch.mjs.map