import * as tsplus_module_1 from "@effect/core/stm/STM/definition/primitives";
import * as tsplus_module_2 from "@effect/core/stm/STM/operations/suspend";
/**
 * inspired by https://github.com/tusharmath/qio/pull/22 (revised)
 */

export class GenSTM {
  constructor(stm) {
    this.stm = stm;
  }

  *[Symbol.iterator]() {
    return yield this;
  }

}

const adapter = (_, __) => {
  return new GenSTM(_);
};
/**
 * Do simulation using Generators
 *
 * @tsplus static effect/core/stm/STM.Ops gen
 */


export function gen(f) {
  return tsplus_module_2.suspend(() => {
    const iterator = f(adapter);
    const state = iterator.next();

    function run(state) {
      if (state.done) {
        return tsplus_module_1.succeed(state.value);
      }

      return tsplus_module_1.flatMap(val => {
        const next = iterator.next(val);
        return run(next);
      })(state.value["stm"]);
    }

    return run(state);
  });
}
//# sourceMappingURL=gen.mjs.map