import * as tsplus_module_1 from "@tsplus/stdlib/collections/Chunk/definition";
import * as tsplus_module_2 from "@effect/core/stm/STM/definition/primitives";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/mutable/ListBuffer";
import * as tsplus_module_4 from "@effect/core/stm/STM/operations/suspend";
/**
 * Loops with the specified transactional function, collecting the results
 * into a list. The moral equivalent of:
 *
 * ```typescript
 * let s  = initial
 * let as = [] as readonly A[]
 *
 * while (cont(s)) {
 *   as = [body(s), ...as]
 *   s  = inc(s)
 * }
 *
 * A.reverse(as)
 * ```
 *
 * @tsplus static effect/core/stm/STM.Ops loop
 */

export function loop(initial, cont, inc) {
  return body => {
    return tsplus_module_2.map(list => tsplus_module_1.from(list))(loopInternal(initial, cont, inc, body));
  };
}

function loopInternal(initial, cont, inc, body) {
  return tsplus_module_4.suspend(() => {
    return cont(initial) ? tsplus_module_2.flatMap(a => tsplus_module_2.map(as => {
      as.prepend(a);
      return as;
    })(loopInternal(inc(initial), cont, inc, body)))(body(initial)) : tsplus_module_2.succeed(tsplus_module_3.ListBuffer.empty());
  });
}
//# sourceMappingURL=loop.mjs.map