import * as tsplus_module_1 from "@effect/core/stm/STM/definition/primitives";
export const repeatWhile = repeatWhile_1;
/**
 * Repeats this `STM` effect while its result satisfies the specified
 * predicate. **WARNING**: `repeatWhile` uses a busy loop to repeat the
 * effect and will consume a thread until it completes (it cannot yield). This
 * is because STM describes a single atomic transaction which must either
 * complete, retry or fail a transaction before yielding back to the ZIO
 * Runtime.
 *   - Use `retryWhile` instead if you don't need to maintain transaction
 *     state for repeats.
 *   - Ensure repeating the STM effect will eventually not satisfy the
 *     predicate.
 *   - Consider using the Blocking thread pool for execution of the
 *     transaction.
 *
 * @tsplus static effect/core/stm/STM.Aspects repeatWhile
 * @tsplus pipeable effect/core/stm/STM repeatWhile
 */

function repeatWhile_1(f) {
  return self => tsplus_module_1.flatMap(a => f(a) ? repeatWhile_1(f)(self) : tsplus_module_1.succeed(a))(self);
}
//# sourceMappingURL=repeatWhile.mjs.map