import * as tsplus_module_1 from "@effect/core/stm/TRef/operations/unsafeGet";
import * as tsplus_module_2 from "@effect/core/stm/THub/operations/THub/capacity";
import * as tsplus_module_3 from "@effect/core/stm/TRef/operations/unsafeMake";
import * as tsplus_module_4 from "@effect/core/stm/THub/definition";
import * as tsplus_module_5 from "@effect/core/stm/THub/operations/_internal/InternalNode";
import * as tsplus_module_6 from "@effect/core/stm/TRef/operations/unsafeSet";
import * as tsplus_module_7 from "@effect/core/stm/THub/definition/THub";
import * as tsplus_module_8 from "@effect/core/stm/STM/operations/effect";
import { STMInterruptException, STMRetryException } from "@effect/core/stm/STM";
import { concreteTHub } from "@effect/core/stm/THub/operations/_internal/InternalTHub";
/**
 * Publishes a message to the hub, returning whether the message was published
 * to the hub.
 *
 * @tsplus static effect/core/stm/THub.Aspects publish
 * @tsplus pipeable effect/core/stm/THub publish
 */

export function publish(value) {
  return self => {
    void 0;
    return tsplus_module_8.effect((journal, fiberId) => {
      const currentPublisherTail = tsplus_module_1.unsafeGet(journal)(self.publisherTail);

      if (currentPublisherTail == null) {
        throw new STMInterruptException(fiberId);
      }

      const currentSubscriberCount = tsplus_module_1.unsafeGet(journal)(self.subscriberCount);

      if (currentSubscriberCount === 0) {
        return true;
      }

      const currentHubSize = tsplus_module_1.unsafeGet(journal)(self.hubSize);

      if (currentHubSize < tsplus_module_2.capacity(self)) {
        const updatedPublisherTail = tsplus_module_3.unsafeMake(undefined);
        const updatedNode = tsplus_module_5.make(value, currentSubscriberCount, updatedPublisherTail);
        tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherTail);
        tsplus_module_6.unsafeSet(updatedPublisherTail, journal)(self.publisherTail);
        tsplus_module_6.unsafeSet(currentHubSize + 1, journal)(self.hubSize);
        return true;
      }

      switch (self.strategy) {
        case tsplus_module_7.BackPressure:
          throw new STMRetryException();

        case tsplus_module_7.Dropping:
          return false;

        case tsplus_module_7.Sliding:
          {
            if (tsplus_module_2.capacity(self) > 0) {
              let currentPublisherHead = tsplus_module_1.unsafeGet(journal)(self.publisherHead);
              let loop = true;

              while (loop) {
                const node = tsplus_module_1.unsafeGet(journal)(currentPublisherHead);

                if (node == null) {
                  throw new STMRetryException();
                } else {
                  const head = node.head;
                  const tail = node.tail;

                  if (head != null) {
                    const updatedNode = tsplus_module_5.make(undefined, node.subscribers, node.tail);
                    tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherHead);
                    tsplus_module_6.unsafeSet(tail, journal)(self.publisherHead);
                    loop = false;
                  } else {
                    currentPublisherHead = tail;
                  }
                }
              }

              const updatedPublisherTail = tsplus_module_3.unsafeMake(undefined);
              const updatedNode = tsplus_module_5.make(value, currentSubscriberCount, updatedPublisherTail);
              tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherTail);
              tsplus_module_6.unsafeSet(updatedPublisherTail, journal)(self.publisherTail);
            }

            return true;
          }
      }

      return false;
    });
  };
}
//# sourceMappingURL=publish.mjs.map