import * as tsplus_module_1 from "@tsplus/stdlib/data/Either/definition";
import * as tsplus_module_2 from "@effect/core/io/Cause/operations/map";
import * as tsplus_module_3 from "@effect/core/io/Exit/operations/failCause";
import * as tsplus_module_4 from "@effect/core/io/Exit/operations/succeed";
import * as tsplus_module_5 from "@effect/core/io/Exit/operations/fail";
import * as tsplus_module_6 from "@effect/core/io/Cause/definition";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/fiberId";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/flatten";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/ImmutableQueue/dequeue";
import * as tsplus_module_12 from "@tsplus/stdlib/collections/ImmutableQueue/size";
import * as tsplus_module_13 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_14 from "@effect/core/io/Deferred/operations/make";
import * as tsplus_module_15 from "@effect/core/io/Effect/operations/excl-forEach";
import * as tsplus_module_16 from "@tsplus/stdlib/data/Either/fold";
import * as tsplus_module_17 from "@effect/core/io/Effect/operations/foldCause";
import * as tsplus_module_18 from "@tsplus/stdlib/collections/ImmutableQueue/append";
import * as tsplus_module_19 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_20 from "@effect/core/io/Effect/operations/zipRight";
import * as tsplus_module_21 from "@tsplus/stdlib/collections/ImmutableQueue/single";
import * as tsplus_module_22 from "@effect/core/io/Ref/operations/make";
import * as tsplus_module_23 from "@effect/core/io/Effect/operations/map";
export const DoneTypeId = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Producer/Done");
export class StateDone {
  constructor(a) {
    this.a = a;
    this._typeId = DoneTypeId;
  }

}
export const ErrorTypeId = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Producer/Error");
export class StateError {
  constructor(cause) {
    this.cause = cause;
    this._typeId = ErrorTypeId;
  }

}
export const EmptyTypeId = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Producer/Empty");
export class StateEmpty {
  constructor(notifyProducer) {
    this.notifyProducer = notifyProducer;
    this._typeId = EmptyTypeId;
  }

}
export const EmitTypeId = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Producer/Emit");
export class StateEmit {
  constructor(notifyConsumers) {
    this.notifyConsumers = notifyConsumers;
    this._typeId = EmitTypeId;
  }

}
/**
 * An MVar-like abstraction for sending data to channels asynchronously.
 * Designed for one producer and multiple consumers.
 *
 * Features the following semantics:
 *   - Buffer of size 1.
 *   - When emitting, the producer waits for a consumer to pick up the value to
 *     prevent "reading ahead" too much.
 *   - Once an emitted element is read by a consumer, it is cleared from the
 *     buffer, so that at most one consumer sees every emitted element.
 *   - When sending a done or error signal, the producer does not wait for a
 *     consumer to pick up the signal. The signal stays in the buffer after
 *     being read by a consumer, so it can be propagated to multiple consumers.
 *   - Trying to publish another emit/error/done after an error/done have
 *     already been published results in an interruption.
 *
 * @tsplus type effect/core/stream/Channel/SingleProducerAsyncInput
 * @tsplus companion effect/core/stream/Channel/SingleProducerAsyncInput.Ops
 */

export class SingleProducerAsyncInput {
  constructor(ref) {
    this.ref = ref;
  }

  get take() {
    return this.takeWith(cause => tsplus_module_3.failCause(tsplus_module_2.map(tsplus_module_1.left)(cause)), element => tsplus_module_4.succeed(element), done => tsplus_module_5.fail(tsplus_module_1.right(done)));
  }

  get close() {
    return tsplus_module_8.flatMap(fiberId => this.error(tsplus_module_6.interrupt(fiberId)))(tsplus_module_7.fiberId);
  }

  get awaitRead() {
    return tsplus_module_10.flatten(this.ref.modify(state => state._typeId === EmptyTypeId ? [state.notifyProducer.await, state] : [tsplus_module_9.unit, state]));
  }

  emit(el) {
    return tsplus_module_8.flatMap(deferred => tsplus_module_10.flatten(this.ref.modify(state => {
      switch (state._typeId) {
        case EmitTypeId:
          {
            const dequeued = tsplus_module_11.dequeue(state.notifyConsumers);

            if (dequeued._tag === "Some") {
              const [notifyConsumer, notifyConsumers] = dequeued.value;
              return [notifyConsumer.succeed(tsplus_module_1.right(el)), tsplus_module_12.size(notifyConsumers) === 0 ? new StateEmpty(deferred) : new StateEmit(notifyConsumers)];
            }

            throw new Error("SingleProducerAsyncInput#emit: queue was empty");
          }

        case ErrorTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case DoneTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case EmptyTypeId:
          {
            return [state.notifyProducer.await, state];
          }
      }
    })))(tsplus_module_14.make());
  }

  done(a) {
    return tsplus_module_10.flatten(this.ref.modify(state => {
      switch (state._typeId) {
        case EmitTypeId:
          {
            return [tsplus_module_15.forEachDiscard(state.notifyConsumers, promise => promise.succeed(tsplus_module_1.left(a))), new StateDone(a)];
          }

        case ErrorTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case DoneTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case EmptyTypeId:
          {
            return [state.notifyProducer.await, state];
          }
      }
    }));
  }

  error(cause) {
    return tsplus_module_10.flatten(this.ref.modify(state => {
      switch (state._typeId) {
        case EmitTypeId:
          {
            return [tsplus_module_15.forEachDiscard(state.notifyConsumers, promise => promise.failCause(() => cause)), new StateError(cause)];
          }

        case ErrorTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case DoneTypeId:
          {
            return [tsplus_module_13.interrupt, state];
          }

        case EmptyTypeId:
          {
            return [state.notifyProducer.await, state];
          }
      }
    }));
  }

  takeWith(onError, onElement, onDone) {
    return tsplus_module_8.flatMap(deferred => tsplus_module_10.flatten(this.ref.modify(state => {
      switch (state._typeId) {
        case EmitTypeId:
          {
            return [tsplus_module_17.foldCause(onError, either => tsplus_module_16.fold(onDone, onElement)(either))(deferred.await), new StateEmit(tsplus_module_18.append(deferred)(state.notifyConsumers))];
          }

        case ErrorTypeId:
          {
            return [tsplus_module_19.sync(() => onError(state.cause)), state];
          }

        case DoneTypeId:
          {
            return [tsplus_module_19.sync(() => onDone(state.a)), state];
          }

        case EmptyTypeId:
          {
            return [tsplus_module_20.zipRight(tsplus_module_17.foldCause(onError, either => tsplus_module_16.fold(onDone, onElement)(either))(deferred.await))(state.notifyProducer.succeed(undefined)), new StateEmit(tsplus_module_21.single(deferred))];
          }
      }
    })))(tsplus_module_14.make());
  }

}
/**
 * Creates a `SingleProducerAsyncInput`.
 *
 * @tsplus static effect/core/stream/Channel/SingleProducerAsyncInput.Ops make
 */

export function make() {
  return tsplus_module_23.map(ref => new SingleProducerAsyncInput(ref))(tsplus_module_8.flatMap(deferred => tsplus_module_22.makeRef(() => new StateEmpty(deferred)))(tsplus_module_14.make()));
}
//# sourceMappingURL=SingleProducerAsyncInput.mjs.map