var _a, _b, _c, _d;

import * as tsplus_module_1 from "@tsplus/stdlib/collections/Collection/functions";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/exit";
import * as tsplus_module_3 from "@effect/core/io/Exit/operations/zipRight";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/zipWith";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/done";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/ImmutableQueue/append";
export const SubexecutorSym = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Subexecutor");
export const Subexecutor = {};
/**
 * Execute upstreamExecutor and for each emitted element, spawn a child
 * channel and continue with processing it by `PullFromChild`.
 */

export class PullFromUpstream {
  constructor(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit) {
    this.upstreamExecutor = upstreamExecutor;
    this.createChild = createChild;
    this.lastDone = lastDone;
    this.activeChildExecutors = activeChildExecutors;
    this.combineChildResults = combineChildResults;
    this.combineWithChildResult = combineWithChildResult;
    this.onPull = onPull;
    this.onEmit = onEmit;
    this._tag = "PullFromUpstream";
    this[_a] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.upstreamExecutor.close(exit);
    const fins = tsplus_module_1.append(fin1)(tsplus_module_1.map(child => child != null ? child.childExecutor.close(exit) : undefined)(this.activeChildExecutors));
    const result = tsplus_module_1.reduce(undefined, (acc, next) => {
      if (acc != null && next != null) {
        return tsplus_module_4.zipWith(tsplus_module_2.exit(next), (a, b) => tsplus_module_3.zipRight(b)(a))(acc);
      } else if (acc != null) {
        return acc;
      } else if (next != null) {
        return tsplus_module_2.exit(next);
      } else {
        return undefined;
      }
    })(fins);
    return result == null ? result : tsplus_module_6.flatMap(exit => tsplus_module_5.done(exit))(result);
  }

  enqueuePullFromChild(child) {
    return new PullFromUpstream(this.upstreamExecutor, this.createChild, this.lastDone, tsplus_module_7.append(child)(this.activeChildExecutors), this.combineChildResults, this.combineWithChildResult, this.onPull, this.onEmit);
  }

}
_a = SubexecutorSym;
/**
 * Execute the childExecutor and on each emitted value, decide what to do by
 * `onEmit`.
 */

export class PullFromChild {
  constructor(childExecutor, parentSubexecutor, onEmit) {
    this.childExecutor = childExecutor;
    this.parentSubexecutor = parentSubexecutor;
    this.onEmit = onEmit;
    this._tag = "PullFromChild";
    this[_b] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.childExecutor.close(exit);
    const fin2 = this.parentSubexecutor.close(exit);

    if (fin1 != null && fin2 != null) {
      return tsplus_module_6.flatMap(exit => tsplus_module_5.done(exit))(tsplus_module_4.zipWith(tsplus_module_2.exit(fin2), (a, b) => tsplus_module_3.zipRight(b)(a))(tsplus_module_2.exit(fin1)));
    } else if (fin1 != null) {
      return fin1;
    } else if (fin2 != null) {
      return fin2;
    } else {
      return undefined;
    }
  }

  enqueuePullFromChild(_) {
    return this;
  }

}
_b = SubexecutorSym;
export class DrainChildExecutors {
  constructor(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull) {
    this.upstreamExecutor = upstreamExecutor;
    this.lastDone = lastDone;
    this.activeChildExecutors = activeChildExecutors;
    this.upstreamDone = upstreamDone;
    this.combineChildResults = combineChildResults;
    this.combineWithChildResult = combineWithChildResult;
    this.onPull = onPull;
    this._tag = "DrainChildExecutors";
    this[_c] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.upstreamExecutor.close(exit);
    const fins = tsplus_module_1.append(fin1)(tsplus_module_1.map(child => child != null ? child.childExecutor.close(exit) : undefined)(this.activeChildExecutors));
    return tsplus_module_1.reduce(undefined, (acc, next) => {
      if (acc != null && next != null) {
        return tsplus_module_4.zipWith(tsplus_module_2.exit(next), (a, b) => tsplus_module_3.zipRight(b)(a))(acc);
      } else if (acc != null) {
        return acc;
      } else if (next != null) {
        return tsplus_module_2.exit(next);
      } else {
        return undefined;
      }
    })(fins);
  }

  enqueuePullFromChild(child) {
    return new DrainChildExecutors(this.upstreamExecutor, this.lastDone, tsplus_module_7.append(child)(this.activeChildExecutors), this.upstreamDone, this.combineChildResults, this.combineWithChildResult, this.onPull);
  }

}
_c = SubexecutorSym;
export class Emit {
  constructor(value, next) {
    this.value = value;
    this.next = next;
    this._tag = "Emit";
    this[_d] = SubexecutorSym;
  }

  close(exit) {
    return this.next.close(exit);
  }

  enqueuePullFromChild(_child) {
    return this;
  }

}
_d = SubexecutorSym;
/**
 * @tsplus macro remove
 */

export function concreteSubexecutor(_) {//
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops PullFromUpstream
 */

export function pullFromUpstream(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit) {
  return new PullFromUpstream(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops PullFromChild
 */

export function pullFromChild(childExecutor, parentSubexecutor, onEmit) {
  return new PullFromChild(childExecutor, parentSubexecutor, onEmit);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops DrainChildExecutors
 */

export function drainChildExecutors(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull) {
  return new DrainChildExecutors(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops Emit
 */

export function emit(value, next) {
  return new Emit(value, next);
}
//# sourceMappingURL=Subexecutor.mjs.map