import { ChannelBase } from "@effect/core/stream/Channel/definition/base";
import { _Env, _InDone, _InElem, _InErr, _OutDone, _OutDone2, _OutElem, _OutErr, _OutErr2 } from "@effect/core/stream/Channel/definition/symbols"; // -----------------------------------------------------------------------------
// PipeTo
// -----------------------------------------------------------------------------

export class PipeTo extends ChannelBase {
  constructor(left, right) {
    super();
    this.left = left;
    this.right = right;
    this._tag = "PipeTo";
  }

} // -----------------------------------------------------------------------------
// Read
// -----------------------------------------------------------------------------

export class Read extends ChannelBase {
  constructor(more, done) {
    super();
    this.more = more;
    this.done = done;
    this._tag = "Read";
  }

} // -----------------------------------------------------------------------------
// SucceedNow
// -----------------------------------------------------------------------------

export class SucceedNow extends ChannelBase {
  constructor(terminal) {
    super();
    this.terminal = terminal;
    this._tag = "SucceedNow";
  }

} // -----------------------------------------------------------------------------
// Fail
// -----------------------------------------------------------------------------

export class Fail extends ChannelBase {
  constructor(error) {
    super();
    this.error = error;
    this._tag = "Fail";
  }

} // -----------------------------------------------------------------------------
// FromEffect
// -----------------------------------------------------------------------------

export class FromEffect extends ChannelBase {
  constructor(effect) {
    super();
    this.effect = effect;
    this._tag = "FromEffect";
  }

} // -----------------------------------------------------------------------------
// Emit
// -----------------------------------------------------------------------------

export class Emit extends ChannelBase {
  constructor(out) {
    super();
    this.out = out;
    this._tag = "Emit";
  }

} // -----------------------------------------------------------------------------
// Succeed
// -----------------------------------------------------------------------------

export class Succeed extends ChannelBase {
  constructor(effect) {
    super();
    this.effect = effect;
    this._tag = "Succeed";
  }

} // -----------------------------------------------------------------------------
// Suspend
// -----------------------------------------------------------------------------

export class Suspend extends ChannelBase {
  constructor(effect) {
    super();
    this.effect = effect;
    this._tag = "Suspend";
  }

} // -----------------------------------------------------------------------------
// Ensuring
// -----------------------------------------------------------------------------

export class Ensuring extends ChannelBase {
  constructor(channel, finalizer) {
    super();
    this.channel = channel;
    this.finalizer = finalizer;
    this._tag = "Ensuring";
  }

} // -----------------------------------------------------------------------------
// ConcatAll
// -----------------------------------------------------------------------------

export class ConcatAll extends ChannelBase {
  constructor(combineInners, combineAll, onPull, onEmit, value, k) {
    super();
    this.combineInners = combineInners;
    this.combineAll = combineAll;
    this.onPull = onPull;
    this.onEmit = onEmit;
    this.value = value;
    this.k = k;
    this._tag = "ConcatAll";
  }

} // -----------------------------------------------------------------------------
// Fold
// -----------------------------------------------------------------------------

export class Fold extends ChannelBase {
  constructor(value, k) {
    super();
    this.value = value;
    this.k = k;
    this._tag = "Fold";
  }

} // -----------------------------------------------------------------------------
// Bridge
// -----------------------------------------------------------------------------

export class Bridge extends ChannelBase {
  constructor(input, channel) {
    super();
    this.input = input;
    this.channel = channel;
    this._tag = "Bridge";
  }

} // -----------------------------------------------------------------------------
// BracketOut
// -----------------------------------------------------------------------------

export class BracketOut extends ChannelBase {
  constructor(acquire, finalizer) {
    super();
    this.acquire = acquire;
    this.finalizer = finalizer;
    this._tag = "BracketOut";
  }

} // -----------------------------------------------------------------------------
// Provide
// -----------------------------------------------------------------------------

export class Provide extends ChannelBase {
  constructor(env, channel) {
    super();
    this.env = env;
    this.channel = channel;
    this._tag = "Provide";
  }

}
/**
 * @tsplus macro remove
 */

export function concrete(_) {//
} // -----------------------------------------------------------------------------
// Continuation
// -----------------------------------------------------------------------------

export class Continuation {}
/**
 * @tsplus macro remove
 */

export function concreteContinuation(_) {//
}
export class ContinuationK extends Continuation {
  constructor(onSuccess, onHalt) {
    super();
    this.onSuccess = onSuccess;
    this.onHalt = onHalt;
    this._tag = "ContinuationK";
  }

  onExit(exit) {
    switch (exit._tag) {
      case "Failure":
        {
          return this.onHalt(exit.cause);
        }

      case "Success":
        {
          return this.onSuccess(exit.value);
        }
    }
  }

}
export class ContinuationFinalizer extends Continuation {
  constructor(finalizer) {
    super();
    this.finalizer = finalizer;
    this._tag = "ContinuationFinalizer";
  }

}
//# sourceMappingURL=primitives.mjs.map