import * as tsplus_module_1 from "@effect/core/stream/Sink/operations/suspend";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/isNonEmpty";
import * as tsplus_module_3 from "@effect/core/stream/Channel/operations/succeed";
import * as tsplus_module_4 from "@effect/core/stream/Channel/operations/write";
import * as tsplus_module_5 from "@effect/core/stream/Channel/operations/flatMap";
import * as tsplus_module_6 from "@effect/core/stream/Channel/operations/fromEffect";
import * as tsplus_module_7 from "@effect/core/stream/Channel/operations/fail";
import * as tsplus_module_8 from "@effect/core/stream/Channel/operations/readWith";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/Chunk/operations/unsafeGet";
import * as tsplus_module_12 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_13 from "@tsplus/stdlib/collections/Chunk/operations/drop";
import * as tsplus_module_14 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_15 from "@tsplus/stdlib/collections/Chunk/operations/concat";
import { SinkInternal } from "@effect/core/stream/Sink/operations/_internal/SinkInternal";
/**
 * Creates a sink that effectfully folds elements of type `In` into a
 * structure of type `S`, until `max` worth of elements (determined by the
 * `costFn`) have been folded.
 *
 * The `decompose` function will be used for decomposing elements that cause
 * an `S` aggregate to cross `max` into smaller elements. Be vigilant with
 * this function, it has to generate "simpler" values or the fold may never
 * end. A value is considered indivisible if `decompose` yields the empty
 * chunk or a single-valued chunk. In these cases, there is no other choice
 * than to yield a value that will cross the threshold.
 *
 * @tsplus static effect/core/stream/Sink.Ops foldWeightedDecomposeEffect
 */

export function foldWeightedDecomposeEffect(z, costFn, max, decompose, f) {
  return tsplus_module_1.suspend(() => new SinkInternal(go(z, costFn, max, decompose, f, false, 0)));
}

function go(s, costFn, max, decompose, f, dirty, cost) {
  return tsplus_module_8.readWith(chunk => tsplus_module_5.flatMap(([nextS, nextCost, nextDirty, leftovers]) => tsplus_module_2.isNonEmpty(leftovers) ? tsplus_module_5.flatMap(() => tsplus_module_3.succeed(nextS))(tsplus_module_4.write(leftovers)) : cost > max ? tsplus_module_3.succeed(nextS) : go(nextS, costFn, max, decompose, f, nextDirty, nextCost))(tsplus_module_6.fromEffect(fold(chunk, s, costFn, max, decompose, f, dirty, cost, 0))), err => tsplus_module_7.fail(err), () => tsplus_module_3.succeed(s));
}

function fold(input, s, costFn, max, decompose, f, dirty, cost, index) {
  if (index === input.length) {
    return tsplus_module_10.sync(() => [s, cost, dirty, tsplus_module_9.empty()]);
  }

  const elem = tsplus_module_11.unsafeGet(index)(input);
  return tsplus_module_12.flatMap(total => {
    if (total <= max) {
      return tsplus_module_12.flatMap(s => fold(input, s, costFn, max, decompose, f, true, total, index + 1))(f(s, elem));
    }

    return tsplus_module_12.flatMap(decomposed => {
      if (decomposed.length <= 1 && !dirty) {
        // If `elem` cannot be decomposed, we need to cross the `max` threshold. To
        // minimize "injury", we only allow this when we haven't added anything else
        // to the aggregate (dirty = false).
        return tsplus_module_14.map(s => [s, total, true, tsplus_module_13.drop(index + 1)(input)])(f(s, elem));
      }

      if (decomposed.length <= 1 && dirty) {
        // If the state is dirty and `elem` cannot be decomposed, we stop folding
        // and include `elem` in the leftovers.
        return tsplus_module_10.sync(() => [s, cost, dirty, tsplus_module_13.drop(index)(input)]);
      } // `elem` got decomposed, so we will recurse with the decomposed elements pushed
      // into the chunk we're processing and see if we can aggregate further.


      return fold(tsplus_module_15.concat(tsplus_module_13.drop(index + 1)(input))(decomposed), s, costFn, max, decompose, f, dirty, cost, 0);
    })(decompose(elem));
  })(tsplus_module_14.map(addedCost => cost + addedCost)(costFn(s, elem)));
}
//# sourceMappingURL=foldWeightedDecomposeEffect.mjs.map