import * as tsplus_module_1 from "@effect/core/io/Effect/operations/logging";
import * as tsplus_module_2 from "@effect/core/stream/Sink/operations/fromEffect";
import * as tsplus_module_3 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/as";
import * as tsplus_module_5 from "@effect/core/stream/Sink/operations/unwrapScoped";
import * as tsplus_module_6 from "@effect/core/io/LogSpan/definition";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/ImmutableMap/set";
/**
 * Logs the specified message at the current log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops log
 */

export function log(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.log(message));
}
/**
 * Logs the specified message at the debug log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logDebug
 */

export function logDebug(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logDebug(message));
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logError
 */

export function logError(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logError(message));
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logErrorCause
 */

export function logErrorCause(cause) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logErrorCause(cause));
}
/**
 * Logs the specified message at the fatal log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logFatal
 */

export function logFatal(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logFatal(message));
}
/**
 * Logs the specified message at the info log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logInfo
 */

export function logInfo(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logInfo(message));
}
/**
 * Logs the specified message at the trace log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logTrace
 */

export function logTrace(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logTrace(message));
}
/**
 * Logs the specified message at the warning log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logWarning
 */

export function logWarning(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logWarning(message));
}
/**
 * Sets the log level for streams composed after this.
 *
 * @tsplus static effect/core/stream/Sink.Ops logLevel
 */

export function logLevel(level) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_4.as(sink)(tsplus_module_3.currentLogLevel.locallyScoped(level)));
}
/**
 * Adjusts the label for the logging span for streams composed after this.
 *
 * @tsplus static effect/core/stream/Sink.Ops logSpan
 */

export function logSpan(label) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_8.flatMap(stack => {
    const now = Date.now();
    const logSpan = tsplus_module_6.apply(label, now);
    return tsplus_module_4.as(sink)(tsplus_module_3.currentLogSpan.locallyScoped(tsplus_module_7.prepend(logSpan)(stack)));
  })(tsplus_module_3.currentLogSpan.get));
}
/**
 * Annotates each log in streams composed after this with the specified log
 * annotation.
 *
 * @tsplus static effect/core/stream/Sink.Ops logAnnotate
 */

export function logAnnotate(key, value) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_8.flatMap(annotations => tsplus_module_4.as(sink)(tsplus_module_3.currentLogAnnotations.locallyScoped(tsplus_module_9.set(key, value)(annotations))))(tsplus_module_3.currentLogAnnotations.get));
}
/**
 * Retrieves the log annotations associated with the current scope.
 *
 * @tsplus static effect/core/stream/Sink.Ops logAnnotations
 */

export function logAnnotations() {
  return tsplus_module_2.fromEffect(tsplus_module_3.currentLogAnnotations.get);
}
//# sourceMappingURL=logging.mjs.map