import * as tsplus_module_1 from "@effect/core/io/Ref/operations/make";
import * as tsplus_module_2 from "@effect/core/io/Deferred/operations/make";
import * as tsplus_module_3 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_4 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/zipRight";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/flatten";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/as";
import * as tsplus_module_8 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/succeed";
export const offer = offer_1;
export const take = take_1;
/**
 * A synchronous queue-like abstraction that allows a producer to offer an
 * element and wait for it to be taken, and allows a consumer to wait for an
 * element to be available.
 *
 * @tsplus type effect/core/stream/Stream/Handoff
 * @tsplus companion effect/core/stream/Stream/Handoff.Ops
 */

export class Handoff {
  constructor(ref) {
    this.ref = ref;
  }

}
export class Empty {
  constructor(notifyConsumer) {
    this.notifyConsumer = notifyConsumer;
    this._tag = "Empty";
  }

}
export class Full {
  constructor(value, notifyProducer) {
    this.value = value;
    this.notifyProducer = notifyProducer;
    this._tag = "Full";
  }

}
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Ops make
 */

export function make() {
  return tsplus_module_4.map(state => new Handoff(state))(tsplus_module_3.flatMap(deferred => tsplus_module_1.makeRef(() => new Empty(deferred)))(tsplus_module_2.make()));
}
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Aspects offer
 * @tsplus pipeable effect/core/stream/Stream/Handoff offer
 */

function offer_1(value) {
  return self => tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_5.zipRight(deferred.await)(state.notifyConsumer.succeed(undefined)), new Full(value, deferred)];
        }

      case "Full":
        {
          return [tsplus_module_5.zipRight(offer_1(value)(self))(state.notifyProducer.await), state];
        }
    }
  })))(tsplus_module_2.make());
}
/**
 * @tsplus getter effect/core/stream/Stream/Handoff take
 */


function take_1(self) {
  return tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_5.zipRight(take_1(self))(state.notifyConsumer.await), state];
        }

      case "Full":
        {
          return [tsplus_module_7.as(state.value)(state.notifyProducer.succeed(undefined)), new Empty(deferred)];
        }
    }
  })))(tsplus_module_2.make());
}
/**
 * @tsplus getter effect/core/stream/Stream/Handoff poll
 */


export function poll(self) {
  return tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_9.succeed(tsplus_module_8.none), state];
        }

      case "Full":
        {
          return [tsplus_module_7.as(tsplus_module_8.some(state.value))(state.notifyProducer.succeed(undefined)), new Empty(deferred)];
        }
    }
  })))(tsplus_module_2.make());
}
//# sourceMappingURL=Handoff.mjs.map