import * as tsplus_module_1 from "@effect/core/io/Queue/operations/dropping";
import * as tsplus_module_2 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_3 from "@effect/core/stream/Stream/operations/rechunk";
import * as tsplus_module_4 from "@effect/core/stream/Stream/operations/via";
import { bufferSignal } from "@effect/core/stream/Stream/operations/_internal/bufferSignal";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Allows a faster producer to progress independently of a slower consumer by
 * buffering up to `capacity` elements in a dropping queue.
 *
 * This combinator destroys the chunking structure. It's recommended to use
 * rechunk afterwards.
 *
 * Note: prefer capacities that are powers of 2 for better performance.
 *
 * @tsplus static effect/core/stream/Stream.Aspects bufferDropping
 * @tsplus pipeable effect/core/stream/Stream bufferDropping
 */

export function bufferDropping(capacity) {
  return self => {
    const queue = tsplus_module_2.acquireRelease(tsplus_module_1.dropping(capacity), queue => queue.shutdown);
    const stream = tsplus_module_4.via(tsplus_module_3.rechunk(1))(self);
    void 0;
    return new StreamInternal(bufferSignal(queue, stream.channel));
  };
}
//# sourceMappingURL=bufferDropping.mjs.map