import * as tsplus_module_1 from "@effect/core/io/Queue/operations/sliding";
import * as tsplus_module_2 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_3 from "@effect/core/stream/Stream/operations/rechunk";
import { bufferSignal } from "@effect/core/stream/Stream/operations/_internal/bufferSignal";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Allows a faster producer to progress independently of a slower consumer by
 * buffering up to `capacity` elements in a sliding queue.
 *
 * This combinator destroys the chunking structure. It's recommended to use
 * rechunk afterwards.
 *
 * Note: prefer capacities that are powers of 2 for better performance.
 *
 * @tsplus static effect/core/stream/Stream.Aspects bufferSliding
 * @tsplus pipeable effect/core/stream/Stream bufferSliding
 */

export function bufferSliding(capacity) {
  return self => {
    const queue = tsplus_module_2.acquireRelease(tsplus_module_1.sliding(capacity), queue => queue.shutdown);
    const stream = tsplus_module_3.rechunk(1)(self);
    void 0;
    return new StreamInternal(bufferSignal(queue, stream.channel));
  };
}
//# sourceMappingURL=bufferSliding.mjs.map