import * as tsplus_module_1 from "@effect/core/stream/Channel/MergeStrategy";
import * as tsplus_module_2 from "@effect/core/stream/Channel/operations/writeChunk";
import * as tsplus_module_3 from "@effect/core/stream/Channel/operations/concatMap";
import * as tsplus_module_4 from "@effect/core/stream/Channel/operations/mergeMap";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Maps each element of this stream to another stream and returns the
 * non-deterministic merge of those streams, executing up to `n` inner streams
 * concurrently. When a new stream is created from an element of the source
 * stream, the oldest executing stream is cancelled. Up to `bufferSize`
 * elements of the produced streams may be buffered in memory by this
 * operator.
 *
 * @tsplus static effect/core/stream/Stream.Aspects flatMapParSwitch
 * @tsplus pipeable effect/core/stream/Stream flatMapParSwitch
 */

export function flatMapParSwitch(n, f, bufferSize = 16) {
  return self => {
    void 0;
    return new StreamInternal(tsplus_module_4.mergeMap(n, a => {
      const stream = f(a);
      void 0;
      return stream.channel;
    }, bufferSize, tsplus_module_1.BufferSliding)(tsplus_module_3.concatMap(tsplus_module_2.writeChunk)(self.channel)));
  };
}
//# sourceMappingURL=flatMapParSwitch.mjs.map