import * as tsplus_module_1 from "@effect/core/io/Effect/operations/logging";
import * as tsplus_module_2 from "@effect/core/stream/Stream/operations/fromEffect";
import * as tsplus_module_3 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_4 from "@effect/core/stream/Stream/operations/scoped";
import * as tsplus_module_5 from "@effect/core/io/LogSpan/definition";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/flatMap";
/**
 * Logs the specified message at the current log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops log
 */

export function log(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.log(message));
}
/**
 * Logs the specified message at the debug log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logDebug
 */

export function logDebug(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logDebug(message));
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logError
 */

export function logError(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logError(message));
}
/**
 * Logs the specified cause as an error.
 *
 * @tsplus static effect/core/stream/Stream.Ops logErrorCause
 */

export function logErrorCause(cause) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logErrorCause(cause));
}
/**
 * Logs the specified message at the fatal log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logFatal
 */

export function logFatal(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logFatal(message));
}
/**
 * Logs the specified message at the informational log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logInfo
 */

export function logInfo(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logInfo(message));
}
/**
 * Logs the specified message at the warning log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logWarning
 */

export function logWarning(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logWarning(message));
}
/**
 * Logs the specified message at the trace log level.
 *
 * @tsplus static effect/core/stream/Stream.Ops logTrace
 */

export function logTrace(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logTrace(message));
}
/**
 * Sets the log level for streams composed after this.
 *
 * @tsplus static effect/core/stream/Stream.Ops logLevel
 */

export function logLevel(level) {
  return tsplus_module_4.scoped(tsplus_module_3.currentLogLevel.locallyScoped(level));
}
/**
 * Adjusts the label for the logging span for streams composed after this.
 *
 * @tsplus static effect/core/stream/Stream.Ops logSpan
 */

export function logSpan(label) {
  return tsplus_module_4.scoped(tsplus_module_7.flatMap(stack => {
    const now = Date.now();
    const logSpan = tsplus_module_5.apply(label, now);
    return tsplus_module_3.currentLogSpan.locallyScoped(tsplus_module_6.prepend(logSpan)(stack));
  })(tsplus_module_3.currentLogSpan.get));
}
//# sourceMappingURL=logging.mjs.map