import * as tsplus_module_1 from "@effect/core/stream/Channel/operations/pipeTo";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/size";
import * as tsplus_module_3 from "@effect/core/stream/Channel/operations/write";
import * as tsplus_module_4 from "@effect/core/stream/Channel/operations/flatMap";
import * as tsplus_module_5 from "@tsplus/stdlib/collections/List/empty";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Chunk/operations/unsafeGet";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/List/reverse";
import * as tsplus_module_9 from "@effect/core/stream/Channel/operations/writeAll";
import * as tsplus_module_10 from "@effect/core/stream/Channel/operations/failCause";
import * as tsplus_module_11 from "@effect/core/stream/Channel/operations/readWithCause";
import * as tsplus_module_12 from "@tsplus/stdlib/collections/Chunk/operations/builder";
import * as tsplus_module_13 from "@effect/core/stream/Channel/operations/unit";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Re-chunks the elements of the stream into chunks of `n` elements each. The
 * last chunk might contain less than `n` elements.
 *
 * @tsplus static effect/core/stream/Stream.Aspects rechunk
 * @tsplus pipeable effect/core/stream/Stream rechunk
 */

export function rechunk(n) {
  return self => {
    void 0;
    return new StreamInternal(tsplus_module_1.pipeTo(process(new Rechunker(n), n))(self.channel));
  };
}

function process(rechunker, target) {
  return tsplus_module_11.readWithCause(chunk => {
    if (tsplus_module_2.size(chunk) === target && rechunker.isEmpty()) {
      return tsplus_module_4.flatMap(() => process(rechunker, target))(tsplus_module_3.write(chunk));
    }

    if (tsplus_module_2.size(chunk) > 0) {
      let chunks = tsplus_module_5.empty();
      let result = undefined;
      let i = 0;

      while (i < tsplus_module_2.size(chunk)) {
        while (i < tsplus_module_2.size(chunk) && result == null) {
          result = rechunker.write(tsplus_module_6.unsafeGet(i)(chunk));
          i = i + 1;
        }

        if (result != null) {
          chunks = tsplus_module_7.prepend(result)(chunks);
          result = undefined;
        }
      }

      return tsplus_module_4.flatMap(() => process(rechunker, target))(tsplus_module_9.writeAll(...tsplus_module_8.reverse(chunks)));
    }

    return process(rechunker, target);
  }, cause => tsplus_module_4.flatMap(() => tsplus_module_10.failCause(cause))(rechunker.emitIfNotEmpty()), () => rechunker.emitIfNotEmpty());
}

class Rechunker {
  constructor(n) {
    this.n = n;
    this.builder = tsplus_module_12.builder();
    this.pos = 0;
  }

  isEmpty() {
    return this.pos === 0;
  }

  write(elem) {
    this.builder.append(elem);
    this.pos += 1;

    if (this.pos === this.n) {
      const result = this.builder.build();
      this.builder = tsplus_module_12.builder();
      this.pos = 0;
      return result;
    }

    return undefined;
  }

  emitIfNotEmpty() {
    if (this.pos !== 0) {
      return tsplus_module_3.write(this.builder.build());
    } else {
      return tsplus_module_13.unit;
    }
  }

}
//# sourceMappingURL=rechunk.mjs.map