import * as tsplus_module_1 from "@effect/core/io/Cause/errors";
import * as tsplus_module_2 from "@effect/core/stream/Stream/operations/dieSync";
import * as tsplus_module_3 from "@effect/core/stream/Channel/operations/pipeTo";
import * as tsplus_module_4 from "@effect/core/stream/Stream/operations/sync";
import * as tsplus_module_5 from "@effect/core/stream/Stream/operations/flatMap";
import * as tsplus_module_6 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/Chunk/operations/zipWithIndex";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/Chunk/operations/collect";
import * as tsplus_module_9 from "@effect/core/stream/Channel/operations/write";
import * as tsplus_module_10 from "@effect/core/stream/Channel/operations/flatMap";
import * as tsplus_module_11 from "@effect/core/stream/Channel/operations/failCause";
import * as tsplus_module_12 from "@effect/core/stream/Channel/operations/unit";
import * as tsplus_module_13 from "@effect/core/stream/Channel/operations/readWithCause";
import * as tsplus_module_14 from "@tsplus/stdlib/collections/Chunk/operations/isEmpty";
import * as tsplus_module_15 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_16 from "@tsplus/stdlib/collections/Chunk/operations/single";
import * as tsplus_module_17 from "@tsplus/stdlib/collections/Chunk/operations/takeRight";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
import { RingBufferNew } from "@effect/core/support/RingBufferNew";
/**
 * Emits a sliding window of `n` elements.
 *
 * @tsplus static effect/core/stream/Stream.Aspects sliding
 * @tsplus pipeable effect/core/stream/Stream sliding
 */

export function sliding(chunkSize, stepSize = 1) {
  return self => {
    if (chunkSize <= 0 || stepSize <= 0) {
      return tsplus_module_2.dieSync(() => new tsplus_module_1.IllegalArgumentException("Invalid bounds - `chunkSize` and `stepSize` must be greater than 0"));
    }

    return tsplus_module_5.flatMap(queue => {
      void 0;
      return new StreamInternal(tsplus_module_3.pipeTo(reader(chunkSize, stepSize, queue, 0))(self.channel));
    })(tsplus_module_4.sync(() => new RingBufferNew(chunkSize)));
  };
}

function reader(chunkSize, stepSize, queue, queueSize) {
  return tsplus_module_13.readWithCause(input => tsplus_module_10.flatMap(() => reader(chunkSize, stepSize, queue, queueSize + input.length))(tsplus_module_9.write(tsplus_module_8.collect(([a, index]) => {
    queue.put(a);
    const currentIndex = queueSize + index + 1;
    return currentIndex < chunkSize || (currentIndex - chunkSize) % stepSize > 0 ? tsplus_module_6.none : tsplus_module_6.some(queue.toChunk());
  })(tsplus_module_7.zipWithIndex(input)))), cause => emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, tsplus_module_11.failCause(cause)), () => emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, tsplus_module_12.unit));
}

function emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, channelEnd) {
  if (queueSize < chunkSize) {
    const items = queue.toChunk();
    const result = tsplus_module_14.isEmpty(items) ? tsplus_module_15.empty() : tsplus_module_16.single(items);
    return tsplus_module_10.flatMap(() => channelEnd)(tsplus_module_9.write(result));
  }

  const lastEmitIndex = queueSize - (queueSize - chunkSize) % stepSize;

  if (lastEmitIndex === queueSize) {
    return channelEnd;
  }

  const leftovers = queueSize - (lastEmitIndex - chunkSize + stepSize);
  const lastItems = tsplus_module_17.takeRight(leftovers)(queue.toChunk());
  const result = tsplus_module_14.isEmpty(lastItems) ? tsplus_module_15.empty() : tsplus_module_16.single(lastItems);
  return tsplus_module_10.flatMap(() => channelEnd)(tsplus_module_9.write(result));
}
//# sourceMappingURL=sliding.mjs.map