import * as tsplus_module_1 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_2 from "@effect/core/stream/Channel/operations/pipeTo";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/Chunk/operations/builder";
import * as tsplus_module_4 from "@tsplus/stdlib/collections/Chunk/operations/empty";
import * as tsplus_module_5 from "@tsplus/stdlib/data/Maybe/getOrElse";
import * as tsplus_module_6 from "@tsplus/stdlib/collections/Chunk/operations/append";
import * as tsplus_module_7 from "@tsplus/stdlib/collections/Chunk/operations/unsafeGet";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/Chunk/operations/take";
import * as tsplus_module_9 from "@tsplus/stdlib/collections/Chunk/operations/unsafeHead";
import * as tsplus_module_10 from "@tsplus/stdlib/collections/Chunk/operations/reduce";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/Chunk/operations/isNonEmpty";
import * as tsplus_module_12 from "@effect/core/stream/Channel/operations/writeChunk";
import * as tsplus_module_13 from "@effect/core/stream/Channel/operations/flatMap";
import * as tsplus_module_14 from "@effect/core/stream/Channel/operations/failCause";
import * as tsplus_module_15 from "@effect/core/stream/Channel/operations/write";
import * as tsplus_module_16 from "@tsplus/stdlib/data/Maybe/fold";
import * as tsplus_module_17 from "@effect/core/stream/Channel/operations/succeed";
import * as tsplus_module_18 from "@effect/core/stream/Channel/operations/readWithCause";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Splits elements on a delimiter and transforms the splits into desired
 * output, flattening the resulting chunks into the stream.
 *
 * @tsplus static effect/core/stream/Stream.Aspects splitOnChunkFlatten
 * @tsplus pipeable effect/core/stream/Stream splitOnChunkFlatten
 */

export function splitOnChunkFlatten(delimiter) {
  return self => {
    void 0;
    return new StreamInternal(tsplus_module_2.pipeTo(next(delimiter, tsplus_module_1.none, 0))(self.channel));
  };
}

function next(delimiter, leftover, delimiterIndex) {
  return tsplus_module_18.readWithCause(inputChunk => {
    const buffer = tsplus_module_3.builder();
    const [carry, delimiterCursor] = tsplus_module_10.reduce([tsplus_module_5.getOrElse_(leftover, () => tsplus_module_4.empty()), delimiterIndex], ([carry, delimiterCursor], a) => {
      const concatenated = tsplus_module_6.append(a)(carry);

      if (delimiterCursor < delimiter.length && a === tsplus_module_7.unsafeGet(delimiterCursor)(delimiter)) {
        if (delimiterCursor + 1 === delimiter.length) {
          buffer.append(tsplus_module_8.take(concatenated.length - delimiter.length)(concatenated));
          return [tsplus_module_4.empty(), 0];
        }

        return [concatenated, delimiterCursor + 1];
      }

      return [concatenated, a === tsplus_module_9.unsafeHead(delimiter) ? 1 : 0];
    })(inputChunk);
    return tsplus_module_13.flatMap(() => next(delimiter, tsplus_module_11.isNonEmpty(carry) ? tsplus_module_1.some(carry) : tsplus_module_1.none, delimiterCursor))(tsplus_module_12.writeChunk(buffer.build()));
  }, cause => tsplus_module_16.fold(() => tsplus_module_14.failCause(cause), chunk => tsplus_module_13.flatMap(() => tsplus_module_14.failCause(cause))(tsplus_module_15.write(chunk)))(leftover), done => tsplus_module_16.fold(() => tsplus_module_17.succeed(done), chunk => tsplus_module_13.flatMap(() => tsplus_module_17.succeed(done))(tsplus_module_15.write(chunk)))(leftover));
}
//# sourceMappingURL=splitOnChunkFlatten.mjs.map