import * as tsplus_module_1 from "@effect/core/stream/Channel/operations/pipeTo";
import * as tsplus_module_2 from "@effect/core/io/Clock/operations/currentTime";
import * as tsplus_module_3 from "@effect/core/stream/Channel/operations/fromEffect";
import * as tsplus_module_4 from "@effect/core/stream/Channel/operations/flatMap";
import * as tsplus_module_5 from "@effect/core/stream/Channel/operations/write";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/zip";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_8 from "@effect/core/stream/Channel/operations/unwrap";
import * as tsplus_module_9 from "@effect/core/stream/Channel/operations/fail";
import * as tsplus_module_10 from "@effect/core/stream/Channel/operations/unit";
import * as tsplus_module_11 from "@effect/core/stream/Channel/operations/readWith";
import { concreteStream, StreamInternal } from "@effect/core/stream/Stream/operations/_internal/StreamInternal";
/**
 * Throttles the chunks of this stream according to the given bandwidth
 * parameters using the token bucket algorithm. Allows for burst in the
 * processing of elements by allowing the token bucket to accumulate tokens up
 * to a `units + burst` threshold. Chunks that do not meet the bandwidth
 * constraints are dropped. The weight of each chunk is determined by the
 * `costFn` effectful function.
 *
 * @tsplus static effect/core/stream/Stream.Aspects throttleEnforceEffect
 * @tsplus pipeable effect/core/stream/Stream throttleEnforceEffect
 */

export function throttleEnforceEffect(units, duration, costFn, burst = 0) {
  return self => {
    void 0;
    return new StreamInternal(tsplus_module_4.flatMap(timestamp => tsplus_module_1.pipeTo(loop(units, duration, costFn, burst, units, timestamp))(self.channel))(tsplus_module_3.fromEffect(tsplus_module_2.currentTime)));
  };
}

function loop(units, duration, costFn, burst, tokens, timestamp) {
  return tsplus_module_11.readWith(input => tsplus_module_8.unwrap(tsplus_module_7.map(([weight, current]) => {
    const elapsed = current - timestamp;
    const cycles = elapsed / duration.millis;
    const sum = tokens + cycles * units;
    const max = units + burst < 0 ? Number.MAX_SAFE_INTEGER : units + burst;
    const available = sum < 0 ? max : Math.min(sum, max);
    return weight <= available ? tsplus_module_4.flatMap(() => loop(units, duration, costFn, burst, available - weight, current))(tsplus_module_5.write(input)) : loop(units, duration, costFn, burst, available, current);
  })(tsplus_module_6.zip(tsplus_module_2.currentTime)(costFn(input)))), err => tsplus_module_9.fail(err), () => tsplus_module_10.unit);
}
//# sourceMappingURL=throttleEnforceEffect.mjs.map