import * as tsplus_module_1 from "@effect/core/stream/Stream/operations/combineChunks";
import * as tsplus_module_2 from "@tsplus/stdlib/collections/Chunk/operations/isEmpty";
import * as tsplus_module_3 from "@tsplus/stdlib/data/Either/fold";
import * as tsplus_module_4 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_5 from "@effect/core/io/Exit/operations/fail";
import * as tsplus_module_6 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_7 from "@effect/core/io/Exit/operations/succeed";
import * as tsplus_module_8 from "@effect/core/io/Effect/operations/unsome";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/zipPar";
import * as tsplus_module_10 from "@effect/core/io/Effect/operations/foldEffect";

class PullBoth {
  constructor() {
    this._tag = "PullBoth";
  }

}

class PullLeft {
  constructor(rightChunk) {
    this.rightChunk = rightChunk;
    this._tag = "PullLeft";
  }

}

class PullRight {
  constructor(leftChunk) {
    this.leftChunk = leftChunk;
    this._tag = "PullRight";
  }

}
/**
 * Zips this stream with another point-wise and applies the function to the
 * paired elements.
 *
 * The new stream will end when one of the sides ends.
 *
 * @tsplus static effect/core/stream/Stream.Aspects zipWithChunks
 * @tsplus pipeable effect/core/stream/Stream zipWithChunks
 */


export function zipWithChunks(that, f) {
  return self => tsplus_module_1.combineChunks(that, new PullBoth(), pull(f))(self);
}

function zipWithChunksInternal(leftChunk, rightChunk, f) {
  const [out, either] = f(leftChunk, rightChunk);
  return tsplus_module_3.fold(leftChunk => tsplus_module_2.isEmpty(leftChunk) ? [out, new PullBoth()] : [out, new PullRight(leftChunk)], rightChunk => tsplus_module_2.isEmpty(rightChunk) ? [out, new PullBoth()] : [out, new PullLeft(rightChunk)])(either);
}

function pull(f) {
  return (state, pullLeft, pullRight) => {
    switch (state._tag) {
      case "PullBoth":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(tsplus_module_4.some(err))), ([left, right]) => {
            if (tsplus_module_4.isSome(left) && tsplus_module_4.isSome(right)) {
              const leftChunk = left.value;
              const rightChunk = right.value;

              if (tsplus_module_2.isEmpty(leftChunk) && tsplus_module_2.isEmpty(rightChunk)) {
                return pull(f)(new PullBoth(), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(leftChunk)) {
                return pull(f)(new PullLeft(rightChunk), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(rightChunk)) {
                return pull(f)(new PullRight(leftChunk), pullLeft, pullRight);
              } else {
                return tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(leftChunk, rightChunk, f)));
              }
            }

            return tsplus_module_6.succeed(tsplus_module_5.fail(tsplus_module_4.none));
          })(tsplus_module_9.zipPar(tsplus_module_8.unsome(pullRight))(tsplus_module_8.unsome(pullLeft)));
        }

      case "PullLeft":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(err)), leftChunk => tsplus_module_2.isEmpty(leftChunk) ? pull(f)(new PullLeft(state.rightChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.rightChunk) ? pull(f)(new PullRight(leftChunk), pullLeft, pullRight) : tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(leftChunk, state.rightChunk, f))))(pullLeft);
        }

      case "PullRight":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(err)), rightChunk => tsplus_module_2.isEmpty(rightChunk) ? pull(f)(new PullRight(state.leftChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.leftChunk) ? pull(f)(new PullLeft(rightChunk), pullLeft, pullRight) : tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(state.leftChunk, rightChunk, f))))(pullRight);
        }
    }
  };
}
//# sourceMappingURL=zipWithChunks.mjs.map