import * as tsplus_module_1 from "@tsplus/stdlib/data/AtomicReference";
import { DoublyLinkedList } from "@tsplus/stdlib/collections/mutable/DoublyLinkedList";
export class DefaultScheduler {
  constructor() {
    this.running = new tsplus_module_1.AtomicReference(false);
    this.tasks = new tsplus_module_1.AtomicReference(new DoublyLinkedList());
    this.promise = Promise.resolve(void 0);
  }

  starveInternal(depth) {
    const toRun = this.tasks.get;
    this.tasks.set(new DoublyLinkedList());
    toRun.forEach(task => {
      task();
    });

    if (this.tasks.get.isEmpty) {
      this.running.set(false);
    } else {
      this.starve(depth);
    }
  }

  starve(depth = 0) {
    if (depth >= 2048) {
      setTimeout(() => this.starveInternal(0), 0);
    } else {
      this.promise.then(() => this.starveInternal(depth + 1));
    }
  }

  scheduleTask(task) {
    this.tasks.get.add(task);

    if (!this.running.get) {
      this.running.set(true);
      this.starve();
    }
  }

}
export const defaultScheduler = /*#__PURE__*/new DefaultScheduler();
export class StagedScheduler {
  constructor() {
    this.tasks = new DoublyLinkedList();
    this.deferred = new tsplus_module_1.AtomicReference(false);
  }

  scheduleTask(task) {
    if (this.deferred.get) {
      defaultScheduler.scheduleTask(task);
    } else {
      this.tasks.add(task);
    }
  }

  flush() {
    while (!this.tasks.isEmpty) {
      this.tasks.shift()();
    }

    this.deferred.set(true);
  }

}
//# sourceMappingURL=Scheduler.mjs.map