import * as tsplus_module_1 from "@effect/core/io/Effect/operations/map";
import * as tsplus_module_2 from "@effect/core/io/Effect/operations/zipRight";
import * as tsplus_module_3 from "@tsplus/stdlib/collections/List/prepend";
import * as tsplus_module_4 from "@effect/core/testing/TestClock/Data";
import * as tsplus_module_5 from "@effect/core/io/Effect/operations/flatMap";
import * as tsplus_module_6 from "@effect/core/io/Deferred/operations/make";
import * as tsplus_module_7 from "@effect/core/io/Effect/operations/unit";
import * as tsplus_module_8 from "@tsplus/stdlib/collections/List/map";
import * as tsplus_module_9 from "@effect/core/io/Effect/operations/zipParLeft";
import * as tsplus_module_10 from "@effect/core/io/Fiber/definition";
import * as tsplus_module_11 from "@tsplus/stdlib/collections/SortedSet/empty";
import * as tsplus_module_12 from "@effect/core/io/Effect/operations/succeed";
import * as tsplus_module_13 from "@tsplus/stdlib/structure/Equals";
import * as tsplus_module_14 from "@tsplus/stdlib/collections/SortedSet/filter";
import * as tsplus_module_15 from "@tsplus/stdlib/collections/SortedSet/union";
import * as tsplus_module_16 from "@tsplus/stdlib/collections/Chunk/operations/reduce";
import * as tsplus_module_17 from "@effect/core/io/Effect/operations/sync";
import * as tsplus_module_18 from "@effect/core/io/Effect/operations/excl-forEach";
import * as tsplus_module_19 from "@effect/core/testing/TestAnnotation/operations/fibers";
import * as tsplus_module_20 from "@effect/core/io/Effect/operations/descriptorWith";
import * as tsplus_module_21 from "@tsplus/stdlib/collections/HashMap/operations/empty";
import * as tsplus_module_22 from "@tsplus/stdlib/collections/HashMap/operations/set";
import * as tsplus_module_23 from "@effect/core/io/Effect/operations/fail";
import * as tsplus_module_24 from "@effect/core/io/Effect/operations/reduce";
import * as tsplus_module_25 from "@effect/core/testing/TestClock/_internal/WarningData";
import * as tsplus_module_26 from "@tsplus/stdlib/data/Duration";
import * as tsplus_module_27 from "@effect/core/io/Effect/operations/logging";
import * as tsplus_module_28 from "@effect/core/io/Effect/operations/delay";
import * as tsplus_module_29 from "@effect/core/io/Fiber/_internal/runtime";
import * as tsplus_module_30 from "@tsplus/stdlib/data/Maybe/definition";
import * as tsplus_module_31 from "@effect/core/io/Fiber/operations/interrupt";
import * as tsplus_module_32 from "@effect/core/io/Effect/operations/as";
import * as tsplus_module_33 from "@tsplus/stdlib/collections/HashMap/_internal/hashMap";
import * as tsplus_module_34 from "@effect/core/io/Effect/operations/sleep";
import * as tsplus_module_35 from "@effect/core/io/Effect/operations/zip";
import * as tsplus_module_36 from "@tsplus/stdlib/data/Function";
import * as tsplus_module_37 from "@effect/core/io/Effect/operations/zipWith";
import * as tsplus_module_38 from "@effect/core/io/Effect/operations/filterOrFail";
import * as tsplus_module_39 from "@effect/core/io/Effect/operations/eventually";
import * as tsplus_module_40 from "@effect/core/testing/TestClock/_internal/SuspendedWarningData";
import * as tsplus_module_41 from "@effect/core/io/Effect/operations/yieldNow";
import * as tsplus_module_42 from "@tsplus/stdlib/prelude/Ord/common";
import * as tsplus_module_43 from "@tsplus/stdlib/prelude/Ord/operations";
import * as tsplus_module_44 from "@tsplus/stdlib/collections/List/sortWith";
import * as tsplus_module_45 from "@tsplus/stdlib/collections/List/definition";
import { LiveClock } from "@effect/core/io/Clock";
/**
 * The warning message that will be displayed if a test is using time but is
 * not advancing the `TestClock`.
 */

export const warning = "Warning: A test is using time, but is not advancing " + "the test clock, which may result in the test hanging. Use TestClock.adjust to " + "manually advance the time.";
/**
 * The warning message that will be displayed if a test is advancing the clock
 * but a fiber is still running.
 */

export const suspendedWarning = "Warning: A test is advancing the test clock, " + "but a fiber is not suspending, which may result in the test hanging. Use " + "TestAspect.diagnose to identity the fiber that is not suspending.";
export class TestClockInternal extends LiveClock {
  constructor(clockState, live, annotations, warningState, suspendedWarningState) {
    super();
    this.clockState = clockState;
    this.live = live;
    this.annotations = annotations;
    this.warningState = warningState;
    this.suspendedWarningState = suspendedWarningState;
  }
  /**
   * Returns the current clock time in milliseconds.
   */


  get currentTime() {
    return tsplus_module_1.map(data => data.instant)(this.clockState.get);
  }
  /**
   * Saves the `TestClock`'s current state in an effect which, when run, will
   * restore the `TestClock` state to the saved state
   */


  get save() {
    return tsplus_module_1.map(data => this.clockState.set(data))(this.clockState.get);
  }
  /**
   * Sets the current clock time to the specified instant. Any effects that
   * were scheduled to occur on or before the new time will be run in order.
   */


  setTime(instant) {
    return tsplus_module_2.zipRight(this.run(() => instant))(this.warningDone);
  }
  /**
   * Semantically blocks the current fiber until the clock time is equal to or
   * greater than the specified duration. Once the clock time is adjusted to
   * on or after the duration, the fiber will automatically be resumed.
   */


  sleep(duration) {
    return tsplus_module_7.asUnit(tsplus_module_5.flatMap(deferred => tsplus_module_5.flatMap(shouldAwait => shouldAwait ? tsplus_module_2.zipRight(deferred.await)(this.warningStart) : deferred.succeed(undefined))(this.clockState.modify(data => {
      const end = data.instant + duration.millis;

      if (end > data.instant) {
        return [true, tsplus_module_4.Data(data.instant, tsplus_module_3.prepend([end, deferred])(data.sleeps))];
      }

      return [false, data];
    })))(tsplus_module_6.make()));
  }
  /**
   * Returns a list of the times at which all queued effects are scheduled to
   * resume.
   */


  get sleeps() {
    return tsplus_module_1.map(data => tsplus_module_8.map(_ => _[0])(data.sleeps))(this.clockState.get);
  }
  /**
   * Increments the current clock time by the specified duration. Any effects
   * that were scheduled to occur on or before the new time will be run in
   * order.
   */


  adjust(duration) {
    return tsplus_module_2.zipRight(this.run(n => n + duration.millis))(this.warningDone);
  }
  /**
   * Increments the current clock time by the specified duration. Any effects
   * that were scheduled to occur on or before the new time will be run in
   * order.
   */


  adjustWith(duration) {
    return effect => tsplus_module_9.zipParLeft(this.adjust(duration))(effect);
  }
  /**
   * Returns a set of all fibers in this test.
   */


  get supervisedFibers() {
    return tsplus_module_20.descriptorWith(descriptor => tsplus_module_5.flatMap(either => {
      switch (either._tag) {
        case "Left":
          {
            return tsplus_module_12.succeed(tsplus_module_11.empty(tsplus_module_10.ordFiber));
          }

        case "Right":
          {
            return tsplus_module_1.map(set => tsplus_module_14.filter(fiber => !tsplus_module_13.equals(fiber.id, descriptor.id))(set))(tsplus_module_1.map(chunk => tsplus_module_16.reduce(tsplus_module_11.empty(tsplus_module_10.ordFiber), (a, b) => tsplus_module_15.union(b)(a))(chunk))(tsplus_module_18.forEach(either.right, ref => tsplus_module_17.sync(() => ref.get))));
          }
      }
    })(this.annotations.get(tsplus_module_19.fibers)));
  }
  /**
   * Captures a "snapshot" of the identifier and status of all fibers in this
   * test other than the current fiber. Fails with the `Unit` value if any of
   * these fibers are not done or suspended. Note that because we cannot
   * synchronize on the status of multiple fibers at the same time this
   * snapshot may not be fully consistent.
   */


  get freeze() {
    return tsplus_module_5.flatMap(fibers => tsplus_module_24.reduce(fibers, tsplus_module_21.empty(), (map, fiber) => tsplus_module_5.flatMap(status => {
      switch (status._tag) {
        case "Done":
          {
            return tsplus_module_12.succeed(tsplus_module_22.set(fiber.id, status)(map));
          }

        case "Suspended":
          {
            return tsplus_module_12.succeed(tsplus_module_22.set(fiber.id, status)(map));
          }

        default:
          {
            return tsplus_module_23.fail(undefined);
          }
      }
    })(fiber.status)))(this.supervisedFibers);
  }
  /**
   * Forks a fiber that will display a warning message if a test is using time
   * but is not advancing the `TestClock`.
   */


  get warningStart() {
    return this.warningState.updateSomeEffect(data => data._tag === "Start" ? tsplus_module_30.some(tsplus_module_1.map(fiber => tsplus_module_25.pending(fiber))(tsplus_module_29.fork(tsplus_module_29.interruptible(this.live.provide(tsplus_module_28.delay(tsplus_module_26.seconds(5))(tsplus_module_27.logWarning(warning))))))) : tsplus_module_30.none);
  }
  /**
   * Cancels the warning message that is displayed if a test is using time but
   * is not advancing the `TestClock`.
   */


  get warningDone() {
    return this.warningState.updateSomeEffect(warningData => {
      switch (warningData._tag) {
        case "Start":
          {
            return tsplus_module_30.some(tsplus_module_12.succeed(tsplus_module_25.done));
          }

        case "Pending":
          {
            return tsplus_module_30.some(tsplus_module_32.as(tsplus_module_25.done)(tsplus_module_31.interrupt(warningData.fiber)));
          }

        default:
          {
            return tsplus_module_30.none;
          }
      }
    });
  }
  /**
   * Returns whether all descendants of this fiber are done or suspended.
   */


  get suspended() {
    return tsplus_module_5.flatMap(([first, last]) => tsplus_module_33.equals_(first, last) ? tsplus_module_12.succeed(first) : tsplus_module_23.fail(void 0))(tsplus_module_35.zip(tsplus_module_2.zipRight(this.freeze)(this.live.provide(tsplus_module_34.sleep(tsplus_module_26.millis(5)))))(this.freeze));
  }
  /**
   * Polls until all descendants of this fiber are done or suspended.
   */


  get awaitSuspended() {
    return tsplus_module_2.zipRight(this.suspendedWarningDone)(tsplus_module_2.zipRight(tsplus_module_39.eventually(tsplus_module_38.filterOrFail(tsplus_module_36.identity, () => undefined)(tsplus_module_37.zipWith(tsplus_module_2.zipRight(this.suspended)(this.live.provide(tsplus_module_34.sleep(tsplus_module_26.millis(10)))), (a, b) => tsplus_module_33.equals_(a, b))(this.suspended))))(this.suspendedWarningStart));
  }
  /**
   * Forks a fiber that will display a warning message if a test is advancing
   * the `TestClock` but a fiber is not suspending.
   */


  get suspendedWarningStart() {
    return this.suspendedWarningState.updateSomeEffect(suspendedWarningData => suspendedWarningData._tag === "Start" ? tsplus_module_30.some(tsplus_module_1.map(fiber => tsplus_module_40.pending(fiber))(tsplus_module_29.fork(tsplus_module_29.interruptible(this.live.provide(tsplus_module_28.delay(tsplus_module_26.seconds(5))(tsplus_module_2.zipRight(this.suspendedWarningState.set(tsplus_module_40.done))(tsplus_module_27.logWarning(suspendedWarning)))))))) : tsplus_module_30.none);
  }
  /**
   * Cancels the warning message that is displayed if a test is advancing the
   * `TestClock` but a fiber is not suspending.
   */


  get suspendedWarningDone() {
    return this.suspendedWarningState.updateSomeEffect(suspendedWarningData => suspendedWarningData._tag === "Pending" ? tsplus_module_30.some(tsplus_module_32.as(tsplus_module_40.start)(tsplus_module_31.interrupt(suspendedWarningData.fiber))) : tsplus_module_30.none);
  }
  /**
   * Runs all effects scheduled to occur on or before the specified instant,
   * which may depend on the current time, in order.
   */


  run(f) {
    return tsplus_module_2.zipRight(tsplus_module_5.flatMap(maybe => {
      switch (maybe._tag) {
        case "None":
          {
            return tsplus_module_7.unit;
          }

        case "Some":
          {
            const [end, deferred] = maybe.value;
            return tsplus_module_2.zipRight(this.run(() => end))(tsplus_module_2.zipRight(tsplus_module_41.yieldNow)(deferred.succeed(undefined)));
          }
      }
    })(this.clockState.modify(data => {
      const end = f(data.instant);
      const sorted = tsplus_module_44.sortWith(tsplus_module_43.contramap(tsplus_module_42.number, _ => _[0]))(data.sleeps);

      if (tsplus_module_45.isCons(sorted)) {
        const [instant, deferred] = sorted.head;

        if (instant <= end) {
          return [tsplus_module_30.some([end, deferred]), tsplus_module_4.Data(instant, sorted.tail)];
        }
      }

      return [tsplus_module_30.none, tsplus_module_4.Data(end, data.sleeps)];
    })))(this.awaitSuspended);
  }

}
//# sourceMappingURL=TestClockInternal.mjs.map