"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MVarSym = exports.MVarInternal = exports.MVar = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/TRef/operations/get"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/STM/definition/primitives"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/TRef/operations/set"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/STM/operations/as"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/STM/operations/collect"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Function"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/STM/operations/retry"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/fold"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const MVarSym = /*#__PURE__*/Symbol.for("@effect/core/concurrent/MVar");
exports.MVarSym = MVarSym;
const MVar = {
  $: {}
};
exports.MVar = MVar;

class MVarInternal {
  constructor(content) {
    this[_a] = MVarSym;
    this._content = content;
  }
  /**
   * Check whether the `MVar` is empty.
   *
   * Notice that the boolean value returned is just a snapshot of the state of
   * the `MVar`. By the time you get to react on its result, the `MVar` may have
   * been filled (or emptied) - so be extremely careful when using this
   * operation. Use `tryTake` instead if possible.
   */


  get isEmpty() {
    return tsplus_module_3.map(maybe => tsplus_module_1.isNone(maybe))(tsplus_module_2.get(this._content)).commit;
  }
  /**
   * A slight variation on `update` that allows a value to be returned (`b`) in
   * addition to the modified value of the `MVar`.
   */


  modify(f) {
    return tsplus_module_3.flatMap(([b, newA]) => tsplus_module_5.as(b)(tsplus_module_4.set(newA)(this._content)))(tsplus_module_6.collectNow(maybe => {
      if (tsplus_module_1.isSome(maybe)) {
        const [b, newA] = f(maybe.value);
        return tsplus_module_1.some([b, tsplus_module_1.some(newA)]);
      }

      return tsplus_module_1.none;
    })(tsplus_module_2.get(this._content))).commit;
  }
  /**
   * Put a `value` into an `MVar`. If the `MVar` is currently full, `put` will
   * wait until it becomes empty.
   */


  put(value) {
    return tsplus_module_3.zipRight(tsplus_module_4.set(tsplus_module_1.some(value))(this._content))(tsplus_module_6.collectNow(maybe => tsplus_module_1.isNone(maybe) ? tsplus_module_1.some(undefined) : tsplus_module_1.none)(tsplus_module_2.get(this._content))).commit;
  }
  /**
   * Atomically read the contents of an `MVar`. If the `MVar` is currently
   * empty, `read` will wait until it is full. `read` is guaranteed to receive
   * the next `put`.
   */


  get read() {
    return tsplus_module_6.collectNow(tsplus_module_7.identity)(tsplus_module_2.get(this._content)).commit;
  }
  /**
   * Take a value from an `MVar`, put a new value into the `MVar` and return the
   * value taken.
   */


  swap(value) {
    return tsplus_module_3.flatMap(ref => tsplus_module_9.fold(() => tsplus_module_8.retry, other => tsplus_module_5.as(other)(tsplus_module_4.set(tsplus_module_1.some(value))(this._content)))(ref))(tsplus_module_2.get(this._content)).commit;
  }
  /**
   * Return the contents of the `MVar`. If the `MVar` is currently empty, `take`
   * will wait until it is full. After a `take`, the `MVar` is left empty.
   */


  get take() {
    return tsplus_module_3.flatMap(a => tsplus_module_5.as(a)(tsplus_module_4.set(tsplus_module_1.none)(this._content)))(tsplus_module_6.collectNow(tsplus_module_7.identity)(tsplus_module_2.get(this._content))).commit;
  }
  /**
   * A non-blocking version of `put`. The `tryPut` function attempts to put the
   * `value` into the `MVar`, returning `true` if it was successful, or
   * `false` otherwise.
   */


  tryPut(value) {
    return tsplus_module_3.flatMap(maybe => tsplus_module_9.fold(() => tsplus_module_5.as(true)(tsplus_module_4.set(tsplus_module_1.some(value))(this._content)), () => tsplus_module_3.succeed(false))(maybe))(tsplus_module_2.get(this._content)).commit;
  }
  /**
   * A non-blocking version of `read`. The `tryRead` function returns
   * immediately, with `None` if the `MVar` was empty, or `Some(x)` if the
   * `MVar` was full with contents.
   */


  get tryRead() {
    return tsplus_module_2.get(this._content).commit;
  }
  /**
   * A non-blocking version of `take`. The `tryTake` action returns immediately,
   * with `None` if the `MVar` was empty, or `Some(x)` if the `MVar` was full
   * with contents. After `tryTake`, the `MVar` is left empty.
   */


  get tryTake() {
    return tsplus_module_3.flatMap(content => tsplus_module_9.fold(() => tsplus_module_3.succeed(tsplus_module_1.none), value => tsplus_module_3.zipRight(tsplus_module_3.succeed(tsplus_module_1.some(value)))(tsplus_module_4.set(tsplus_module_1.none)(this._content)))(content))(tsplus_module_2.get(this._content)).commit;
  }
  /**
   * Replaces the contents of an `MVar` with the result of `f(a)`.
   */


  update(f) {
    return tsplus_module_3.flatMap(value => tsplus_module_4.set(value)(this._content))(tsplus_module_6.collectNow(maybe => tsplus_module_1.isSome(maybe) ? tsplus_module_1.some(tsplus_module_1.some(f(maybe.value))) : tsplus_module_1.none)(tsplus_module_2.get(this._content))).commit;
  }

}

exports.MVarInternal = MVarInternal;
_a = MVarSym;
//# sourceMappingURL=definition.js.map