import { Effect } from "@effect/core/io/Effect/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { FiberId } from "@effect/core/io/FiberId/definition";
/**
 * Imports an asynchronous effect into a pure `Effect` value, possibly returning
 * the value synchronously.
 *
 * If the register function returns a value synchronously, then the callback
 * function `Effect<R, E, A> => any` must not be called. Otherwise the callback
 * function must be called at most once.
 * @tsplus static effect/core/io/Effect.Ops asyncMaybe
 * @tsplus location "@effect/core/io/Effect/operations/asyncMaybe"
 */
export declare function asyncMaybe<R, E, A>(register: (callback: (_: Effect<R, E, A>) => void) => Maybe<Effect<R, E, A>>): Effect<R, E, A>;
/**
 * Imports an asynchronous effect into a pure `Effect` value, possibly returning
 * the value synchronously.
 *
 * If the register function returns a value synchronously, then the callback
 * function `Effect<R, E, A> => any` must not be called. Otherwise the callback
 * function must be called at most once.
 *
 * The list of fibers, that may complete the async callback, is used to
 * provide better diagnostics.
 * @tsplus static effect/core/io/Effect.Ops asyncMaybeBlockingOn
 * @tsplus location "@effect/core/io/Effect/operations/asyncMaybe"
 */
export declare function asyncMaybeBlockingOn<R, E, A>(register: (callback: (_: Effect<R, E, A>) => void) => Maybe<Effect<R, E, A>>, blockingOn: FiberId): Effect<R, E, A>;
//# sourceMappingURL=asyncMaybe.d.ts.map