import { Effect } from "@effect/core/io/Effect/definition";
import { Cause } from "@effect/core/io/Cause/definition";
import { ImmutableMap } from "@tsplus/stdlib/collections/ImmutableMap/definition";
/**
 * Logs the specified message at the current log level.
 * @tsplus static effect/core/io/Effect.Ops log
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function log(message: string): Effect<never, never, void>;
/**
 * Logs the specified message at the debug log level.
 * @tsplus static effect/core/io/Effect.Ops logDebug
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logDebug(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the debug log level.
 * @tsplus static effect/core/io/Effect.Ops logDebugCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logDebugCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the debug log level.
 * @tsplus static effect/core/io/Effect.Ops logDebugCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logDebugCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message at the error log level.
 * @tsplus static effect/core/io/Effect.Ops logError
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logError(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the error log level.
 * @tsplus static effect/core/io/Effect.Ops logErrorCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logErrorCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the error log level.
 * @tsplus static effect/core/io/Effect.Ops logErrorCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logErrorCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message at the fatal log level.
 * @tsplus static effect/core/io/Effect.Ops logFatal
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logFatal(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the fatal log level.
 * @tsplus static effect/core/io/Effect.Ops logFatalCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logFatalCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the fatal log level.
 * @tsplus static effect/core/io/Effect.Ops logFatalCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logFatalCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message at the informational log level.
 * @tsplus static effect/core/io/Effect.Ops logInfo
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logInfo(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the informational log level.
 * @tsplus static effect/core/io/Effect.Ops logInfoCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logInfoCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the informational log level.
 * @tsplus static effect/core/io/Effect.Ops logInfoCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logInfoCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message at the warning log level.
 * @tsplus static effect/core/io/Effect.Ops logWarning
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logWarning(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the warning log level.
 * @tsplus static effect/core/io/Effect.Ops logWarningCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logWarningCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the warning log level.
 * @tsplus static effect/core/io/Effect.Ops logWarningCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logWarningCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message at the trace log level.
 * @tsplus static effect/core/io/Effect.Ops logTrace
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logTrace(message: string): Effect<never, never, void>;
/**
 * Logs the specified cause at the trace log level.
 * @tsplus static effect/core/io/Effect.Ops logTraceCause
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logTraceCause<E>(cause: Cause<E>): Effect<never, never, void>;
/**
 * Logs the specified message and cause at the trace log level.
 * @tsplus static effect/core/io/Effect.Ops logTraceCauseMessage
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logTraceCauseMessage<E>(message: string, cause: Cause<E>): Effect<never, never, void>;
/**
 * Adjusts the label for the current logging span.
 * @tsplus static effect/core/io/Effect.Ops logSpan
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logSpan(label: string): <R, E, A>(effect: import("../definition").Effect<R, E, A>) => import("../definition").Effect<R, E, A>;
/**
 * Annotates each log in this effect with the specified log annotation.
 * @tsplus static effect/core/io/Effect.Ops logAnnotate
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logAnnotate(key: string, value: string): <R, E, A>(effect: import("../definition").Effect<R, E, A>) => import("../definition").Effect<R, E, A>;
/**
 * Retrieves the log annotations associated with the current scope.
 * @tsplus static effect/core/io/Effect.Ops logAnnotations
 * @tsplus location "@effect/core/io/Effect/operations/logging"
 */
export declare function logAnnotations(): Effect<never, never, ImmutableMap<string, string>>;
//# sourceMappingURL=logging.d.ts.map