"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parallelism = parallelism;
exports.parallelismWith = parallelismWith;
exports.withParallelism = withParallelism;
exports.withParallelismUnbounded = withParallelismUnbounded;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/_internal/runtime"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/suspendSucceed"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Retrieves the maximum number of fibers for parallel operators or `None` if
 * it is unbounded.
 *
 * @tsplus static effect/core/io/Effect.Ops parallelism
 */
function parallelism() {
  return tsplus_module_1.currentParallelism.get;
}
/**
 * Retrieves the current maximum number of fibers for parallel operators and
 * uses it to run the specified effect.
 *
 * @tsplus static effect/core/io/Effect.Ops parallelismWith
 */


function parallelismWith(f) {
  return tsplus_module_1.currentParallelism.getWith(f);
}
/**
 * Runs the specified effect with the specified maximum number of fibers for
 * parallel operators.
 *
 * @tsplus static effect/core/io/Effect.Aspects withParallelism
 * @tsplus pipeable effect/core/io/Effect withParallelism
 */


function withParallelism(n) {
  return self => tsplus_module_3.suspendSucceed(() => tsplus_module_1.currentParallelism.locally(tsplus_module_2.some(n))(self));
}
/**
 * Runs the specified effect with an unbounded maximum number of fibers for
 * parallel operators.
 *
 * @tsplus getter effect/core/io/Effect withParallelismUnbounded
 */


function withParallelismUnbounded(self) {
  return tsplus_module_3.suspendSucceed(() => tsplus_module_1.currentParallelism.locally(tsplus_module_2.none)(self));
}
//# sourceMappingURL=parallelism.js.map