import { LazyArg } from "@tsplus/stdlib/data/Function";
import { Effect } from "@effect/core/io/Effect/definition";
/**
 * Create an `Effect` that when executed will construct `promise` and wait for
 * its result, errors will be handled using `onReject`.
 * @tsplus static effect/core/io/Effect.Ops tryCatchPromise
 * @tsplus location "@effect/core/io/Effect/operations/promise"
 */
export declare function tryCatchPromise<E, A>(promise: LazyArg<Promise<A>>, onReject: (reason: unknown) => E): Effect<never, E, A>;
/**
 * Create an `Effect` that when executed will construct `promise` and wait for
 * its result, errors will produce failure as `unknown`.
 * @tsplus static effect/core/io/Effect.Ops tryPromise
 * @tsplus location "@effect/core/io/Effect/operations/promise"
 */
export declare function tryPromise<A>(promise: LazyArg<Promise<A>>): Effect<never, unknown, A>;
/**
 * Like `tryPromise` but produces a defect in case of errors.
 * @tsplus static effect/core/io/Effect.Ops promise
 * @tsplus location "@effect/core/io/Effect/operations/promise"
 */
export declare function promise<A>(promise: LazyArg<Promise<A>>): Effect<never, never, A>;
//# sourceMappingURL=promise.d.ts.map