"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.raceFibersWith = raceFibersWith;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/AtomicBoolean"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/_internal/runtime"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/from"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/FiberId/operations/combineAll"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/async"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/withFiberRuntime"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Forks this effect and the specified effect into their own fibers, and races
 * them, calling one of two specified callbacks depending on which fiber wins
 * the race. This method does not interrupt, join, or otherwise do anything
 * with the fibers. It can be considered a low-level building block for
 * higher-level operators like `race`.
 *
 * @tsplus static effect/core/io/Effect.Aspects raceFibersWith
 * @tsplus pipeable effect/core/io/Effect raceFibersWith
 */
function raceFibersWith(that, selfWins, thatWins) {
  return self => tsplus_module_7.withFiberRuntime((parentState, parentStatus) => {
    const parentRuntimeFlags = parentStatus.runtimeFlags;
    const raceIndicator = new tsplus_module_1.AtomicBoolean(true);
    const leftFiber = (0, tsplus_module_2.unsafeForkUnstarted)(self, parentState, parentRuntimeFlags);
    const rightFiber = (0, tsplus_module_2.unsafeForkUnstarted)(that, parentState, parentRuntimeFlags);
    leftFiber.setFiberRef(tsplus_module_2.forkScopeOverride, tsplus_module_3.some(parentState.scope));
    rightFiber.setFiberRef(tsplus_module_2.forkScopeOverride, tsplus_module_3.some(parentState.scope));
    return tsplus_module_6.asyncBlockingOn(cb => {
      leftFiber.addObserver(() => complete(leftFiber, rightFiber, selfWins, raceIndicator, cb));
      rightFiber.addObserver(() => complete(rightFiber, leftFiber, thatWins, raceIndicator, cb));
      leftFiber.startFork(self);
      rightFiber.startFork(that);
    }, tsplus_module_5.combineAll(tsplus_module_4.from([leftFiber.id, rightFiber.id])));
  });
}

function complete(winner, loser, cont, ab, cb) {
  if (ab.compareAndSet(true, false)) {
    cb(cont(winner, loser));
  }
}
//# sourceMappingURL=raceFibersWith.js.map