import { Schedule } from "@effect/core/io/Schedule/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Either } from "@tsplus/stdlib/data/Either/definition";
/**
 * Returns a new effect that repeats this effect according to the specified
 * schedule or until the first failure, at which point, the failure value and
 * schedule output are passed to the specified handler.
 *
 * Scheduled recurrences are in addition to the first execution, so that
 * `io.repeat(Schedule.once)` yields an effect that executes `io`, and then if
 * that succeeds, executes `io` an additional time.
 * @tsplus static effect/core/io/Effect.Aspects repeatOrElseEither
 * @tsplus pipeable effect/core/io/Effect repeatOrElseEither
 * @tsplus location "@effect/core/io/Effect/operations/repeatOrElseEither"
 */
export declare function repeatOrElseEither<S, R1, A, B, E, R2, E2, C>(schedule: Schedule<S, R1, A, B>, orElse: (e: E, option: Maybe<B>) => Effect<R2, E2, C>): <R>(self: Effect<R, E, A>) => Effect<R | R1 | R2, E2, Either<C, B>>;
//# sourceMappingURL=repeatOrElseEither.d.ts.map