import { Effect } from "@effect/core/io/Effect/definition";
import { Runtime } from "@effect/core/io/Runtime/definition";
/**
 * Returns an effect that accesses the runtime, which can be used to
 * (unsafely) execute tasks. This is useful for integration with legacy code
 * that must call back into Effect code.
 * @tsplus static effect/core/io/Effect.Ops runtime
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare function runtime<R>(): Effect<R, never, Runtime<R>>;
export declare const defaultFlags: import("../../RuntimeFlags").RuntimeFlags;
export declare const defaultRuntime: import("../../Runtime").Runtime<never>;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunPromise
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunPromise
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunPromise: <E, A>(effect: import("../definition").Effect<never, E, A>) => Promise<A>;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunAsync
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunAsync
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunAsync: <E, A>(effect: import("../definition").Effect<never, E, A>) => void;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunAsyncWith
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunAsyncWith
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunAsyncWith: <E, A>(effect: import("../definition").Effect<never, E, A>, k: (exit: import("../../Exit").Exit<E, A>) => void) => void;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunPromiseExit
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunPromiseExit
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunPromiseExit: <E, A>(effect: import("../definition").Effect<never, E, A>) => Promise<import("../../Exit").Exit<E, A>>;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunWith
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunWith
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunWith: <E, A>(effect: import("../definition").Effect<never, E, A>, k: (exit: import("../../Exit").Exit<E, A>) => void) => (fiberId: import("../../FiberId").FiberId) => (_: (exit: import("../../Exit").Exit<E, A>) => void) => void;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunSync
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunSync
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunSync: <E, A>(effect: import("../definition").Effect<never, E, A>) => A;
/**
 * @tsplus fluent effect/core/io/Effect unsafeRunSyncExit
 * @tsplus static effect/core/io/Effect.Aspects unsafeRunSyncExit
 * @tsplus location "@effect/core/io/Effect/operations/runtime"
 */
export declare const unsafeRunSyncExit: <E, A>(effect: import("../definition").Effect<never, E, A>) => import("../../Exit").Exit<E, A>;
//# sourceMappingURL=runtime.d.ts.map