/**
 * Describes a strategy for evaluating multiple effects, potentially in
 * parallel. There are three possible execution strategies: `Sequential`,
 * `Parallel`, and `ParallelN`.
 *
 * @tsplus type effect/core/io/ExecutionStrategy
 */
export type ExecutionStrategy = Sequential | Parallel | ParallelN;
/**
 * @tsplus type effect/core/io/ExecutionStrategy.Ops
 */
export interface ExecutionStrategyOps {
    $: ExecutionStrategyAspects;
}
export declare const ExecutionStrategy: ExecutionStrategyOps;
/**
 * @tsplus type effect/core/io/ExecutionStrategy.Aspects
 */
export interface ExecutionStrategyAspects {
}
/**
 * Execute effects sequentially.
 */
export declare class Sequential {
    readonly _tag = "Sequential";
}
/**
 * Execute effects in parallel.
 */
export declare class Parallel {
    readonly _tag = "Parallel";
}
/**
 * Execute effects in parallel, up to the specified number of concurrent
 * fibers.
 */
export declare class ParallelN {
    readonly n: number;
    readonly _tag = "ParallelN";
    constructor(n: number);
}
/**
 * Execute effects sequentially.
 * @tsplus static effect/core/io/ExecutionStrategy.Ops Sequential
 * @tsplus location "@effect/core/io/ExecutionStrategy/definition"
 */
export declare const sequential: ExecutionStrategy;
/**
 * Execute effects in parallel.
 * @tsplus static effect/core/io/ExecutionStrategy.Ops Parallel
 * @tsplus location "@effect/core/io/ExecutionStrategy/definition"
 */
export declare const parallel: ExecutionStrategy;
/**
 * Execute effects in parallel, up to the specified number of concurrent
 * fibers.
 * @tsplus static effect/core/io/ExecutionStrategy.Ops ParallelN
 * @tsplus location "@effect/core/io/ExecutionStrategy/definition"
 */
export declare function parallelN(n: number): ExecutionStrategy;
//# sourceMappingURL=definition.d.ts.map