import { FiberId } from "@effect/core/io/FiberId/definition";
import { Fiber } from "@effect/core/io/Fiber/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Exit } from "@effect/core/io/Exit/definition";
/**
 * A fiber that is already interrupted.
 * @tsplus static effect/core/io/Fiber.Ops interruptAs
 * @tsplus location "@effect/core/io/Fiber/operations/interruptAs"
 */
export declare function interruptAs(fiberId: FiberId): Fiber<never, never>;
/**
 * Interrupts the fiber as if interrupted from the specified fiber. If the
 * fiber has already exited, the returned effect will resume immediately.
 * Otherwise, the effect will resume when the fiber exits.
 * @tsplus static effect/core/io/Fiber.Aspects interruptAs
 * @tsplus static effect/core/io/RuntimeFiber.Aspects interruptAs
 * @tsplus pipeable effect/core/io/Fiber interruptAs
 * @tsplus pipeable effect/core/io/RuntimeFiber interruptAs
 * @tsplus location "@effect/core/io/Fiber/operations/interruptAs"
 */
export declare function interruptAsNow(fiberId: FiberId): <E, A>(self: import("@effect/core/io/Fiber/definition").Fiber<E, A>) => import("../../Effect").Effect<never, never, import("../../Exit").Exit<E, A>>;
/**
 * Interrupts the fiber as if interrupted from the specified fiber. If the
 * fiber has already exited, the returned effect will resume immediately.
 * Otherwise, the effect will resume when the fiber exits.
 * @tsplus static effect/core/io/Fiber.Aspects interruptAsFork
 * @tsplus static effect/core/io/RuntimeFiber.Aspects interruptAsFork
 * @tsplus pipeable effect/core/io/Fiber interruptAsFork
 * @tsplus pipeable effect/core/io/RuntimeFiber interruptAsFork
 * @tsplus location "@effect/core/io/Fiber/operations/interruptAs"
 */
export declare function interruptAsFork(fiberId: FiberId): <E, A>(self: import("@effect/core/io/Fiber/definition").Fiber<E, A>) => Effect<never, never, void>;
//# sourceMappingURL=interruptAs.d.ts.map