import { ImmutableMap } from "@tsplus/stdlib/collections/ImmutableMap/definition";
import { FiberRef } from "@effect/core/io/FiberRef/definition";
import { List } from "@tsplus/stdlib/collections/List/definition";
import { FiberId } from "@effect/core/io/FiberId/definition";
import { Equals } from "@tsplus/stdlib/structure/Equals";
import { HashSet } from "@tsplus/stdlib/collections/HashSet/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { Effect } from "@effect/core/io/Effect/definition";
export declare const FiberRefsSym: unique symbol;
export type FiberRefsSym = typeof FiberRefsSym;
/**
 * `FiberRefs` is a data type that represents a collection of `FiberRef` values.
 * This allows safely propagating `FiberRef` values across fiber boundaries, for
 * example between an asynchronous producer and consumer.
 *
 * @tsplus type effect/core/io/FiberRefs
 * @tsplus companion effect/core/io/FiberRefs.Ops
 */
export declare class FiberRefs {
    readonly locals: ImmutableMap<FiberRef<any>, List.NonEmpty<readonly [FiberId.Runtime, unknown]>>;
    readonly [FiberRefsSym]: FiberRefsSym;
    constructor(locals: ImmutableMap<FiberRef<any>, List.NonEmpty<readonly [FiberId.Runtime, unknown]>>);
    joinAs(fiberId: FiberId.Runtime, that: FiberRefs): FiberRefs;
    /**
     * Forks this collection of fiber refs as the specified child fiber id. This
     * will potentially modify the value of the fiber refs, as determined by the
     * individual fiber refs that make up the collection.
     */
    forkAs(childId: FiberId.Runtime): FiberRefs;
    get fiberRefs(): import("@tsplus/stdlib/collections/HashSet").HashSet<import("../FiberRef").FiberRef<any>>;
    get<A>(fiberRef: FiberRef<A>): Maybe<A>;
    getOrDefault<A>(fiberRef: FiberRef<A>): A;
    get setAll(): import("../Effect").Effect<never, never, void>;
    updateAs<A>(fiberId: FiberId.Runtime, fiberRef: FiberRef<A>, value: A): FiberRefs;
    delete<A>(fiberRef: FiberRef<A>): FiberRefs;
}
/**
 * @tsplus type effect/core/io/FiberRefs.Aspects
 */
export interface FiberRefsAspects {
}
/**
 * @tsplus static effect/core/io/FiberRefs.Ops $
 * @tsplus location "@effect/core/io/FiberRefs/definition"
 */
export declare const FiberRefsAspects: FiberRefsAspects;
//# sourceMappingURL=definition.d.ts.map