"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FiberRefsSym = exports.FiberRefsAspects = exports.FiberRefs = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableMap/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/from"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableMap/get"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/map"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/getOrElse"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/excl-forEach"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/empty"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableMap/set"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableMap/remove"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const FiberRefsSym = /*#__PURE__*/Symbol.for("@effect/core/io/FiberRefs");
/**
 * `FiberRefs` is a data type that represents a collection of `FiberRef` values.
 * This allows safely propagating `FiberRef` values across fiber boundaries, for
 * example between an asynchronous producer and consumer.
 *
 * @tsplus type effect/core/io/FiberRefs
 * @tsplus companion effect/core/io/FiberRefs.Ops
 */

exports.FiberRefsSym = FiberRefsSym;

class FiberRefs {
  constructor(locals) {
    this.locals = locals;
    this[_a] = FiberRefsSym;
  }

  joinAs(fiberId, that) {
    const parentFiberRefs = new Map(this.locals.internalMap);
    that.locals.internalMap.forEach((childStack, fiberRef) => {
      const ref = fiberRef;
      const childValue = childStack.head[1];

      if (!tsplus_module_1.equals(childStack.head[0], fiberId)) {
        if (!parentFiberRefs.has(ref)) {
          if (tsplus_module_1.equals(childValue, ref.initial)) {
            return;
          } else {
            parentFiberRefs.set(fiberRef, tsplus_module_2.cons([fiberId, ref.join(ref.initial, childValue)], tsplus_module_2.nil()));
            return;
          }
        }

        const parentStack = parentFiberRefs.get(ref);
        const [ancestor, wasModified] = findAnchestor(fiberRef, parentStack, childStack);

        if (wasModified) {
          const patch = ref.diff(ancestor, childValue);
          const oldValue = parentStack.head[1];
          const newValue = ref.join(oldValue, ref.patch(patch)(oldValue));

          if (!tsplus_module_1.equals(oldValue, newValue)) {
            let newStack;
            const [parentFiberId] = parentStack.head;

            if (tsplus_module_1.equals(parentFiberId, fiberId)) {
              newStack = tsplus_module_2.cons([parentFiberId, newValue], parentStack.tail);
            } else {
              newStack = tsplus_module_2.cons([fiberId, newValue], parentStack);
            }

            parentFiberRefs.set(ref, newStack);
          }
        }
      }
    });
    return new FiberRefs(new tsplus_module_3.ImmutableMap(parentFiberRefs));
  }
  /**
   * Forks this collection of fiber refs as the specified child fiber id. This
   * will potentially modify the value of the fiber refs, as determined by the
   * individual fiber refs that make up the collection.
   */


  forkAs(childId) {
    const map = new Map();
    this.locals.internalMap.forEach((stack, fiberRef) => {
      const oldValue = stack.head[1];
      const newValue = fiberRef.patch(fiberRef.fork)(oldValue);

      if (tsplus_module_1.equals(oldValue, newValue)) {
        map.set(fiberRef, stack);
      } else {
        map.set(fiberRef, tsplus_module_2.cons([childId, newValue], stack));
      }
    });
    return new FiberRefs(new tsplus_module_3.ImmutableMap(map));
  }

  get fiberRefs() {
    return tsplus_module_4.from(this.locals.internalMap.keys());
  }

  get(fiberRef) {
    return tsplus_module_6.map(list => list.head[1])(tsplus_module_5.get(fiberRef)(this.locals));
  }

  getOrDefault(fiberRef) {
    return tsplus_module_7.getOrElse_(this.get(fiberRef), () => fiberRef.initial);
  }

  get setAll() {
    return tsplus_module_8.forEachDiscard(this.fiberRefs, fiberRef => fiberRef.set(this.getOrDefault(fiberRef)));
  }

  updateAs(fiberId, fiberRef, value) {
    const oldStack = tsplus_module_7.getOrElse_(tsplus_module_5.get(fiberRef)(this.locals), () => tsplus_module_9.empty());
    const newStack = tsplus_module_2.isNil(oldStack) ? tsplus_module_2.cons([fiberId, value], tsplus_module_2.nil()) : tsplus_module_1.equals(oldStack.head[0], fiberId) ? tsplus_module_2.cons([fiberId, value], oldStack.tail) : tsplus_module_2.cons([fiberId, value], oldStack);
    return new FiberRefs(tsplus_module_10.set(fiberRef, newStack)(this.locals));
  }

  delete(fiberRef) {
    return new FiberRefs(tsplus_module_11.remove(fiberRef)(this.locals));
  }

}

exports.FiberRefs = FiberRefs;
_a = FiberRefsSym;
/**
 * @tsplus static effect/core/io/FiberRefs.Ops $
 */

const FiberRefsAspects = {};
/**
 * @tsplus tailRec
 */

exports.FiberRefsAspects = FiberRefsAspects;

function findAnchestor(ref, parentStack, childStack, childModified = false) {
  var ref_1 = ref,
      parentStack_1 = parentStack,
      childStack_1 = childStack,
      childModified_1 = childModified;
  var ref_2 = ref,
      parentStack_2 = parentStack,
      childStack_2 = childStack,
      childModified_2 = childModified;

  while (1) {
    if (tsplus_module_2.isCons(parentStack_1) && tsplus_module_2.isCons(childStack_1)) {
      const [parentFiberId] = parentStack_1.head;
      const parentAncestors = parentStack_1.tail;
      const [childFiberId, childRefValue] = childStack_1.head;
      const childAncestors = childStack_1.tail;

      if (parentFiberId.startTimeMillis < childFiberId.startTimeMillis) {
        ref_2 = ref_1;
        parentStack_2 = parentStack_1;
        childStack_2 = childAncestors;
        childModified_2 = true;
        ref_1 = ref_2;
        parentStack_1 = parentStack_2;
        childStack_1 = childStack_2;
        childModified_1 = childModified_2;
        continue;
      } else if (parentFiberId.startTimeMillis > childFiberId.startTimeMillis) {
        ref_2 = ref_1;
        parentStack_2 = parentAncestors;
        childStack_2 = childStack_1;
        childModified_2 = childModified_1;
        ref_1 = ref_2;
        parentStack_1 = parentStack_2;
        childStack_1 = childStack_2;
        childModified_1 = childModified_2;
        continue;
      } else {
        if (parentFiberId.id < childFiberId.id) {
          ref_2 = ref_1;
          parentStack_2 = parentStack_1;
          childStack_2 = childAncestors;
          childModified_2 = true;
          ref_1 = ref_2;
          parentStack_1 = parentStack_2;
          childStack_1 = childStack_2;
          childModified_1 = childModified_2;
          continue;
        } else if (parentFiberId.id > childFiberId.id) {
          ref_2 = ref_1;
          parentStack_2 = parentAncestors;
          childStack_2 = childStack_1;
          childModified_2 = childModified_1;
          ref_1 = ref_2;
          parentStack_1 = parentStack_2;
          childStack_1 = childStack_2;
          childModified_1 = childModified_2;
          continue;
        } else {
          return [childRefValue, childModified_1];
        }
      }
    }

    return [ref_1.initial, true];
  }
}
//# sourceMappingURL=definition.js.map