"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Strategy = exports.Sliding = exports.Dropping = exports.BackPressure = void 0;
exports.backPressureStrategy = backPressureStrategy;
exports.droppingStrategy = droppingStrategy;
exports.slidingStrategy = slidingStrategy;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/mutable/MutableQueue"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/prepend"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/mutable/MutableQueue/unbounded"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/_internal/runtime"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/sync"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/suspendSucceed"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/withFiberRuntime"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/unit"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/excl-forEach"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/fiberId"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/do"));

var tsplus_module_12 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/tap"));

var tsplus_module_13 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/filter"));

var tsplus_module_14 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

var _unsafeCompleteDeferred = /*#__PURE__*/require("@effect/core/io/Hub/operations/_internal/unsafeCompleteDeferred");

var _unsafeOfferAll = /*#__PURE__*/require("@effect/core/io/Hub/operations/_internal/unsafeOfferAll");

var _unsafePollAllQueue = /*#__PURE__*/require("@effect/core/io/Hub/operations/_internal/unsafePollAllQueue");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const Strategy = {};
exports.Strategy = Strategy;

class BaseStrategy {
  unsafeCompletePollers(hub, subscribers, subscription, pollers) {
    let keepPolling = true;

    while (keepPolling && !subscription.isEmpty) {
      const poller = pollers.poll(tsplus_module_1.EmptyMutableQueue);

      if (poller === tsplus_module_1.EmptyMutableQueue) {
        const subPollerPair = [subscription, pollers];
        subscribers.remove(subPollerPair);

        if (pollers.isEmpty) {
          keepPolling = false;
        } else {
          subscribers.add(subPollerPair);
        }
      } else {
        const pollResult = subscription.poll(tsplus_module_1.EmptyMutableQueue);

        if (pollResult == tsplus_module_1.EmptyMutableQueue) {
          (0, _unsafeOfferAll.unsafeOfferAll)(pollers, tsplus_module_2.prepend(poller)((0, _unsafePollAllQueue.unsafePollAllQueue)(pollers)));
        } else {
          (0, _unsafeCompleteDeferred.unsafeCompleteDeferred)(poller, pollResult);
          this.unsafeOnHubEmptySpace(hub, subscribers);
        }
      }
    }
  }

  unsafeCompleteSubscribers(hub, subscribers) {
    for (const [subscription, pollers] of subscribers) {
      this.unsafeCompletePollers(hub, subscribers, subscription, pollers);
    }
  }

}
/**
 * A strategy that applies back pressure to publishers when the hub is at
 * capacity. This guarantees that all subscribers will receive all messages
 * published to the hub while they are subscribed. However, it creates the
 * risk that a slow subscriber will slow down the rate at which messages
 * are published and received by other subscribers.
 */


class BackPressure extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.publishers = tsplus_module_3.unbounded();
  }

  handleSurplus(hub, subscribers, as, isShutdown) {
    return tsplus_module_7.withFiberRuntime(state => {
      const deferred = tsplus_module_4.unsafeMakeDeferred(state.id);
      return tsplus_module_4.onInterrupt(() => tsplus_module_5.sync(() => this.unsafeRemove(deferred)))(tsplus_module_6.suspendSucceed(() => {
        this.unsafeOffer(as, deferred);
        this.unsafeOnHubEmptySpace(hub, subscribers);
        this.unsafeCompleteSubscribers(hub, subscribers);
        return isShutdown.get ? tsplus_module_4.interrupt : deferred.await;
      }));
    });
  }

  get shutdown() {
    return tsplus_module_8.asUnit(tsplus_module_12.tap(({
      fiberId,
      publishers
    }) => tsplus_module_9.forEachParDiscard(publishers, ([_, deferred, last]) => last ? deferred.interruptAs(fiberId) : tsplus_module_8.unit))(tsplus_module_11.bind("publishers", () => tsplus_module_5.sync(() => (0, _unsafePollAllQueue.unsafePollAllQueue)(this.publishers)))(tsplus_module_11.bind("fiberId", () => tsplus_module_10.fiberId)(tsplus_module_11.Do()))));
  }

  unsafeOnHubEmptySpace(hub, subscribers) {
    let keepPolling = true;

    while (keepPolling && !hub.isFull) {
      const publisher = this.publishers.poll(tsplus_module_1.EmptyMutableQueue);

      if (publisher === tsplus_module_1.EmptyMutableQueue) {
        keepPolling = false;
      } else {
        const published = hub.publish(publisher[0]);

        if (published && publisher[2]) {
          (0, _unsafeCompleteDeferred.unsafeCompleteDeferred)(publisher[1], true);
        } else if (!published) {
          (0, _unsafeOfferAll.unsafeOfferAll)(this.publishers, tsplus_module_2.prepend(publisher)((0, _unsafePollAllQueue.unsafePollAllQueue)(this.publishers)));
        }

        this.unsafeCompleteSubscribers(hub, subscribers);
      }
    }
  }

  unsafeOffer(as, deferred) {
    const it = as[Symbol.iterator]();
    let curr = it.next();

    if (!curr.done) {
      let next;

      while ((next = it.next()) && !next.done) {
        this.publishers.offer([curr.value, deferred, false]);
        curr = next;
      }

      this.publishers.offer([curr.value, deferred, true]);
    }
  }

  unsafeRemove(deferred) {
    (0, _unsafeOfferAll.unsafeOfferAll)(this.publishers, tsplus_module_13.filter(([_, a]) => a !== deferred)((0, _unsafePollAllQueue.unsafePollAllQueue)(this.publishers)));
  }

}
/**
 * A strategy that drops new messages when the hub is at capacity. This
 * guarantees that a slow subscriber will not slow down the rate at which
 * messages are published. However, it creates the risk that a slow
 * subscriber will slow down the rate at which messages are received by
 * other subscribers and that subscribers may not receive all messages
 * published to the hub while they are subscribed.
 */


exports.BackPressure = BackPressure;

class Dropping extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.shutdown = tsplus_module_8.unit;
  }

  handleSurplus(_hub, _subscribers, _as, _isShutdown) {
    return tsplus_module_14.succeed(false);
  }

  unsafeOnHubEmptySpace(_hub, _subscribers) {//
  }

}
/**
 * A strategy that adds new messages and drops old messages when the hub is
 * at capacity. This guarantees that a slow subscriber will not slow down
 * the rate at which messages are published and received by other
 * subscribers. However, it creates the risk that a slow subscriber will
 * not receive some messages published to the hub while it is subscribed.
 */


exports.Dropping = Dropping;

class Sliding extends BaseStrategy {
  constructor() {
    super(...arguments);
    this.shutdown = tsplus_module_8.unit;
  }

  unsafeSlidingPublish(hub, as) {
    const it = as[Symbol.iterator]();
    let next = it.next();

    if (!next.done && hub.capacity > 0) {
      let a = next.value;
      let loop = true;

      while (loop) {
        hub.slide();
        const pub = hub.publish(a);

        if (pub && (next = it.next()) && !next.done) {
          a = next.value;
        } else if (pub) {
          loop = false;
        }
      }
    }
  }

  handleSurplus(hub, subscribers, as, _isShutdown) {
    return tsplus_module_5.sync(() => {
      this.unsafeSlidingPublish(hub, as);
      this.unsafeCompleteSubscribers(hub, subscribers);
      return true;
    });
  }

  unsafeOnHubEmptySpace(_hub, _subscribers) {//
  }

}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops BackPressure
 */


exports.Sliding = Sliding;

function backPressureStrategy() {
  return new BackPressure();
}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops Dropping
 */


function droppingStrategy() {
  return new Dropping();
}
/**
 * @tsplus static effect/core/io/Hub/Strategy.Ops Sliding
 */


function slidingStrategy() {
  return new Sliding();
}
//# sourceMappingURL=strategy.js.map