import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Hash } from "@tsplus/stdlib/structure/Hash";
/**
 * The `InterruptStatus` of a fiber determines whether or not it can be
 * interrupted. The status can change over time in different regions.
 *
 * @tsplus type effect/core/io/InterruptStatus
 */
export interface InterruptStatus extends Equals {
    readonly isInterruptible: boolean;
    readonly isUninterruptible: boolean;
    readonly toBoolean: boolean;
}
/**
 * @tsplus type effect/core/io/InterruptStatus.Ops
 */
export interface InterruptStatusOps {
    $: InterruptStatusAspects;
}
export declare const InterruptStatus: InterruptStatusOps;
/**
 * @tsplus type effect/core/io/InterruptStatus.Aspects
 */
export interface InterruptStatusAspects {
}
export declare class InterruptStatusImpl implements InterruptStatus {
    readonly isInterruptible: boolean;
    constructor(isInterruptible: boolean);
    get isUninterruptible(): boolean;
    get toBoolean(): boolean;
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
/**
 * Indicates the fiber can be interrupted right now.
 * @tsplus static effect/core/io/InterruptStatus.Ops Interruptible
 * @tsplus location "@effect/core/io/InterruptStatus/definition"
 */
export declare const Interruptible: InterruptStatus;
/**
 * Indicates the fiber cannot be interrupted right now.
 * @tsplus static effect/core/io/InterruptStatus.Ops Uninterruptible
 * @tsplus location "@effect/core/io/InterruptStatus/definition"
 */
export declare const Uninterruptible: InterruptStatus;
/**
 * @tsplus static effect/core/io/InterruptStatus.Ops fromBoolean
 * @tsplus location "@effect/core/io/InterruptStatus/definition"
 */
export declare function fromBoolean(b: boolean): InterruptStatus;
//# sourceMappingURL=definition.d.ts.map