import { Ref } from "@effect/core/io/Ref/definition";
import { Layer } from "@effect/core/io/Layer/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Scope } from "@effect/core/io/Scope/definition";
import { Env } from "@tsplus/stdlib/service/Env";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { FiberRefs } from "@effect/core/io/FiberRefs/definition";
import { Deferred } from "@effect/core/io/Deferred/definition";
import { Exit } from "@effect/core/io/Exit/definition";
/**
 * A `MemoMap` memoizes layers.
 */
export declare class MemoMap {
    readonly ref: Ref.Synchronized<Map<Layer<any, any, any>, readonly [Effect<never, any, any>, Scope.Finalizer]>>;
    constructor(ref: Ref.Synchronized<Map<Layer<any, any, any>, readonly [Effect<never, any, any>, Scope.Finalizer]>>);
    /**
     * Checks the memo map to see if a layer exists. If it is, immediately
     * returns it. Otherwise, obtains the layer, stores it in the memo map,
     * and adds a finalizer to the `Scope`.
     */
    getOrElseMemoize<RIn, E, ROut>(layer: Layer<RIn, E, ROut>, scope: Scope): Effect<RIn, E, Env<ROut>>;
}
/**
 * Creates an empty `MemoMap`.
 */
export declare function makeMemoMap(): Effect<never, never, MemoMap>;
/**
 * Builds a layer into a scoped value.
 * @tsplus getter effect/core/io/Layer build
 * @tsplus location "@effect/core/io/Layer/memoMap"
 */
export declare function build<RIn, E, ROut>(self: Layer<RIn, E, ROut>): Effect<RIn | Scope, E, Env<ROut>>;
/**
 * Builds a layer into an Effect value. Any resources associated with this layer
 * will be released when the specified scope is closed unless their scope has
 * been extended. This allows building layers where the lifetime of some of
 * the services output by the layer exceed the lifetime of the effect the
 * layer is provided to.
 * @tsplus static effect/core/io/Layer.Aspects buildWithScope
 * @tsplus pipeable effect/core/io/Layer buildWithScope
 * @tsplus location "@effect/core/io/Layer/memoMap"
 */
export declare function buildWithScope(scope: Scope): <RIn, E, ROut>(self: import("@effect/core/io/Layer/definition").Layer<RIn, E, ROut>) => import("../Effect").Effect<RIn, E, import("@tsplus/stdlib/service/Env").Env<ROut>>;
/**
 * @tsplus static effect/core/io/Layer.Aspects withScope
 * @tsplus pipeable effect/core/io/Layer withScope
 * @tsplus location "@effect/core/io/Layer/memoMap"
 */
export declare function withScope(scope: Scope): <RIn, E, ROut>(self: import("@effect/core/io/Layer/definition").Layer<RIn, E, ROut>) => import("../Effect").Effect<never, never, (_: MemoMap) => import("../Effect").Effect<RIn, E, import("@tsplus/stdlib/service/Env").Env<ROut>>>;
/**
 * Returns whether this layer is a fresh version that will not be shared.
 * @tsplus getter effect/core/io/Layer isFresh
 * @tsplus location "@effect/core/io/Layer/memoMap"
 */
export declare function isFresh<R, E, A>(self: Layer<R, E, A>): boolean;
//# sourceMappingURL=memoMap.d.ts.map