/**
 * A `LogLevel` represents the log level associated with an individual logging
 * operation. Log levels are used both to describe the granularity (or
 * importance) of individual log statements, as well as to enable tuning
 * verbosity of log output.
 *
 * @param ordinal
 *   The priority of the log message. Larger values indicate higher priority.
 * @param label
 *   A label associated with the log level.
 * @param syslog
 *   The syslog severity level of the log level.
 *
 * @tsplus type effect/core/io/LogLevel
 */
export type LogLevel = All | Fatal | Error | Warning | Info | Debug | Trace | None;
/**
 * @tsplus type effect/core/io/LogLevel.Ops
 */
export interface LogLevelOps {
    $: LogLevelAspects;
}
export declare const LogLevel: LogLevelOps;
/**
 * @tsplus type effect/core/io/LogLevel.Aspects
 */
export interface LogLevelAspects {
}
export interface All {
    readonly _tag: "All";
    readonly label: "ALL";
    readonly syslog: 0;
    readonly ordinal: number;
}
export interface Fatal {
    readonly _tag: "Fatal";
    readonly label: "FATAL";
    readonly syslog: 2;
    readonly ordinal: number;
}
export interface Error {
    readonly _tag: "Error";
    readonly label: "ERROR";
    readonly syslog: 3;
    readonly ordinal: number;
}
export interface Warning {
    readonly _tag: "Warning";
    readonly label: "WARN";
    readonly syslog: 4;
    readonly ordinal: number;
}
export interface Info {
    readonly _tag: "Info";
    readonly label: "INFO";
    readonly syslog: 6;
    readonly ordinal: number;
}
export interface Debug {
    readonly _tag: "Debug";
    readonly label: "DEBUG";
    readonly syslog: 7;
    readonly ordinal: number;
}
export interface Trace {
    readonly _tag: "Trace";
    readonly label: "TRACE";
    readonly syslog: 7;
    readonly ordinal: number;
}
export interface None {
    readonly _tag: "None";
    readonly label: "OFF";
    readonly syslog: 7;
    readonly ordinal: number;
}
/**
 * @tsplus static effect/core/io/LogLevel.Ops All
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const All: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Fatal
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Fatal: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Error
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Error: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Warning
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Warning: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Info
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Info: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Debug
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Debug: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops Trace
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const Trace: LogLevel;
/**
 * @tsplus static effect/core/io/LogLevel.Ops None
 * @tsplus location "@effect/core/io/LogLevel/definition"
 */
export declare const None: LogLevel;
//# sourceMappingURL=definition.d.ts.map