import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
import { Hash } from "@tsplus/stdlib/structure/Hash";
export declare const HistogramBoundariesSym: unique symbol;
export type HistogramBoundariesSym = typeof HistogramBoundariesSym;
/**
 * @tsplus type effect/core/io/Metrics/Histogram/Boundaries
 */
export interface HistogramBoundaries extends Equals {
    readonly values: Chunk<number>;
}
export declare class HistogramBoundariesInternal implements HistogramBoundaries {
    readonly values: Chunk<number>;
    readonly [HistogramBoundariesSym]: HistogramBoundariesSym;
    constructor(values: Chunk<number>);
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
/**
 * @tsplus type effect/core/io/Metrics/Histogram/Boundaries.Ops
 */
export interface HistogramBoundariesOps {
}
export declare const HistogramBoundaries: HistogramBoundariesOps;
/**
 * @tsplus static effect/core/io/Metrics/Histogram/Boundaries.Ops linear
 * @tsplus location "@effect/core/io/Metrics/Boundaries"
 */
export declare function fromChunk(chunk: Chunk<number>): HistogramBoundaries;
/**
 * A helper method to create histogram bucket boundaries for a histogram
 * with linear increasing values.
 * @tsplus static effect/core/io/Metrics/Histogram/Boundaries.Ops linear
 * @tsplus location "@effect/core/io/Metrics/Boundaries"
 */
export declare function linear(start: number, width: number, count: number): HistogramBoundaries;
/**
 * A helper method to create histogram bucket boundaries for a histogram
 * with exponentially increasing values.
 * @tsplus static effect/core/io/Metrics/Histogram/Boundaries.Ops exponential
 * @tsplus location "@effect/core/io/Metrics/Boundaries"
 */
export declare function exponential(start: number, factor: number, count: number): HistogramBoundaries;
/**
 * @tsplus static effect/core/io/Metrics/Histogram/Boundaries.Ops isHistogramBoundaries
 * @tsplus location "@effect/core/io/Metrics/Boundaries"
 */
export declare function isHistogramBoundaries(u: unknown): u is HistogramBoundaries;
//# sourceMappingURL=Boundaries.d.ts.map