"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MetricKeySym = exports.MetricKey = void 0;
exports.counter = counter;
exports.frequency = frequency;
exports.gauge = gauge;
exports.histogram = histogram;
exports.isMetricKey = void 0;
exports.summary = summary;
exports.tagged = tagged;
exports.taggedWithLabelSet = void 0;
exports.taggedWithLabels = taggedWithLabels;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/empty"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Hash"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/structure/Equals"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/equals"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricKeyType"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/definition"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricLabel"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/make"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/from"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/size"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/union"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

const taggedWithLabelSet = taggedWithLabelSet_1;
exports.taggedWithLabelSet = taggedWithLabelSet;
const isMetricKey = isMetricKey_1;
exports.isMetricKey = isMetricKey;
const MetricKeySym = /*#__PURE__*/Symbol.for("@effect/core/io/Metric/MetricKey");
/**
 * A `MetricKey` is a unique key associated with each metric. The key is based
 * on a combination of the metric type, the name and tags associated with the
 * metric, and any other information to describe a a metric, such as the
 * boundaries of a histogram. In this way, it is impossible to ever create
 * different metrics with conflicting keys.
 *
 * @tsplus type effect/core/io/Metrics/MetricKey
 * @tsplus companion effect/core/io/Metrics/MetricKey.Ops
 */

exports.MetricKeySym = MetricKeySym;

class MetricKey {
  constructor(name, keyType, tags = tsplus_module_1.empty()) {
    this.name = name;
    this.keyType = keyType;
    this.tags = tags;
    this[_a] = MetricKeySym;
  }

  [(_a = MetricKeySym, tsplus_module_2.Hash.sym)]() {
    return tsplus_module_2.combine(tsplus_module_2.hashString(this.name), tsplus_module_2.combine(tsplus_module_2.hashUnknown(this.keyType), tsplus_module_2.hashUnknown(this.tags)));
  }

  [tsplus_module_3.Equals.sym](u) {
    return isMetricKey_1(u) && this.name === u.name && tsplus_module_2.hashUnknown(this.keyType) === tsplus_module_2.hashUnknown(u.keyType) && tsplus_module_4.equals(u.tags)(this.tags);
  }

}
/**
 * Creates a metric key for a counter, with the specified name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Counter
 */


exports.MetricKey = MetricKey;

function counter(name) {
  return new MetricKey(name, tsplus_module_5.counter);
}
/**
 * Creates a metric key for a gauge, with the specified name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Gauge
 */


function gauge(name) {
  return new MetricKey(name, tsplus_module_5.gauge);
}
/**
 * Creates a metric key for a categorical frequency table, with the specified
 * name.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Frequency
 */


function frequency(name) {
  return new MetricKey(name, tsplus_module_5.frequency);
}
/**
 * Creates a metric key for a histogram, with the specified name and boundaries.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Histogram
 */


function histogram(name, boundaries) {
  return new MetricKey(name, tsplus_module_5.histogram(boundaries));
}
/**
 * Creates a metric key for a histogram, with the specified name, maxAge,
 * maxSize, error, and quantiles.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops Summary
 */


function summary(name, maxAge, maxSize, error, quantiles) {
  return new MetricKey(name, tsplus_module_5.summary(maxAge, maxSize, error, quantiles));
}
/**
 * Returns a new `MetricKey` with the specified tag appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects tagged
 * @tsplus pipeable effect/core/io/Metrics/MetricKey tagged
 */


function tagged(key, value) {
  return self => taggedWithLabelSet_1(tsplus_module_8.make(tsplus_module_7.make(key, value)))(self);
}
/**
 * Returns a new `MetricKey` with the specified tags appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects taggedWithLabels
 * @tsplus pipeable effect/core/io/Metrics/MetricKey taggedWithLabels
 */


function taggedWithLabels(extraTags) {
  return self => taggedWithLabelSet_1(tsplus_module_9.from(extraTags))(self);
}
/**
 * Returns a new `MetricKey` with the specified tags appended.
 *
 * @tsplus static effect/core/io/Metrics/MetricKey.Aspects taggedWithLabelSet
 * @tsplus pipeable effect/core/io/Metrics/MetricKey taggedWithLabelSet
 */


function taggedWithLabelSet_1(extraTags) {
  return self => tsplus_module_10.size(extraTags) === 0 ? self : new MetricKey(self.name, self.keyType, tsplus_module_11.union(extraTags)(self.tags));
}
/**
 * @tsplus static effect/core/io/Metrics/MetricKey.Ops isMetricKey
 */


function isMetricKey_1(u) {
  return typeof u === "object" && u != null && MetricKeySym in u;
}
//# sourceMappingURL=MetricKey.js.map