import { Equals } from "@tsplus/stdlib/structure/Equals";
import { Hash } from "@tsplus/stdlib/structure/Hash";
import { MetricState } from "@effect/core/io/Metrics/MetricState";
import { Metric } from "@effect/core/io/Metrics/definition";
import { Duration } from "@tsplus/stdlib/data/Duration";
import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
export declare const MetricKeyTypeSym: unique symbol;
export type MetricKeyTypeSym = typeof MetricKeyTypeSym;
export declare const _In: unique symbol;
export type _In = typeof _In;
export declare const _Out: unique symbol;
export type _Out = typeof _Out;
export declare const CounterKeySym: unique symbol;
export type CounterKeySym = typeof CounterKeySym;
export declare const FrequencyKeySym: unique symbol;
export type FrequencyKeySym = typeof FrequencyKeySym;
export declare const GaugeKeySym: unique symbol;
export type GaugeKeySym = typeof GaugeKeySym;
export declare const HistogramKeySym: unique symbol;
export type HistogramKeySym = typeof HistogramKeySym;
export declare const SummaryKeySym: unique symbol;
export type SummaryKeySym = typeof SummaryKeySym;
/**
 * @tsplus type effect/core/io/Metrics/MetricKeyType
 */
export interface MetricKeyType<In, Out> extends Equals {
    readonly [MetricKeyTypeSym]: MetricKeyTypeSym;
    readonly [_In]: () => In;
    readonly [_Out]: () => Out;
}
export declare namespace MetricKeyType {
    type Untyped = MetricKeyType<any, any>;
    type Counter = CounterKey;
    type Gauge = GaugeKey;
    type Frequency = FrequencyKey;
    type Histogram = HistogramKey;
    type Summary = SummaryKey;
}
export declare abstract class BaseMetricKeyType<In, Out> implements MetricKeyType<In, Out> {
    readonly [MetricKeyTypeSym]: MetricKeyTypeSym;
    readonly [_In]: () => In;
    readonly [_Out]: () => Out;
    abstract [Hash.sym](): number;
    abstract [Equals.sym](u: unknown): boolean;
}
export declare class CounterKey extends BaseMetricKeyType<number, MetricState.Counter> {
    readonly [CounterKeySym]: CounterKeySym;
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
export declare class GaugeKey extends BaseMetricKeyType<number, MetricState.Gauge> {
    readonly [GaugeKeySym]: GaugeKeySym;
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
export declare class FrequencyKey extends BaseMetricKeyType<string, MetricState.Frequency> {
    readonly [FrequencyKeySym]: FrequencyKeySym;
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
export declare class HistogramKey extends BaseMetricKeyType<number, MetricState.Histogram> {
    readonly boundaries: Metric.Histogram.Boundaries;
    readonly [HistogramKeySym]: HistogramKeySym;
    constructor(boundaries: Metric.Histogram.Boundaries);
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
export declare class SummaryKey extends BaseMetricKeyType<readonly [number, number], MetricState.Summary> {
    readonly maxAge: Duration;
    readonly maxSize: number;
    readonly error: number;
    readonly quantiles: Chunk<number>;
    readonly [SummaryKeySym]: SummaryKeySym;
    constructor(maxAge: Duration, maxSize: number, error: number, quantiles: Chunk<number>);
    [Hash.sym](): number;
    [Equals.sym](u: unknown): boolean;
}
/**
 * @tsplus type effect/core/io/Metrics/MetricKeyType.Ops
 */
export interface MetricKeyTypeOps {
}
export declare const MetricKeyType: MetricKeyTypeOps;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops Counter
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare const counter: MetricKeyType.Counter;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops Frequency
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare const frequency: MetricKeyType.Frequency;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops Gauge
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare const gauge: MetricKeyType.Gauge;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops Histogram
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function histogram(boundaries: Metric.Histogram.Boundaries): MetricKeyType.Histogram;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops Summary
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function summary(maxAge: Duration, maxSize: number, error: number, quantiles: Chunk<number>): MetricKeyType.Summary;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isMetricKeyType
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isMetricKeyType(u: unknown): u is MetricKeyType<unknown, unknown>;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isCounterKey
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isCounterKey(u: unknown): u is CounterKey;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isGaugeKey
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isGaugeKey(u: unknown): u is GaugeKey;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isFrequencyKey
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isFrequencyKey(u: unknown): u is FrequencyKey;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isHistogramKey
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isHistogramKey(u: unknown): u is HistogramKey;
/**
 * @tsplus static effect/core/io/Metrics/MetricKeyType.Ops isSummaryKey
 * @tsplus location "@effect/core/io/Metrics/MetricKeyType"
 */
export declare function isSummaryKey(u: unknown): u is SummaryKey;
//# sourceMappingURL=MetricKeyType.d.ts.map