"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MetricRegistry = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/mutable/MutableHashMap/empty"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricListener"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricPair"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/from"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/toUndefined"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricHooks"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Metrics/MetricHook"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/mutable/MutableHashMap/has"));

var _MetricKeyType = /*#__PURE__*/require("@effect/core/io/Metrics/MetricKeyType");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class MetricRegistry {
  constructor() {
    this.map = tsplus_module_1.empty();
    this.listeners = new Set();
    this.listener = new tsplus_module_2.MetricListener(key => update => {
      for (const listener of this.listeners) {
        listener.unsafeUpdate(key)(update);
      }
    });
  }

  installListener(listener) {
    this.listeners.add(listener);
  }

  removeListener(listener) {
    this.listeners.delete(listener);
  }

  snapshot() {
    const result = [];

    for (const [key, hook] of this.map) {
      result.push(tsplus_module_3.unsafeMake(key, hook.get()));
    }

    return tsplus_module_4.from(result);
  }

  get(key) {
    const hook = tsplus_module_5.toUndefined(this.map.get(key));

    if (hook == null) {
      if (key.keyType instanceof _MetricKeyType.CounterKey) {
        return this.getCounter(key);
      }

      if (key.keyType instanceof _MetricKeyType.GaugeKey) {
        return this.getGauge(key);
      }

      if (key.keyType instanceof _MetricKeyType.FrequencyKey) {
        return this.getFrequency(key);
      }

      if (key.keyType instanceof _MetricKeyType.HistogramKey) {
        return this.getHistogram(key);
      }

      if (key.keyType instanceof _MetricKeyType.SummaryKey) {
        return this.getSummary(key);
      }

      throw new Error("Bug, unknown MetricKeyType");
    } else {
      return hook;
    }
  }

  getCounter(key) {
    let value = tsplus_module_5.toUndefined(this.map.get(key));

    if (value == null) {
      const updater = this.listener.unsafeUpdate(key);
      const counter = tsplus_module_7.onUpdate(updater)(tsplus_module_6.counter(key));

      if (!tsplus_module_8.has(key)(this.map)) {
        this.map.set(key, counter);
      }

      value = counter;
    }

    return value;
  }

  getGauge(key) {
    let value = tsplus_module_5.toUndefined(this.map.get(key));

    if (value == null) {
      const updater = this.listener.unsafeUpdate(key);
      const gauge = tsplus_module_7.onUpdate(updater)(tsplus_module_6.gauge(key, 0));

      if (!tsplus_module_8.has(key)(this.map)) {
        this.map.set(key, gauge);
      }

      value = gauge;
    }

    return value;
  }

  getFrequency(key) {
    let value = tsplus_module_5.toUndefined(this.map.get(key));

    if (value == null) {
      const updater = this.listener.unsafeUpdate(key);
      const frequency = tsplus_module_7.onUpdate(updater)(tsplus_module_6.frequency(key));

      if (!tsplus_module_8.has(key)(this.map)) {
        this.map.set(key, frequency);
      }

      value = frequency;
    }

    return value;
  }

  getHistogram(key) {
    let value = tsplus_module_5.toUndefined(this.map.get(key));

    if (value == null) {
      const updater = this.listener.unsafeUpdate(key);
      const histogram = tsplus_module_7.onUpdate(updater)(tsplus_module_6.histogram(key));

      if (!tsplus_module_8.has(key)(this.map)) {
        this.map.set(key, histogram);
      }

      value = histogram;
    }

    return value;
  }

  getSummary(key) {
    let value = tsplus_module_5.toUndefined(this.map.get(key));

    if (value == null) {
      const updater = this.listener.unsafeUpdate(key);
      const summary = tsplus_module_7.onUpdate(updater)(tsplus_module_6.summary(key));

      if (!tsplus_module_8.has(key)(this.map)) {
        this.map.set(key, summary);
      }

      value = summary;
    }

    return value;
  }

}

exports.MetricRegistry = MetricRegistry;
//# sourceMappingURL=MetricRegistry.js.map