"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.live = exports.defaultRandom = exports.LiveRandom = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/io/Random/PCGRandom"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/sync"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/map"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/excl-forEach"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/suspendSucceed"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Random/definition"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Layer/operations/fromValue"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a;

class LiveRandom {
  constructor(seed) {
    this.seed = seed;
    this[_a] = tsplus_module_7.RandomSym;
    this.PRNG = new tsplus_module_1.PCGRandom(seed);
  }

  get next() {
    return tsplus_module_2.sync(() => this.PRNG.number());
  }

  get nextBoolean() {
    return tsplus_module_3.map(n => n > 0.5)(this.next);
  }

  get nextInt() {
    return tsplus_module_2.sync(() => this.PRNG.integer(0));
  }

  nextRange(low, high) {
    return tsplus_module_3.map(n => (high - low) * n + low)(this.next);
  }

  nextIntBetween(low, high) {
    return tsplus_module_2.sync(() => this.PRNG.integer(high - low) + low);
  }

  shuffle(collection) {
    return shuffleWith(collection, n => this.nextIntBetween(0, n));
  }

}

exports.LiveRandom = LiveRandom;
_a = tsplus_module_7.RandomSym;

function shuffleWith(collection, nextIntBounded) {
  return tsplus_module_6.suspendSucceed(() => {
    return tsplus_module_4.flatMap(buffer => {
      const swap = (i1, i2) => tsplus_module_2.sync(() => {
        const tmp = buffer[i1];
        buffer[i1] = buffer[i2];
        buffer[i2] = tmp;
        return buffer;
      });

      const ns = [];

      for (let i = buffer.length; i >= 2; i = i - 1) {
        ns.push(i);
      }

      return tsplus_module_3.map(() => buffer)(tsplus_module_5.forEachDiscard(ns, n => tsplus_module_4.flatMap(k => swap(n - 1, k))(nextIntBounded(n))));
    })(tsplus_module_2.sync(() => {
      const buffer = [];

      for (const element of collection) {
        buffer.push(element);
      }

      return buffer;
    }));
  });
}
/**
 * @tsplus static effect/core/io/Random.Ops default
 */


const defaultRandom_1 = /*#__PURE__*/new LiveRandom( /*#__PURE__*/Math.random() * 4294967296 >>> 0);
const defaultRandom = defaultRandom_1;
/**
 * @tsplus static effect/core/io/Random.Ops live
 */

exports.defaultRandom = defaultRandom;
const live = /*#__PURE__*/tsplus_module_8.fromValue(tsplus_module_7.Random.Tag, () => defaultRandom_1);
exports.live = live;
//# sourceMappingURL=live.js.map