"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UnsafeAPI = exports.AtomicInternal = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/getOrElse"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/sync"));

var _definition2 = /*#__PURE__*/require("@effect/core/io/Ref/definition");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class UnsafeAPI {
  constructor(value) {
    this.value = value;
  }

  get get() {
    return this.value.get;
  }

  getAndSet(a) {
    const current = this.value.get;
    this.value.set(a);
    return current;
  }

  getAndUpdate(f) {
    const current = this.value.get;
    this.value.set(f(current));
    return current;
  }

  getAndUpdateSome(pf) {
    const current = this.value.get;
    const opt = pf(current);

    if (tsplus_module_1.isSome(opt)) {
      this.value.set(opt.value);
    }

    return current;
  }

  modify(f) {
    const current = this.value.get;
    const [b, a] = f(current);
    this.value.set(a);
    return b;
  }

  modifySome(fallback, pf) {
    const current = this.value.get;
    const tuple = tsplus_module_2.getOrElse_(pf(current), () => [fallback, current]);
    this.value.set(tuple[1]);
    return tuple[0];
  }

  set(a) {
    return this.value.set(a);
  }

  update(f) {
    const current = this.value.get;
    this.value.set(f(current));
  }

  updateAndGet(f) {
    const current = this.value.get;
    const next = f(current);
    this.value.set(next);
    return next;
  }

  updateSome(pf) {
    const current = this.value.get;
    const opt = pf(current);

    if (tsplus_module_1.isSome(opt)) {
      this.value.set(opt.value);
    }
  }

  updateSomeAndGet(pf) {
    const current = this.value.get;
    const next = pf(current);

    if (tsplus_module_1.isSome(next)) {
      this.value.set(next.value);
      return next.value;
    }

    return current;
  }

}

exports.UnsafeAPI = UnsafeAPI;

class AtomicInternal {
  constructor(unsafe) {
    this.unsafe = unsafe;
  }

  get [_definition2._A]() {
    return a => a;
  }
  /**
   * Internal Discriminator
   */


  get [_definition2.RefSym]() {
    return _definition2.RefSym;
  }

  get get() {
    return tsplus_module_3.sync(() => this.unsafe.get);
  }

  modify(f) {
    return tsplus_module_3.sync(() => this.unsafe.modify(f));
  }

  set(a) {
    return tsplus_module_3.sync(() => this.unsafe.set(a));
  }

  getAndSet(a) {
    return this.modify(v => [v, a]);
  }

  getAndUpdate(f) {
    return this.modify(v => [v, f(v)]);
  }

  getAndUpdateSome(pf) {
    return this.modify(v => [v, tsplus_module_2.getOrElse_(pf(v), () => v)]);
  }

  modifySome(fallback, pf) {
    return this.modify(v => tsplus_module_2.getOrElse_(pf(v), () => [fallback, v]));
  }

  update(f) {
    return this.modify(v => [undefined, f(v)]);
  }

  updateAndGet(f) {
    return this.modify(v => {
      const result = f(v);
      return [result, result];
    });
  }

  updateSome(pf) {
    return this.modify(v => [undefined, tsplus_module_2.getOrElse_(pf(v), () => v)]);
  }

  updateSomeAndGet(pf) {
    return this.modify(v => {
      const result = tsplus_module_2.getOrElse_(pf(v), () => v);
      return [result, result];
    });
  }

}

exports.AtomicInternal = AtomicInternal;
//# sourceMappingURL=AtomicInternal.js.map