"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Runtime = exports.AsyncFiber = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/FiberId/operations/make"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/_internal/runtime"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Supervisor/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/FiberScope/definition"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/operations/interruptAs"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/flatten"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Function"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Cause/operations/squashWith"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/die"));

var _Scheduler = /*#__PURE__*/require("@effect/core/support/Scheduler");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class AsyncFiber {
  constructor(fiber) {
    this.fiber = fiber;
    this._tag = "AsyncFiber";
  }

}

exports.AsyncFiber = AsyncFiber;

class Runtime {
  constructor(environment, runtimeFlags, fiberRefs) {
    this.environment = environment;
    this.runtimeFlags = runtimeFlags;
    this.fiberRefs = fiberRefs;

    this.unsafeFork = effect => {
      const fiberId = tsplus_module_1.unsafeMake();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment);
      const context = new tsplus_module_2.FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      return context;
    };

    this.unsafeRunWith = (effect, k) => {
      const fiberId = tsplus_module_1.unsafeMake();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment);
      const context = new tsplus_module_2.FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      context.addObserver(exit => {
        k(exit);
      });
      return id => k => this.unsafeRunAsyncWith(tsplus_module_6.interruptAsNow(id)(context), exit => k(tsplus_module_7.flatten(exit)));
    };

    this.unsafeRunSync = effect => {
      const exit = this.unsafeRunSyncExit(effect);

      if (exit._tag === "Failure") {
        throw tsplus_module_9.squashWith(tsplus_module_8.identity)(exit.cause);
      }

      return exit.value;
    };

    this.unsafeRunSyncExit = effect => {
      const fiberId = tsplus_module_1.unsafeMake();
      const scheduler = new _Scheduler.StagedScheduler();
      const fiberRefs = this.fiberRefs.updateAs(fiberId, tsplus_module_2.currentEnvironment, this.environment).updateAs(fiberId, tsplus_module_2.currentScheduler, scheduler);
      const context = new tsplus_module_2.FiberRuntime(fiberId, fiberRefs, this.runtimeFlags);
      const supervisor = context.getSupervisor;

      if (supervisor != tsplus_module_3.none) {
        supervisor.onStart(this.environment, effect, tsplus_module_4.none, context);
        context.addObserver(exit => supervisor.onEnd(exit, context));
      }

      context.start(effect);
      tsplus_module_5.globalScope.add(this.runtimeFlags, context);
      scheduler.flush();
      const result = context.unsafePoll;

      if (result) {
        return result;
      }

      return tsplus_module_10.die(new AsyncFiber(context));
    };
    /**
     * Executes the effect asynchronously, discarding the result of execution.
     *
     * This method is effectful and should only be invoked at the edges of your
     * program.
     */


    this.unsafeRunAsync = effect => {
      return this.unsafeRunAsyncWith(effect, tsplus_module_8.constVoid);
    };
    /**
     * Executes the effect asynchronously, eventually passing the exit value to
     * the specified callback.
     *
     * This method is effectful and should only be invoked at the edges of your
     * program.
     */


    this.unsafeRunAsyncWith = (effect, k) => {
      this.unsafeRunWith(effect, k);
    };
    /**
     * Runs the `Effect`, returning a JavaScript `Promise` that will be resolved
     * with the value of the effect once the effect has been executed, or will be
     * rejected with the first error or exception throw by the effect.
     *
     * This method is effectful and should only be used at the edges of your
     * program.
     */


    this.unsafeRunPromise = effect => {
      return new Promise((resolve, reject) => {
        this.unsafeRunAsyncWith(effect, exit => {
          switch (exit._tag) {
            case "Success":
              {
                resolve(exit.value);
                break;
              }

            case "Failure":
              {
                reject(tsplus_module_9.squashWith(tsplus_module_8.identity)(exit.cause));
                break;
              }
          }
        });
      });
    };
    /**
     * Runs the `Effect`, returning a JavaScript `Promise` that will be resolved
     * with the `Exit` state of the effect once the effect has been executed.
     *
     * This method is effectful and should only be used at the edges of your
     * program.
     */


    this.unsafeRunPromiseExit = effect => {
      return new Promise(resolve => {
        this.unsafeRunAsyncWith(effect, exit => {
          resolve(exit);
        });
      });
    };
  }

}

exports.Runtime = Runtime;
//# sourceMappingURL=definition.js.map