"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.IntervalSym = exports.Interval = void 0;
exports.after = after;
exports.before = before;
exports.fromStartEndMillis = exports.empty = void 0;
exports.intersect = intersect;
exports.isEmpty = void 0;
exports.isNonEmpty = isNonEmpty;
exports.lessThan = lessThan;
exports.max = max;
exports.min = void 0;
exports.size = size;
exports.union = union;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/Ord/common"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/prelude/Ord/operations"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var _Duration = /*#__PURE__*/require("@tsplus/stdlib/data/Duration");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const fromStartEndMillis = fromStartEndMillis_1;
exports.fromStartEndMillis = fromStartEndMillis;
const min = min_1;
exports.min = min;
const isEmpty = isEmpty_1;
exports.isEmpty = isEmpty;
const IntervalSym = /*#__PURE__*/Symbol.for("@effect/core/io/Schedule/Interval");
exports.IntervalSym = IntervalSym;
const Interval = {
  $: {}
};
/**
 * An `Interval` of zero-width.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops empty
 */

exports.Interval = Interval;
const empty_1 = /*#__PURE__*/fromStartEndMillis_1(0, 0);
const empty = empty_1;
/**
 * Constructs a new interval from the two specified endpoints. If the start
 * endpoint greater than the end endpoint, then a zero size interval will be
 * returned.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops __call
 */

exports.empty = empty;

function fromStartEndMillis_1(startMilliseconds, endMilliseconds) {
  return startMilliseconds > endMilliseconds ? empty_1 : {
    [IntervalSym]: IntervalSym,
    startMillis: startMilliseconds,
    endMillis: endMilliseconds
  };
}
/**
 * @tsplus pipeable-operator effect/core/io/Schedule/Interval <
 * @tsplus static effect/core/io/Schedule/Interval.Aspects lessThan
 * @tsplus pipeable effect/core/io/Schedule/Interval lessThan
 */


function lessThan(that) {
  return self => min_1(that)(self) === self;
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects min
 * @tsplus pipeable effect/core/io/Schedule/Interval min
 */


function min_1(that) {
  return self => {
    if (self.endMillis <= that.startMillis) return self;
    if (that.endMillis <= self.startMillis) return that;
    if (self.startMillis < that.startMillis) return self;
    if (that.startMillis < self.startMillis) return that;
    if (self.endMillis <= that.endMillis) return self;
    return that;
  };
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects max
 * @tsplus pipeable effect/core/io/Schedule/Interval max
 */


function max(that) {
  return self => min_1(that)(self) === self ? that : self;
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval isEmpty
 */


function isEmpty_1(self) {
  return self.startMillis >= self.endMillis;
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval isNonEmpty
 */


function isNonEmpty(self) {
  return !isEmpty_1(self);
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects intersect
 * @tsplus pipeable effect/core/io/Schedule/Interval intersect
 */


function intersect(that) {
  return self => {
    const start = Math.max(self.startMillis, that.startMillis);
    const end = Math.min(self.endMillis, that.endMillis);
    return fromStartEndMillis_1(start, end);
  };
}
/**
 * @tsplus getter effect/core/io/Schedule/Interval size
 */


function size(self) {
  return new _Duration.DurationInternal(self.endMillis - self.startMillis);
}
/**
 * @tsplus static effect/core/io/Schedule/Interval.Aspects union
 * @tsplus pipeable effect/core/io/Schedule/Interval union
 */


function union(that) {
  return self => {
    const start = tsplus_module_2.max(tsplus_module_1.number)(self.startMillis, that.startMillis);
    const end = tsplus_module_2.min(tsplus_module_1.number)(self.endMillis, that.endMillis);
    return start < end ? tsplus_module_3.none : tsplus_module_3.some(fromStartEndMillis_1(start, end));
  };
}
/**
 * Construct an `Interval` that includes all time equal to and after the
 * specified start time.
 *
 * @tsplus static effect/core/io/Schedule/Interval.Ops after
 */


function after(startMilliseconds) {
  return fromStartEndMillis_1(startMilliseconds, Number.MAX_SAFE_INTEGER);
}
/**
 * Construct an `Interval` that includes all time equal to and before the
 * specified end time.
 *
 * @tsplus static effect/core/io/Schedule/Interval before
 */


function before(endMilliseconds) {
  return fromStartEndMillis_1(Number.MIN_SAFE_INTEGER, endMilliseconds);
}
//# sourceMappingURL=Interval.js.map