"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.empty = exports.IntervalsSym = exports.Intervals = void 0;
exports.end = end;
exports.fromIntervals = fromIntervals;
exports.intersect = intersect;
exports.isNonEmpty = isNonEmpty;
exports.make = exports.lessThan = void 0;
exports.max = max;
exports.union = exports.start = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/empty"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/reverse"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Interval"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/head"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/getOrElse"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const make = make_1;
exports.make = make;
const union = union_1;
exports.union = union;
const start = start_1;
exports.start = start;
const lessThan = lessThan_1;
exports.lessThan = lessThan;
const IntervalsSym = /*#__PURE__*/Symbol.for("@effect/core/io/Schedule/Intervals");
exports.IntervalsSym = IntervalsSym;
const Intervals = {
  $: {}
};
/**
 * @tsplus static effect/core/io/Schedule/Intervals.Ops __call
 * @tsplus static effect/core/io/Schedule/Intervals.Ops make
 */

exports.Intervals = Intervals;

function make_1(intervals) {
  return {
    [IntervalsSym]: IntervalsSym,
    intervals
  };
}
/**
 * The empty set of intervals.
 *
 * @tsplus static effect/core/io/Schedule/Intervals.Ops empty
 */


const empty_1 = /*#__PURE__*/make_1( /*#__PURE__*/tsplus_module_1.empty());
const empty = empty_1;
/**
 * Constructs a set of intervals from the specified intervals.
 *
 * @tsplus static effect/core/io/Schedule/Intervals.Ops fromIntervals
 */

exports.empty = empty;

function fromIntervals(...intervals) {
  return intervals.reduce((intervals, interval) => union_1(make_1(tsplus_module_2.make(interval)))(intervals), empty_1);
}
/**
 * Produces the union of this set of intervals and the specified set of intervals.
 *
 * @tsplus pipeable-operator effect/core/io/Schedule/Intervals ||
 * @tsplus static effect/core/io/Schedule/Intervals.Aspects union
 * @tsplus pipeable effect/core/io/Schedule/Intervals union
 */


function union_1(that) {
  return self => {
    if (tsplus_module_3.isNil(that.intervals)) {
      return self;
    }

    if (tsplus_module_3.isNil(self.intervals)) {
      return that;
    } // const { head: left, tail: lefts } = self.intervals
    // const { head: right, tail: rights } = that.intervals


    if (self.intervals.head.startMillis < that.intervals.head.startMillis) {
      return unionLoop(self.intervals.tail, that.intervals, self.intervals.head, tsplus_module_3.nil());
    }

    return unionLoop(self.intervals, that.intervals.tail, that.intervals.head, tsplus_module_3.nil());
  };
}
/**
 * @tsplus tailRec
 */


function unionLoop(self, that, interval, acc) {
  var self_1 = self,
      that_1 = that,
      interval_1 = interval,
      acc_1 = acc;
  var self_2 = self,
      that_2 = that,
      interval_2 = interval,
      acc_2 = acc;

  while (1) {
    switch (self_1._tag) {
      case "Nil":
        {
          switch (that_1._tag) {
            case "Nil":
              {
                return make_1(tsplus_module_5.reverse(tsplus_module_4.prepend(interval_1)(acc_1)));
              }

            case "Cons":
              {
                // const { head: right, tail: rights } = that
                if (interval_1.endMillis < that_1.head.startMillis) {
                  self_2 = tsplus_module_3.nil();
                  that_2 = that_1.tail;
                  interval_2 = that_1.head;
                  acc_2 = tsplus_module_4.prepend(interval_1)(acc_1);
                  self_1 = self_2;
                  that_1 = that_2;
                  interval_1 = interval_2;
                  acc_1 = acc_2;
                  continue;
                }

                self_2 = tsplus_module_3.nil();
                that_2 = that_1.tail;
                interval_2 = tsplus_module_6.fromStartEndMillis(interval_1.startMillis, that_1.head.endMillis);
                acc_2 = acc_1;
                self_1 = self_2;
                that_1 = that_2;
                interval_1 = interval_2;
                acc_1 = acc_2;
                continue;
              }
          }
        }

      case "Cons":
        {
          switch (that_1._tag) {
            case "Nil":
              {
                // const { head: left, tail: lefts } = self
                if (interval_1.endMillis < self_1.head.startMillis) {
                  self_2 = self_1.tail;
                  that_2 = tsplus_module_3.nil();
                  interval_2 = self_1.head;
                  acc_2 = tsplus_module_4.prepend(interval_1)(acc_1);
                  self_1 = self_2;
                  that_1 = that_2;
                  interval_1 = interval_2;
                  acc_1 = acc_2;
                  continue;
                }

                self_2 = self_1.tail;
                that_2 = tsplus_module_3.nil();
                interval_2 = tsplus_module_6.fromStartEndMillis(interval_1.startMillis, self_1.head.endMillis);
                acc_2 = acc_1;
                self_1 = self_2;
                that_1 = that_2;
                interval_1 = interval_2;
                acc_1 = acc_2;
                continue;
              }

            case "Cons":
              {
                // const { head: left, tail: lefts } = self
                // const { head: right, tail: rights } = that
                if (self_1.head.startMillis < that_1.head.startMillis) {
                  if (interval_1.endMillis < self_1.head.startMillis) {
                    self_2 = self_1.tail;
                    that_2 = that_1;
                    interval_2 = self_1.head;
                    acc_2 = tsplus_module_4.prepend(interval_1)(acc_1);
                    self_1 = self_2;
                    that_1 = that_2;
                    interval_1 = interval_2;
                    acc_1 = acc_2;
                    continue;
                  }

                  self_2 = self_1.tail;
                  that_2 = that_1;
                  interval_2 = tsplus_module_6.fromStartEndMillis(interval_1.startMillis, self_1.head.endMillis);
                  acc_2 = acc_1;
                  self_1 = self_2;
                  that_1 = that_2;
                  interval_1 = interval_2;
                  acc_1 = acc_2;
                  continue;
                }

                if (interval_1.endMillis < that_1.head.startMillis) {
                  self_2 = self_1;
                  that_2 = that_1.tail;
                  interval_2 = that_1.head;
                  acc_2 = tsplus_module_4.prepend(interval_1)(acc_1);
                  self_1 = self_2;
                  that_1 = that_2;
                  interval_1 = interval_2;
                  acc_1 = acc_2;
                  continue;
                }

                self_2 = self_1;
                that_2 = that_1.tail;
                interval_2 = tsplus_module_6.fromStartEndMillis(interval_1.startMillis, that_1.head.endMillis);
                acc_2 = acc_1;
                self_1 = self_2;
                that_1 = that_2;
                interval_1 = interval_2;
                acc_1 = acc_2;
                continue;
              }
          }
        }
    }
  }
}
/**
 * Produces the intersection of this set of intervals and the specified set of intervals.
 *
 * @tsplus pipeable-operator effect/core/io/Schedule/Intervals &&
 * @tsplus static effect/core/io/Schedule/Intervals.Aspects intersect
 * @tsplus pipeable effect/core/io/Schedule/Intervals intersect
 */


function intersect(that) {
  return self => intersectLoop(self.intervals, that.intervals, tsplus_module_3.nil());
}
/**
 * @tsplus tailRec
 */


function intersectLoop(left, right, acc) {
  var left_1 = left,
      right_1 = right,
      acc_3 = acc;
  var left_2 = left,
      right_2 = right,
      acc_4 = acc;

  while (1) {
    if (tsplus_module_3.isNil(left_1) || tsplus_module_3.isNil(right_1)) {
      return make_1(tsplus_module_5.reverse(acc_3));
    }

    const interval = tsplus_module_6.intersect(right_1.head)(left_1.head);
    const intervals = tsplus_module_6.isEmpty(interval) ? acc_3 : tsplus_module_4.prepend(interval)(acc_3);

    if (tsplus_module_6.lessThan(right_1.head)(left_1.head)) {
      left_2 = left_1.tail;
      right_2 = right_1;
      acc_4 = intervals;
      left_1 = left_2;
      right_1 = right_2;
      acc_3 = acc_4;
      continue;
    }

    left_2 = left_1;
    right_2 = right_1.tail;
    acc_4 = intervals;
    left_1 = left_2;
    right_1 = right_2;
    acc_3 = acc_4;
    continue;
  }
}
/**
 * The start of the earliest interval in this set.
 *
 * @tsplus static effect/core/io/Schedule/Intervals.Ops start
 * @tsplus getter effect/core/io/Schedule/Intervals start
 */


function start_1(self) {
  return tsplus_module_8.getOrElse_(tsplus_module_7.head(self.intervals), () => tsplus_module_6.empty).startMillis;
}
/**
 * The end of the latest interval in this set.
 *
 * @tsplus static effect/core/io/Schedule/Intervals.Ops end
 * @tsplus getter effect/core/io/Schedule/Intervals end
 */


function end(self) {
  return tsplus_module_8.getOrElse_(tsplus_module_7.head(self.intervals), () => tsplus_module_6.empty).endMillis;
}
/**
 * Whether the start of this set of intervals is before the start of the
 * specified set of intervals.
 *
 * @tsplus pipeable-operator effect/core/io/Schedule/Intervals <
 * @tsplus static effect/core/io/Schedule/Intervals.Aspects lessThan
 * @tsplus pipeable effect/core/io/Schedule/Intervals lessThan
 */


function lessThan_1(that) {
  return self => start_1(self) < start_1(that);
}
/**
 * Returns whether this set of intervals is empty.
 *
 * @tsplus getter effect/core/io/Schedule/Intervals isNonEmpty
 */


function isNonEmpty(self) {
  return tsplus_module_3.isCons(self.intervals);
}
/**
 * The set of intervals that starts last.
 *
 * @tsplus static effect/core/io/Schedule/Intervals.Aspects max
 * @tsplus pipeable effect/core/io/Schedule/Intervals max
 */


function max(that) {
  return self => lessThan_1(that)(self) ? that : self;
}
//# sourceMappingURL=Intervals.js.map