"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.intersectWith = intersectWith;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Decision"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipWith"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Intervals"));

var _makeWithState = /*#__PURE__*/require("@effect/core/io/Schedule/operations/_internal/makeWithState");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Returns a new schedule that combines this schedule with the specified
 * schedule, continuing as long as both schedules want to continue and merging
 * the next intervals according to the specified merge function.
 *
 * @tsplus static effect/core/io/Schedule.Aspects intersectWith
 * @tsplus pipeable effect/core/io/Schedule intersectWith
 */
function intersectWith(that, f) {
  return self => (0, _makeWithState.makeWithState)([self.initial, that.initial], (now, input, state) => {
    const left = self.step(now, input, state[0]);
    const right = that.step(now, input, state[1]);
    return tsplus_module_4.flatMap(([[lState, out, lDecision], [rState, out2, rDecision]]) => {
      if (lDecision._tag === "Continue" && rDecision._tag === "Continue") {
        return intersectWithLoop(self, that, input, lState, out, lDecision.intervals, rState, out2, rDecision.intervals, f);
      }

      return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
    })(tsplus_module_3.zipWith(right, (a, b) => [a, b])(left));
  });
}

function intersectWithLoop(self, that, input, lState, out, lInterval, rState, out2, rInterval, f) {
  const combined = f(lInterval, rInterval);

  if (tsplus_module_5.isNonEmpty(combined)) {
    return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1._continue(combined)]);
  }

  if (tsplus_module_5.lessThan(rInterval)(lInterval)) {
    return tsplus_module_4.flatMap(([lState, out, decision]) => {
      switch (decision._tag) {
        case "Done":
          {
            return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
          }

        case "Continue":
          {
            return intersectWithLoop(self, that, input, lState, out, decision.intervals, rState, out2, rInterval, f);
          }
      }
    })(self.step(tsplus_module_5.end(lInterval), input, lState));
  }

  return tsplus_module_4.flatMap(([rState, out2, decision]) => {
    switch (decision._tag) {
      case "Done":
        {
          return tsplus_module_2.succeed([[lState, rState], [out, out2], tsplus_module_1.done]);
        }

      case "Continue":
        {
          return intersectWithLoop(self, that, input, lState, out, lInterval, rState, out2, decision.intervals, f);
        }
    }
  })(that.step(tsplus_module_5.end(rInterval), input, rState));
}
//# sourceMappingURL=intersectWith.js.map