"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.secondOfMinute = secondOfMinute;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Cause/errors"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/dieSync"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Interval"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Decision"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

var _makeWithState = /*#__PURE__*/require("@effect/core/io/Schedule/operations/_internal/makeWithState");

var _time = /*#__PURE__*/require("@effect/core/io/Schedule/operations/_internal/time");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Cron-like schedule that recurs every specified `second` of each minute. It
 * triggers at zero nanosecond of the second. Producing a count of repeats: 0,
 * 1, 2.
 *
 * NOTE: `second` parameter is validated lazily. Must be in range 0...59.
 *
 * @tsplus static effect/core/io/Schedule.Ops secondOfMinute
 */
function secondOfMinute(second) {
  return (0, _makeWithState.makeWithState)([Number.MIN_SAFE_INTEGER, 0], (now, _, state) => {
    if (!Number.isInteger(second) || second < 0 || 59 < second) {
      return tsplus_module_2.dieSync(() => new tsplus_module_1.IllegalArgumentException(`Invalid argument in: secondOfMinute(${second}). Must be in range 0...59`));
    }

    const [end0, n] = state;
    const now0 = Math.max(end0, now);
    const second0 = (0, _time.nextSecond)(now0, second);
    const start = Math.max((0, _time.beginningOfSecond)(second0), now0);
    const end = (0, _time.endOfSecond)(second0);
    const interval = tsplus_module_3.fromStartEndMillis(start, end);
    return tsplus_module_5.succeed([[end, n + 1], n, tsplus_module_4.continueWith(interval)]);
  });
}
//# sourceMappingURL=secondOfMinute.js.map