"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unionWith = unionWith;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Schedule/Decision"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipWith"));

var _makeWithState = /*#__PURE__*/require("@effect/core/io/Schedule/operations/_internal/makeWithState");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Returns a new schedule that combines this schedule with the specified
 * schedule, continuing as long as either schedule wants to continue and
 * merging the next intervals according to the specified merge function.
 *
 * @tsplus static effect/core/io/Schedule.Aspects unionWith
 * @tsplus pipeable effect/core/io/Schedule unionWith
 */
function unionWith(that, f) {
  return self => (0, _makeWithState.makeWithState)([self.initial, that.initial], (now, input, state) => {
    const left = self.step(now, input, state[0]);
    const right = that.step(now, input, state[1]);
    return tsplus_module_2.zipWith(right, ([lState, l, lDecision], [rState, r, rDecision]) => {
      if (lDecision._tag === "Done" && rDecision._tag === "Done") {
        return [[lState, rState], [l, r], tsplus_module_1.done];
      }

      if (lDecision._tag === "Done" && rDecision._tag === "Continue") {
        return [[lState, rState], [l, r], tsplus_module_1._continue(rDecision.intervals)];
      }

      if (lDecision._tag === "Continue" && rDecision._tag === "Done") {
        return [[lState, rState], [l, r], tsplus_module_1._continue(lDecision.intervals)];
      }

      if (lDecision._tag === "Continue" && rDecision._tag === "Continue") {
        const combined = f(lDecision.intervals, rDecision.intervals);
        return [[lState, rState], [l, r], tsplus_module_1._continue(combined)];
      }

      throw new Error("bug");
    })(left);
  });
}
//# sourceMappingURL=unionWith.js.map