"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Zip = exports.Track = exports.SupervisorURI = exports.Supervisor = exports.ProxySupervisor = exports.Const = void 0;
exports.fibersIn = fibersIn;
exports.unsafeTrack = exports.track = exports.none = exports.fromEffect = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/map"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zip"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/sync"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/unit"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/SortedSet/add"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/SortedSet/remove"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const unsafeTrack = unsafeTrack_1;
exports.unsafeTrack = unsafeTrack;
const fromEffect = fromEffect_1;
exports.fromEffect = fromEffect;
const SupervisorURI = /*#__PURE__*/Symbol.for("@effect/core/io/Supervisor");
/**
 * A `Supervisor<A>` is allowed to supervise the launching and termination of
 * fibers, producing some visible value of type `A` from the supervision.
 *
 * @tsplus type effect/core/io/Supervisor
 * @tsplus companion effect/core/io/Supervisor.Ops
 */

exports.SupervisorURI = SupervisorURI;

class Supervisor {
  /**
   * Maps this supervisor to another one, which has the same effect, but whose
   * value has been transformed by the specified function.
   */
  map(f) {
    return new ProxySupervisor(this, () => tsplus_module_1.map(f)(this.value));
  }
  /**
   * Returns a new supervisor that performs the function of this supervisor, and
   * the function of the specified supervisor, producing a tuple of the outputs
   * produced by both supervisors.
   */


  zip(right) {
    return new Zip(this, right);
  }

}

exports.Supervisor = Supervisor;

class ProxySupervisor extends Supervisor {
  constructor(underlying, value0) {
    super();
    this.underlying = underlying;
    this.value0 = value0;
  }

  get value() {
    return this.value0();
  }

  onStart(environment, effect, parent, fiber) {
    this.underlying.onStart(environment, effect, parent, fiber);
  }

  onEnd(value, fiber) {
    this.underlying.onEnd(value, fiber);
  }

  onEffect(fiber, effect) {
    this.underlying.onEffect(fiber, effect);
  }

  onSuspend(fiber) {
    this.underlying.onSuspend(fiber);
  }

  onResume(fiber) {
    this.underlying.onResume(fiber);
  }

}

exports.ProxySupervisor = ProxySupervisor;

class Zip extends Supervisor {
  constructor(left, right) {
    super();
    this.left = left;
    this.right = right;
  }

  get value() {
    return tsplus_module_2.zip(this.right.value)(this.left.value);
  }

  onStart(environment, effect, parent, fiber) {
    this.left.onStart(environment, effect, parent, fiber);
    this.right.onStart(environment, effect, parent, fiber);
  }

  onEnd(value, fiber) {
    this.left.onEnd(value, fiber);
    this.right.onEnd(value, fiber);
  }

  onEffect(fiber, effect) {
    this.left.onEffect(fiber, effect);
    this.right.onEffect(fiber, effect);
  }

  onSuspend(fiber) {
    this.left.onSuspend(fiber);
    this.right.onSuspend(fiber);
  }

  onResume(fiber) {
    this.left.onResume(fiber);
    this.right.onResume(fiber);
  }

}

exports.Zip = Zip;

class Track extends Supervisor {
  constructor() {
    super(...arguments);
    this.fibers = new Set();
  }

  get value() {
    return tsplus_module_4.sync(() => tsplus_module_3.from(this.fibers));
  }

  onStart(_environment, _effect, _parent, fiber) {
    this.fibers.add(fiber);
  }

  onEnd(_value, fiber) {
    this.fibers.delete(fiber);
  }

  onEffect(_fiber, _effect) {//
  }

  onSuspend(_fiber) {//
  }

  onResume(_fiber) {//
  }

}

exports.Track = Track;

class Const extends Supervisor {
  constructor(effect) {
    super();
    this.effect = effect;
  }

  get value() {
    return this.effect;
  }

  onStart(_environment, _effect, _parent, _fiber) {//
  }

  onEnd(_value, _fiber) {//
  }

  onEffect(_fiber, _effect) {//
  }

  onSuspend(_fiber) {//
  }

  onResume(_fiber) {//
  }

}
/**
 * Creates a new supervisor that tracks children in a set.
 *
 * @tsplus static effect/core/io/Supervisor.Ops unsafeTrack
 */


exports.Const = Const;

function unsafeTrack_1() {
  return new Track();
}
/**
 * Creates a new supervisor that tracks children in a set.
 *
 * @tsplus static effect/core/io/Supervisor.Ops track
 */


const track = /*#__PURE__*/tsplus_module_4.sync(unsafeTrack_1);
/**
 * Creates a new supervisor that constantly yields effect when polled
 *
 * @tsplus static effect/core/io/Supervisor.Ops fromEffect
 */

exports.track = track;

function fromEffect_1(effect) {
  return new Const(effect);
}
/**
 * A supervisor that doesn't do anything in response to supervision events.
 *
 * @tsplus static effect/core/io/Supervisor.Ops none
 */


const none = /*#__PURE__*/fromEffect_1(tsplus_module_5.unit);
exports.none = none;

class FibersIn extends Supervisor {
  constructor(ref) {
    super();
    this.ref = ref;
  }

  get value() {
    return tsplus_module_4.sync(() => this.ref.get);
  }

  onStart(_environment, _effect, _parent, fiber) {
    this.ref.set(tsplus_module_6.add(fiber)(this.ref.get));
  }

  onEnd(_value, fiber) {
    this.ref.set(tsplus_module_7.remove(fiber)(this.ref.get));
  }

  onEffect(_fiber, _effect) {//
  }

  onSuspend(_fiber) {//
  }

  onResume(_fiber) {//
  }

}
/**
 * Creates a new supervisor that tracks children in a set.
 *
 * @tsplus static effect/core/io/Supervisor.Ops fibersIn
 */


function fibersIn(ref) {
  return tsplus_module_4.sync(() => new FibersIn(ref));
}
//# sourceMappingURL=definition.js.map