"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combinePatch = exports.applyPatch = exports.RemoveSupervisor = exports.Empty = exports.AndThen = exports.AddSupervisor = void 0;
exports.diff = diff;
exports.emptyPatch = exports.differ = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/definition"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/from"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Supervisor/definition"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/empty"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/union"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/make"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/difference"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/HashSet/operations/reduce"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Differ/operations/make"));

var _Equals = /*#__PURE__*/require("@tsplus/stdlib/structure/Equals");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const combinePatch = combinePatch_1;
exports.combinePatch = combinePatch;
const applyPatch = applyPatch_1;
exports.applyPatch = applyPatch;

class AddSupervisor {
  constructor(supervisor) {
    this.supervisor = supervisor;
    this._tag = "AddSupervisor";
  }

}

exports.AddSupervisor = AddSupervisor;

class Empty {
  constructor() {
    this._tag = "Empty";
  }

}

exports.Empty = Empty;

class RemoveSupervisor {
  constructor(supervisor) {
    this.supervisor = supervisor;
    this._tag = "RemoveSupervisor";
  }

}

exports.RemoveSupervisor = RemoveSupervisor;

class AndThen {
  constructor(first, second) {
    this.first = first;
    this.second = second;
    this._tag = "AndThen";
  }

}
/**
 * Combines two patches to produce a new patch that describes applying the
 * updates from this patch and then the updates from the specified patch.
 *
 * @tsplus static effect/core/io/Supervisor.Ops combinePatch
 */


exports.AndThen = AndThen;

function combinePatch_1(self, that) {
  return new AndThen(self, that);
}
/**
 * An empty patch
 *
 * @tsplus static effect/core/io/Supervisor.Ops emptyPatch
 */


const emptyPatch_1 = /*#__PURE__*/new Empty();
const emptyPatch = emptyPatch_1;
/**
 * Applies an update to the supervisor to produce a new supervisor.
 *
 * @tsplus static effect/core/io/Supervisor.Ops applyPatch
 */

exports.emptyPatch = emptyPatch;

function applyPatch_1(self, supervisor) {
  /**
   * @tsplus tailRec
   */
  function loop(supervisor, patches) {
    if (tsplus_module_1.isCons(patches)) {
      switch (patches.head._tag) {
        case "AddSupervisor":
          {
            return loop(supervisor.zip(patches.head.supervisor), patches.tail);
          }

        case "RemoveSupervisor":
          {
            return loop(removeSupervisor(supervisor, patches.head.supervisor), patches.tail);
          }

        case "AndThen":
          {
            return loop(supervisor, tsplus_module_1.cons(patches.head.first, tsplus_module_1.cons(patches.head.second, patches.tail)));
          }

        case "Empty":
          {
            return loop(supervisor, patches.tail);
          }
      }
    }

    return supervisor;
  }

  return loop(supervisor, tsplus_module_2.make(self));
}

function removeSupervisor(self, that) {
  if ((0, _Equals.equals)(self, that)) {
    return tsplus_module_3.none;
  } else {
    if (self instanceof tsplus_module_3.Zip) {
      return removeSupervisor(self.left, that).zip(removeSupervisor(self.right, that));
    } else {
      return self;
    }
  }
}

function toSet(self) {
  if ((0, _Equals.equals)(self, tsplus_module_3.none)) {
    return tsplus_module_4.empty();
  } else {
    if (self instanceof tsplus_module_3.Zip) {
      return tsplus_module_5.union(toSet(self.right))(toSet(self.left));
    } else {
      return tsplus_module_6.make(self);
    }
  }
}
/**
 * @tsplus static effect/core/io/Supervisor.Ops diff
 */


function diff(oldValue, newValue) {
  if ((0, _Equals.equals)(oldValue, newValue)) {
    return emptyPatch_1;
  }

  const oldSupervisors = toSet(oldValue);
  const newSupervisors = toSet(newValue);
  const added = tsplus_module_8.reduce(emptyPatch_1, (p, s) => combinePatch_1(p, new AddSupervisor(s)))(tsplus_module_7.difference(oldSupervisors)(newSupervisors));
  const removed = tsplus_module_8.reduce(emptyPatch_1, (p, s) => combinePatch_1(p, new RemoveSupervisor(s)))(tsplus_module_7.difference(newSupervisors)(oldSupervisors));
  return combinePatch_1(added, removed);
}
/**
 * @tsplus static effect/core/io/Supervisor.Ops differ
 */


const differ = /*#__PURE__*/tsplus_module_9.make({
  empty: emptyPatch_1,
  patch: applyPatch_1,
  combine: combinePatch_1,
  diff
});
exports.differ = differ;
//# sourceMappingURL=patch.js.map