"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.publish = publish;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/TRef/operations/unsafeGet"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/THub/operations/THub/capacity"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/TRef/operations/unsafeMake"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/THub/definition"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/THub/operations/_internal/InternalNode"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/TRef/operations/unsafeSet"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/THub/definition/THub"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stm/STM/operations/effect"));

var _STM = /*#__PURE__*/require("@effect/core/stm/STM");

var _InternalTHub = /*#__PURE__*/require("@effect/core/stm/THub/operations/_internal/InternalTHub");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Publishes a message to the hub, returning whether the message was published
 * to the hub.
 *
 * @tsplus static effect/core/stm/THub.Aspects publish
 * @tsplus pipeable effect/core/stm/THub publish
 */
function publish(value) {
  return self => {
    void 0;
    return tsplus_module_8.effect((journal, fiberId) => {
      const currentPublisherTail = tsplus_module_1.unsafeGet(journal)(self.publisherTail);

      if (currentPublisherTail == null) {
        throw new _STM.STMInterruptException(fiberId);
      }

      const currentSubscriberCount = tsplus_module_1.unsafeGet(journal)(self.subscriberCount);

      if (currentSubscriberCount === 0) {
        return true;
      }

      const currentHubSize = tsplus_module_1.unsafeGet(journal)(self.hubSize);

      if (currentHubSize < tsplus_module_2.capacity(self)) {
        const updatedPublisherTail = tsplus_module_3.unsafeMake(undefined);
        const updatedNode = tsplus_module_5.make(value, currentSubscriberCount, updatedPublisherTail);
        tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherTail);
        tsplus_module_6.unsafeSet(updatedPublisherTail, journal)(self.publisherTail);
        tsplus_module_6.unsafeSet(currentHubSize + 1, journal)(self.hubSize);
        return true;
      }

      switch (self.strategy) {
        case tsplus_module_7.BackPressure:
          throw new _STM.STMRetryException();

        case tsplus_module_7.Dropping:
          return false;

        case tsplus_module_7.Sliding:
          {
            if (tsplus_module_2.capacity(self) > 0) {
              let currentPublisherHead = tsplus_module_1.unsafeGet(journal)(self.publisherHead);
              let loop = true;

              while (loop) {
                const node = tsplus_module_1.unsafeGet(journal)(currentPublisherHead);

                if (node == null) {
                  throw new _STM.STMRetryException();
                } else {
                  const head = node.head;
                  const tail = node.tail;

                  if (head != null) {
                    const updatedNode = tsplus_module_5.make(undefined, node.subscribers, node.tail);
                    tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherHead);
                    tsplus_module_6.unsafeSet(tail, journal)(self.publisherHead);
                    loop = false;
                  } else {
                    currentPublisherHead = tail;
                  }
                }
              }

              const updatedPublisherTail = tsplus_module_3.unsafeMake(undefined);
              const updatedNode = tsplus_module_5.make(value, currentSubscriberCount, updatedPublisherTail);
              tsplus_module_6.unsafeSet(updatedNode, journal)(currentPublisherTail);
              tsplus_module_6.unsafeSet(updatedPublisherTail, journal)(self.publisherTail);
            }

            return true;
          }
      }

      return false;
    });
  };
}
//# sourceMappingURL=publish.js.map