"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SubexecutorSym = exports.Subexecutor = exports.PullFromUpstream = exports.PullFromChild = exports.Emit = exports.DrainChildExecutors = void 0;
exports.concreteSubexecutor = concreteSubexecutor;
exports.drainChildExecutors = drainChildExecutors;
exports.emit = emit;
exports.pullFromChild = pullFromChild;
exports.pullFromUpstream = pullFromUpstream;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Collection/functions"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/exit"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/zipRight"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipWith"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/done"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableQueue/append"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var _a, _b, _c, _d;

const SubexecutorSym = /*#__PURE__*/Symbol.for("@effect/core/stream/Channel/Subexecutor");
exports.SubexecutorSym = SubexecutorSym;
const Subexecutor = {};
/**
 * Execute upstreamExecutor and for each emitted element, spawn a child
 * channel and continue with processing it by `PullFromChild`.
 */

exports.Subexecutor = Subexecutor;

class PullFromUpstream {
  constructor(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit) {
    this.upstreamExecutor = upstreamExecutor;
    this.createChild = createChild;
    this.lastDone = lastDone;
    this.activeChildExecutors = activeChildExecutors;
    this.combineChildResults = combineChildResults;
    this.combineWithChildResult = combineWithChildResult;
    this.onPull = onPull;
    this.onEmit = onEmit;
    this._tag = "PullFromUpstream";
    this[_a] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.upstreamExecutor.close(exit);
    const fins = tsplus_module_1.append(fin1)(tsplus_module_1.map(child => child != null ? child.childExecutor.close(exit) : undefined)(this.activeChildExecutors));
    const result = tsplus_module_1.reduce(undefined, (acc, next) => {
      if (acc != null && next != null) {
        return tsplus_module_4.zipWith(tsplus_module_2.exit(next), (a, b) => tsplus_module_3.zipRight(b)(a))(acc);
      } else if (acc != null) {
        return acc;
      } else if (next != null) {
        return tsplus_module_2.exit(next);
      } else {
        return undefined;
      }
    })(fins);
    return result == null ? result : tsplus_module_6.flatMap(exit => tsplus_module_5.done(exit))(result);
  }

  enqueuePullFromChild(child) {
    return new PullFromUpstream(this.upstreamExecutor, this.createChild, this.lastDone, tsplus_module_7.append(child)(this.activeChildExecutors), this.combineChildResults, this.combineWithChildResult, this.onPull, this.onEmit);
  }

}

exports.PullFromUpstream = PullFromUpstream;
_a = SubexecutorSym;
/**
 * Execute the childExecutor and on each emitted value, decide what to do by
 * `onEmit`.
 */

class PullFromChild {
  constructor(childExecutor, parentSubexecutor, onEmit) {
    this.childExecutor = childExecutor;
    this.parentSubexecutor = parentSubexecutor;
    this.onEmit = onEmit;
    this._tag = "PullFromChild";
    this[_b] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.childExecutor.close(exit);
    const fin2 = this.parentSubexecutor.close(exit);

    if (fin1 != null && fin2 != null) {
      return tsplus_module_6.flatMap(exit => tsplus_module_5.done(exit))(tsplus_module_4.zipWith(tsplus_module_2.exit(fin2), (a, b) => tsplus_module_3.zipRight(b)(a))(tsplus_module_2.exit(fin1)));
    } else if (fin1 != null) {
      return fin1;
    } else if (fin2 != null) {
      return fin2;
    } else {
      return undefined;
    }
  }

  enqueuePullFromChild(_) {
    return this;
  }

}

exports.PullFromChild = PullFromChild;
_b = SubexecutorSym;

class DrainChildExecutors {
  constructor(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull) {
    this.upstreamExecutor = upstreamExecutor;
    this.lastDone = lastDone;
    this.activeChildExecutors = activeChildExecutors;
    this.upstreamDone = upstreamDone;
    this.combineChildResults = combineChildResults;
    this.combineWithChildResult = combineWithChildResult;
    this.onPull = onPull;
    this._tag = "DrainChildExecutors";
    this[_c] = SubexecutorSym;
  }

  close(exit) {
    const fin1 = this.upstreamExecutor.close(exit);
    const fins = tsplus_module_1.append(fin1)(tsplus_module_1.map(child => child != null ? child.childExecutor.close(exit) : undefined)(this.activeChildExecutors));
    return tsplus_module_1.reduce(undefined, (acc, next) => {
      if (acc != null && next != null) {
        return tsplus_module_4.zipWith(tsplus_module_2.exit(next), (a, b) => tsplus_module_3.zipRight(b)(a))(acc);
      } else if (acc != null) {
        return acc;
      } else if (next != null) {
        return tsplus_module_2.exit(next);
      } else {
        return undefined;
      }
    })(fins);
  }

  enqueuePullFromChild(child) {
    return new DrainChildExecutors(this.upstreamExecutor, this.lastDone, tsplus_module_7.append(child)(this.activeChildExecutors), this.upstreamDone, this.combineChildResults, this.combineWithChildResult, this.onPull);
  }

}

exports.DrainChildExecutors = DrainChildExecutors;
_c = SubexecutorSym;

class Emit {
  constructor(value, next) {
    this.value = value;
    this.next = next;
    this._tag = "Emit";
    this[_d] = SubexecutorSym;
  }

  close(exit) {
    return this.next.close(exit);
  }

  enqueuePullFromChild(_child) {
    return this;
  }

}

exports.Emit = Emit;
_d = SubexecutorSym;
/**
 * @tsplus macro remove
 */

function concreteSubexecutor(_) {//
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops PullFromUpstream
 */


function pullFromUpstream(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit) {
  return new PullFromUpstream(upstreamExecutor, createChild, lastDone, activeChildExecutors, combineChildResults, combineWithChildResult, onPull, onEmit);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops PullFromChild
 */


function pullFromChild(childExecutor, parentSubexecutor, onEmit) {
  return new PullFromChild(childExecutor, parentSubexecutor, onEmit);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops DrainChildExecutors
 */


function drainChildExecutors(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull) {
  return new DrainChildExecutors(upstreamExecutor, lastDone, activeChildExecutors, upstreamDone, combineChildResults, combineWithChildResult, onPull);
}
/**
 * @tsplus static effect/core/stream/Channel/Subexecutor.Ops Emit
 */


function emit(value, next) {
  return new Emit(value, next);
}
//# sourceMappingURL=Subexecutor.js.map