import { Lazy } from "@tsplus/stdlib/data/Function";
import { Cause } from "@effect/core/io/Cause/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Exit } from "@effect/core/io/Exit/definition";
import { Env } from "@tsplus/stdlib/service/Env";
import type { ChildExecutorDecision } from "@effect/core/stream/Channel/ChildExecutorDecision";
import type { Channel } from "@effect/core/stream/Channel/definition/base";
import { ChannelBase } from "@effect/core/stream/Channel/definition/base";
import { _Env, _InDone, _InElem, _InErr, _OutDone, _OutDone2, _OutElem, _OutErr, _OutErr2 } from "@effect/core/stream/Channel/definition/symbols";
import type { AsyncInputProducer } from "@effect/core/stream/Channel/SingleProducerAsyncInput";
import type { UpstreamPullRequest } from "@effect/core/stream/Channel/UpstreamPullRequest";
import type { UpstreamPullStrategy } from "@effect/core/stream/Channel/UpstreamPullStrategy";
export declare class PipeTo<Env, InErr, InElem, InDone, OutErr2, OutElem2, OutDone2, OutErr, OutElem, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr2, OutElem2, OutDone2> {
    readonly left: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>>;
    readonly right: Lazy<Channel<Env, OutErr, OutElem, OutDone, OutErr2, OutElem2, OutDone2>>;
    readonly _tag = "PipeTo";
    constructor(left: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>>, right: Lazy<Channel<Env, OutErr, OutElem, OutDone, OutErr2, OutElem2, OutDone2>>);
}
export declare class Read<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2, OutErr, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2> {
    readonly more: (i: InElem) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>;
    readonly done: ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>;
    readonly _tag = "Read";
    constructor(more: (i: InElem) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>, done: ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>);
}
export declare class SucceedNow<OutDone> extends ChannelBase<never, unknown, unknown, unknown, never, never, OutDone> {
    readonly terminal: OutDone;
    readonly _tag = "SucceedNow";
    constructor(terminal: OutDone);
}
export declare class Fail<OutErr> extends ChannelBase<never, unknown, unknown, unknown, OutErr, never, never> {
    readonly error: Lazy<Cause<OutErr>>;
    readonly _tag = "Fail";
    constructor(error: Lazy<Cause<OutErr>>);
}
export declare class FromEffect<Env, OutErr, OutDone> extends ChannelBase<Env, unknown, unknown, unknown, OutErr, never, OutDone> {
    readonly effect: Effect<Env, OutErr, OutDone>;
    readonly _tag = "FromEffect";
    constructor(effect: Effect<Env, OutErr, OutDone>);
}
export declare class Emit<OutElem, OutDone> extends ChannelBase<never, unknown, unknown, unknown, never, OutElem, OutDone> {
    readonly out: OutElem;
    readonly _tag = "Emit";
    constructor(out: OutElem);
}
export declare class Succeed<OutDone> extends ChannelBase<never, unknown, unknown, unknown, never, never, OutDone> {
    readonly effect: Lazy<OutDone>;
    readonly _tag = "Succeed";
    constructor(effect: Lazy<OutDone>);
}
export declare class Suspend<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> {
    readonly effect: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>>;
    readonly _tag = "Suspend";
    constructor(effect: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>>);
}
export declare class Ensuring<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> {
    readonly channel: Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>;
    readonly finalizer: (exit: Exit<OutErr, OutDone>) => Effect<Env, never, unknown>;
    readonly _tag = "Ensuring";
    constructor(channel: Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>, finalizer: (exit: Exit<OutErr, OutDone>) => Effect<Env, never, unknown>);
}
export declare class ConcatAll<Env, InErr, InElem, InDone, OutErr, OutElem2, OutDone3, OutElem, OutDone, OutDone2> extends ChannelBase<Env, InErr, InElem, InDone, OutErr, OutElem2, OutDone3> {
    readonly combineInners: (x: OutDone, y: OutDone) => OutDone;
    readonly combineAll: (x: OutDone, y: OutDone2) => OutDone3;
    readonly onPull: (pr: UpstreamPullRequest<OutElem>) => UpstreamPullStrategy<OutElem2>;
    readonly onEmit: (o: OutElem2) => ChildExecutorDecision;
    readonly value: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone2>>;
    readonly k: (o: OutElem) => Channel<Env, InErr, InElem, InDone, OutErr, OutElem2, OutDone>;
    readonly _tag = "ConcatAll";
    constructor(combineInners: (x: OutDone, y: OutDone) => OutDone, combineAll: (x: OutDone, y: OutDone2) => OutDone3, onPull: (pr: UpstreamPullRequest<OutElem>) => UpstreamPullStrategy<OutElem2>, onEmit: (o: OutElem2) => ChildExecutorDecision, value: Lazy<Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone2>>, k: (o: OutElem) => Channel<Env, InErr, InElem, InDone, OutErr, OutElem2, OutDone>);
}
export declare class Fold<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2, OutErr, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2> {
    readonly value: Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>;
    readonly k: ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>;
    readonly _tag = "Fold";
    constructor(value: Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>, k: ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>);
}
export declare class Bridge<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> extends ChannelBase<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> {
    readonly input: AsyncInputProducer<InErr, InElem, InDone>;
    readonly channel: Channel<Env, unknown, unknown, unknown, OutErr, OutElem, OutDone>;
    readonly _tag = "Bridge";
    constructor(input: AsyncInputProducer<InErr, InElem, InDone>, channel: Channel<Env, unknown, unknown, unknown, OutErr, OutElem, OutDone>);
}
export declare class BracketOut<R, E, Z, OutDone> extends ChannelBase<R, unknown, unknown, unknown, E, Z, OutDone> {
    readonly acquire: Lazy<Effect<R, E, Z>>;
    readonly finalizer: (z: Z, exit: Exit<unknown, unknown>) => Effect<R, never, unknown>;
    readonly _tag = "BracketOut";
    constructor(acquire: Lazy<Effect<R, E, Z>>, finalizer: (z: Z, exit: Exit<unknown, unknown>) => Effect<R, never, unknown>);
}
export declare class Provide<R, InErr, InElem, InDone, OutErr, OutElem, OutDone> extends ChannelBase<never, InErr, InElem, InDone, OutErr, OutElem, OutDone> {
    readonly env: Env<R>;
    readonly channel: Channel<R, InErr, InElem, InDone, OutErr, OutElem, OutDone>;
    readonly _tag = "Provide";
    constructor(env: Env<R>, channel: Channel<R, InErr, InElem, InDone, OutErr, OutElem, OutDone>);
}
/**
 * @tsplus macro remove
 */
export declare function concrete<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>(_: Channel<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>): asserts _ is PipeTo<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone, any, any, any> | Read<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone, any, any> | SucceedNow<OutDone> | Fail<OutErr> | FromEffect<Env, OutErr, OutDone> | Emit<OutElem, OutDone> | Succeed<OutDone> | Suspend<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> | Ensuring<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> | ConcatAll<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone, any, any, any> | Fold<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone, any, any> | Bridge<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone> | BracketOut<Env, OutErr, OutElem, OutDone> | Provide<Env, InErr, InElem, InDone, OutErr, OutElem, OutDone>;
export declare abstract class Continuation<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2> {
    readonly [_Env]: () => Env;
    readonly [_InErr]: (_: InErr) => void;
    readonly [_InElem]: (_: InElem) => void;
    readonly [_InDone]: (_: InDone) => void;
    readonly [_OutErr]: (_: OutErr) => OutErr;
    readonly [_OutDone]: (_: OutDone) => OutDone;
    readonly [_OutErr2]: () => OutErr2;
    readonly [_OutElem]: () => OutElem;
    readonly [_OutDone2]: () => OutDone2;
}
/**
 * @tsplus macro remove
 */
export declare function concreteContinuation<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>(_: Continuation<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2>): asserts _ is ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2> | ContinuationFinalizer<Env, OutErr, OutDone>;
export declare class ContinuationK<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2> extends Continuation<Env, InErr, InElem, InDone, OutErr, OutErr2, OutElem, OutDone, OutDone2> {
    readonly onSuccess: (o: OutDone) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>;
    readonly onHalt: (c: Cause<OutErr>) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>;
    readonly _tag = "ContinuationK";
    constructor(onSuccess: (o: OutDone) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>, onHalt: (c: Cause<OutErr>) => Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>);
    onExit(exit: Exit<OutErr, OutDone>): Channel<Env, InErr, InElem, InDone, OutErr2, OutElem, OutDone2>;
}
export declare class ContinuationFinalizer<Env, OutErr, OutDone> extends Continuation<Env, unknown, unknown, unknown, OutErr, never, never, OutDone, never> {
    readonly finalizer: (exit: Exit<OutErr, OutDone>) => Effect<Env, never, unknown>;
    readonly _tag = "ContinuationFinalizer";
    constructor(finalizer: (exit: Exit<OutErr, OutDone>) => Effect<Env, never, unknown>);
}
//# sourceMappingURL=primitives.d.ts.map