"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.foldWeightedDecomposeEffect = foldWeightedDecomposeEffect;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Sink/operations/suspend"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/isNonEmpty"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/succeed"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/write"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/flatMap"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/fromEffect"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/fail"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/readWith"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/empty"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/sync"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/unsafeGet"));

var tsplus_module_12 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_13 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/drop"));

var tsplus_module_14 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/map"));

var tsplus_module_15 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/concat"));

var _SinkInternal = /*#__PURE__*/require("@effect/core/stream/Sink/operations/_internal/SinkInternal");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Creates a sink that effectfully folds elements of type `In` into a
 * structure of type `S`, until `max` worth of elements (determined by the
 * `costFn`) have been folded.
 *
 * The `decompose` function will be used for decomposing elements that cause
 * an `S` aggregate to cross `max` into smaller elements. Be vigilant with
 * this function, it has to generate "simpler" values or the fold may never
 * end. A value is considered indivisible if `decompose` yields the empty
 * chunk or a single-valued chunk. In these cases, there is no other choice
 * than to yield a value that will cross the threshold.
 *
 * @tsplus static effect/core/stream/Sink.Ops foldWeightedDecomposeEffect
 */
function foldWeightedDecomposeEffect(z, costFn, max, decompose, f) {
  return tsplus_module_1.suspend(() => new _SinkInternal.SinkInternal(go(z, costFn, max, decompose, f, false, 0)));
}

function go(s, costFn, max, decompose, f, dirty, cost) {
  return tsplus_module_8.readWith(chunk => tsplus_module_5.flatMap(([nextS, nextCost, nextDirty, leftovers]) => tsplus_module_2.isNonEmpty(leftovers) ? tsplus_module_5.flatMap(() => tsplus_module_3.succeed(nextS))(tsplus_module_4.write(leftovers)) : cost > max ? tsplus_module_3.succeed(nextS) : go(nextS, costFn, max, decompose, f, nextDirty, nextCost))(tsplus_module_6.fromEffect(fold(chunk, s, costFn, max, decompose, f, dirty, cost, 0))), err => tsplus_module_7.fail(err), () => tsplus_module_3.succeed(s));
}

function fold(input, s, costFn, max, decompose, f, dirty, cost, index) {
  if (index === input.length) {
    return tsplus_module_10.sync(() => [s, cost, dirty, tsplus_module_9.empty()]);
  }

  const elem = tsplus_module_11.unsafeGet(index)(input);
  return tsplus_module_12.flatMap(total => {
    if (total <= max) {
      return tsplus_module_12.flatMap(s => fold(input, s, costFn, max, decompose, f, true, total, index + 1))(f(s, elem));
    }

    return tsplus_module_12.flatMap(decomposed => {
      if (decomposed.length <= 1 && !dirty) {
        // If `elem` cannot be decomposed, we need to cross the `max` threshold. To
        // minimize "injury", we only allow this when we haven't added anything else
        // to the aggregate (dirty = false).
        return tsplus_module_14.map(s => [s, total, true, tsplus_module_13.drop(index + 1)(input)])(f(s, elem));
      }

      if (decomposed.length <= 1 && dirty) {
        // If the state is dirty and `elem` cannot be decomposed, we stop folding
        // and include `elem` in the leftovers.
        return tsplus_module_10.sync(() => [s, cost, dirty, tsplus_module_13.drop(index)(input)]);
      } // `elem` got decomposed, so we will recurse with the decomposed elements pushed
      // into the chunk we're processing and see if we can aggregate further.


      return fold(tsplus_module_15.concat(tsplus_module_13.drop(index + 1)(input))(decomposed), s, costFn, max, decompose, f, dirty, cost, 0);
    })(decompose(elem));
  })(tsplus_module_14.map(addedCost => cost + addedCost)(costFn(s, elem)));
}
//# sourceMappingURL=foldWeightedDecomposeEffect.js.map