import { Sink } from "@effect/core/stream/Sink/definition/base";
import { Cause } from "@effect/core/io/Cause/definition";
import { LogLevel } from "@effect/core/io/LogLevel/definition";
import { ImmutableMap } from "@tsplus/stdlib/collections/ImmutableMap/definition";
/**
 * Logs the specified message at the current log level.
 * @tsplus static effect/core/stream/Sink.Ops log
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function log(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the debug log level.
 * @tsplus static effect/core/stream/Sink.Ops logDebug
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logDebug(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the error log level.
 * @tsplus static effect/core/stream/Sink.Ops logError
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logError(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the error log level.
 * @tsplus static effect/core/stream/Sink.Ops logErrorCause
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logErrorCause(cause: Cause<unknown>): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the fatal log level.
 * @tsplus static effect/core/stream/Sink.Ops logFatal
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logFatal(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the info log level.
 * @tsplus static effect/core/stream/Sink.Ops logInfo
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logInfo(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the trace log level.
 * @tsplus static effect/core/stream/Sink.Ops logTrace
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logTrace(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Logs the specified message at the warning log level.
 * @tsplus static effect/core/stream/Sink.Ops logWarning
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logWarning(message: string): Sink<never, never, unknown, unknown, void>;
/**
 * Sets the log level for streams composed after this.
 * @tsplus static effect/core/stream/Sink.Ops logLevel
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logLevel(level: LogLevel): <R, E, In, L, Z>(sink: import("../definition").Sink<R, E, In, L, Z>) => import("../definition").Sink<R, E, In, L, Z>;
/**
 * Adjusts the label for the logging span for streams composed after this.
 * @tsplus static effect/core/stream/Sink.Ops logSpan
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logSpan(label: string): <R, E, In, L, Z>(sink: import("../definition").Sink<R, E, In, L, Z>) => import("../definition").Sink<R, E, In, L, Z>;
/**
 * Annotates each log in streams composed after this with the specified log
 * annotation.
 * @tsplus static effect/core/stream/Sink.Ops logAnnotate
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logAnnotate(key: string, value: string): <R, E, In, L, Z>(sink: import("../definition").Sink<R, E, In, L, Z>) => import("../definition").Sink<R, E, In, L, Z>;
/**
 * Retrieves the log annotations associated with the current scope.
 * @tsplus static effect/core/stream/Sink.Ops logAnnotations
 * @tsplus location "@effect/core/stream/Sink/operations/logging"
 */
export declare function logAnnotations(): Sink<never, never, unknown, unknown, ImmutableMap<string, string>>;
//# sourceMappingURL=logging.d.ts.map