"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.log = log;
exports.logAnnotate = logAnnotate;
exports.logAnnotations = logAnnotations;
exports.logDebug = logDebug;
exports.logError = logError;
exports.logErrorCause = logErrorCause;
exports.logFatal = logFatal;
exports.logInfo = logInfo;
exports.logLevel = logLevel;
exports.logSpan = logSpan;
exports.logTrace = logTrace;
exports.logWarning = logWarning;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/logging"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Sink/operations/fromEffect"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Fiber/_internal/runtime"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/as"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Sink/operations/unwrapScoped"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/LogSpan/definition"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/List/prepend"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/ImmutableMap/set"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Logs the specified message at the current log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops log
 */
function log(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.log(message));
}
/**
 * Logs the specified message at the debug log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logDebug
 */


function logDebug(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logDebug(message));
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logError
 */


function logError(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logError(message));
}
/**
 * Logs the specified message at the error log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logErrorCause
 */


function logErrorCause(cause) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logErrorCause(cause));
}
/**
 * Logs the specified message at the fatal log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logFatal
 */


function logFatal(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logFatal(message));
}
/**
 * Logs the specified message at the info log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logInfo
 */


function logInfo(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logInfo(message));
}
/**
 * Logs the specified message at the trace log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logTrace
 */


function logTrace(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logTrace(message));
}
/**
 * Logs the specified message at the warning log level.
 *
 * @tsplus static effect/core/stream/Sink.Ops logWarning
 */


function logWarning(message) {
  return tsplus_module_2.fromEffect(tsplus_module_1.logWarning(message));
}
/**
 * Sets the log level for streams composed after this.
 *
 * @tsplus static effect/core/stream/Sink.Ops logLevel
 */


function logLevel(level) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_4.as(sink)(tsplus_module_3.currentLogLevel.locallyScoped(level)));
}
/**
 * Adjusts the label for the logging span for streams composed after this.
 *
 * @tsplus static effect/core/stream/Sink.Ops logSpan
 */


function logSpan(label) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_8.flatMap(stack => {
    const now = Date.now();
    const logSpan = tsplus_module_6.apply(label, now);
    return tsplus_module_4.as(sink)(tsplus_module_3.currentLogSpan.locallyScoped(tsplus_module_7.prepend(logSpan)(stack)));
  })(tsplus_module_3.currentLogSpan.get));
}
/**
 * Annotates each log in streams composed after this with the specified log
 * annotation.
 *
 * @tsplus static effect/core/stream/Sink.Ops logAnnotate
 */


function logAnnotate(key, value) {
  return sink => tsplus_module_5.unwrapScoped(tsplus_module_8.flatMap(annotations => tsplus_module_4.as(sink)(tsplus_module_3.currentLogAnnotations.locallyScoped(tsplus_module_9.set(key, value)(annotations))))(tsplus_module_3.currentLogAnnotations.get));
}
/**
 * Retrieves the log annotations associated with the current scope.
 *
 * @tsplus static effect/core/stream/Sink.Ops logAnnotations
 */


function logAnnotations() {
  return tsplus_module_2.fromEffect(tsplus_module_3.currentLogAnnotations.get);
}
//# sourceMappingURL=logging.js.map