import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
import { Ord } from "@tsplus/stdlib/prelude/Ord/definition";
import { SortedByKey } from "@effect/core/stream/SortedByKey/definition";
import { Stream } from "@effect/core/stream/Stream/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { Exit } from "@effect/core/io/Exit/definition";
export declare class DrainLeft {
    readonly _tag = "DrainLeft";
}
export declare class DrainRight {
    readonly _tag = "DrainRight";
}
export declare class PullBoth {
    readonly _tag = "PullBoth";
}
export declare class PullLeft<K, B> {
    readonly rightChunk: Chunk<readonly [K, B]>;
    readonly _tag = "PullLeft";
    constructor(rightChunk: Chunk<readonly [K, B]>);
}
export declare class PullRight<K, A> {
    readonly leftChunk: Chunk<readonly [K, A]>;
    readonly _tag = "PullRight";
    constructor(leftChunk: Chunk<readonly [K, A]>);
}
/**
 * Zips this stream that is sorted by distinct keys and the specified stream
 * that is sorted by distinct keys to produce a new stream that is sorted by
 * distinct keys. Uses the functions `left`, `right`, and `both` to handle
 * the cases where a key and value exist in this stream, that stream, or
 * both streams.
 *
 * This allows zipping potentially unbounded streams of data by key in
 * constant space but the caller is responsible for ensuring that the
 * streams are sorted by distinct keys.
 * @tsplus static effect/core/stream/SortedByKey.Aspects zipAllSortedByKeyWith
 * @tsplus pipeable effect/core/stream/SortedByKey zipAllSortedByKeyWith
 * @tsplus static effect/core/stream/Stream.Aspects zipAllSortedByKeyWith
 * @tsplus pipeable effect/core/stream/Stream zipAllSortedByKeyWith
 * @tsplus location "@effect/core/stream/SortedByKey/operations/zipAllSortedByKeyWith"
 */
export declare function zipAllSortedByKeyWith<K, R2, E2, A2, A, C1, C2, C3>(ord: Ord<K>, that: SortedByKey<R2, E2, K, A2>, left: (a: A) => C1, right: (b: A2) => C2, both: (a: A, b: A2) => C3): <R, E>(self: import("../definition").SortedByKey<R, E, K, A>) => import("../../Stream").Stream<R2 | R, E2 | E, readonly [K, C1 | C2 | C3]>;
//# sourceMappingURL=zipAllSortedByKeyWith.d.ts.map