import { Ref } from "@effect/core/io/Ref/definition";
import { Deferred } from "@effect/core/io/Deferred/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
/**
 * A synchronous queue-like abstraction that allows a producer to offer an
 * element and wait for it to be taken, and allows a consumer to wait for an
 * element to be available.
 *
 * @tsplus type effect/core/stream/Stream/Handoff
 * @tsplus companion effect/core/stream/Stream/Handoff.Ops
 */
export declare class Handoff<A> {
    readonly ref: Ref<HandoffState<A>>;
    constructor(ref: Ref<HandoffState<A>>);
}
export type HandoffState<A> = Empty | Full<A>;
export declare class Empty {
    readonly notifyConsumer: Deferred<never, void>;
    readonly _tag = "Empty";
    constructor(notifyConsumer: Deferred<never, void>);
}
export declare class Full<A> {
    readonly value: A;
    readonly notifyProducer: Deferred<never, void>;
    readonly _tag = "Full";
    constructor(value: A, notifyProducer: Deferred<never, void>);
}
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Ops make
 * @tsplus location "@effect/core/stream/Stream/operations/_internal/Handoff"
 */
export declare function make<A>(): Effect<never, never, Handoff<A>>;
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Aspects offer
 * @tsplus pipeable effect/core/stream/Stream/Handoff offer
 * @tsplus location "@effect/core/stream/Stream/operations/_internal/Handoff"
 */
export declare function offer<A>(value: A): (self: Handoff<A>) => Effect<never, never, void>;
/**
 * @tsplus getter effect/core/stream/Stream/Handoff take
 * @tsplus location "@effect/core/stream/Stream/operations/_internal/Handoff"
 */
export declare function take<A>(self: Handoff<A>): Effect<never, never, A>;
/**
 * @tsplus getter effect/core/stream/Stream/Handoff poll
 * @tsplus location "@effect/core/stream/Stream/operations/_internal/Handoff"
 */
export declare function poll<A>(self: Handoff<A>): Effect<never, never, Maybe<A>>;
//# sourceMappingURL=Handoff.d.ts.map