"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Handoff = exports.Full = exports.Empty = void 0;
exports.make = make;
exports.offer = void 0;
exports.poll = poll;
exports.take = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Ref/operations/make"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Deferred/operations/make"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatMap"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/map"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipRight"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/flatten"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/as"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const offer = offer_1;
exports.offer = offer;
const take = take_1;
/**
 * A synchronous queue-like abstraction that allows a producer to offer an
 * element and wait for it to be taken, and allows a consumer to wait for an
 * element to be available.
 *
 * @tsplus type effect/core/stream/Stream/Handoff
 * @tsplus companion effect/core/stream/Stream/Handoff.Ops
 */

exports.take = take;

class Handoff {
  constructor(ref) {
    this.ref = ref;
  }

}

exports.Handoff = Handoff;

class Empty {
  constructor(notifyConsumer) {
    this.notifyConsumer = notifyConsumer;
    this._tag = "Empty";
  }

}

exports.Empty = Empty;

class Full {
  constructor(value, notifyProducer) {
    this.value = value;
    this.notifyProducer = notifyProducer;
    this._tag = "Full";
  }

}
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Ops make
 */


exports.Full = Full;

function make() {
  return tsplus_module_4.map(state => new Handoff(state))(tsplus_module_3.flatMap(deferred => tsplus_module_1.makeRef(() => new Empty(deferred)))(tsplus_module_2.make()));
}
/**
 * @tsplus static effect/core/stream/Stream/Handoff.Aspects offer
 * @tsplus pipeable effect/core/stream/Stream/Handoff offer
 */


function offer_1(value) {
  return self => tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_5.zipRight(deferred.await)(state.notifyConsumer.succeed(undefined)), new Full(value, deferred)];
        }

      case "Full":
        {
          return [tsplus_module_5.zipRight(offer_1(value)(self))(state.notifyProducer.await), state];
        }
    }
  })))(tsplus_module_2.make());
}
/**
 * @tsplus getter effect/core/stream/Stream/Handoff take
 */


function take_1(self) {
  return tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_5.zipRight(take_1(self))(state.notifyConsumer.await), state];
        }

      case "Full":
        {
          return [tsplus_module_7.as(state.value)(state.notifyProducer.succeed(undefined)), new Empty(deferred)];
        }
    }
  })))(tsplus_module_2.make());
}
/**
 * @tsplus getter effect/core/stream/Stream/Handoff poll
 */


function poll(self) {
  return tsplus_module_3.flatMap(deferred => tsplus_module_6.flatten(self.ref.modify(state => {
    switch (state._tag) {
      case "Empty":
        {
          return [tsplus_module_9.succeed(tsplus_module_8.none), state];
        }

      case "Full":
        {
          return [tsplus_module_7.as(tsplus_module_8.some(state.value))(state.notifyProducer.succeed(undefined)), new Empty(deferred)];
        }
    }
  })))(tsplus_module_2.make());
}
//# sourceMappingURL=Handoff.js.map