import { Sink } from "@effect/core/stream/Sink/definition/base";
import { Schedule } from "@effect/core/io/Schedule/definition";
import { Maybe } from "@tsplus/stdlib/data/Maybe/definition";
import { Stream } from "@effect/core/stream/Stream/definition";
import { Either } from "@tsplus/stdlib/data/Either/definition";
import { Channel } from "@effect/core/stream/Channel/definition/base";
import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
import { Effect } from "@effect/core/io/Effect/definition";
import { Ref } from "@effect/core/io/Ref/definition";
import { Scope } from "@effect/core/io/Scope/definition";
import { Fiber } from "@effect/core/io/Fiber/definition";
/**
 * Aggregates elements using the provided sink until it completes, or until
 * the delay signalled by the schedule has passed.
 *
 * This operator divides the stream into two asynchronous islands. Operators
 * upstream of this operator run on one fiber, while downstream operators run
 * on another. Elements will be aggregated by the sink until the downstream
 * fiber pulls the aggregated value, or until the schedule's delay has passed.
 *
 * Aggregated elements will be fed into the schedule to determine the delays
 * between pulls.
 * @param sink Used for the aggregation
 * @param schedule Used for signalling for when to stop the aggregation
 * @tsplus static effect/core/stream/Stream.Aspects aggregateWithinEither
 * @tsplus pipeable effect/core/stream/Stream aggregateWithinEither
 * @tsplus location "@effect/core/stream/Stream/operations/aggregateWithinEither"
 */
export declare function aggregateWithinEither<A, R2, E2, A2, S, R3, B, C>(sink: Sink<R2, E2, A | A2, A2, B>, schedule: Schedule<S, R3, Maybe<B>, C>): <R, E>(self: import("../definition").Stream<R, E, A>) => import("../definition").Stream<R2 | R3 | R, E2 | E, import("@tsplus/stdlib/data/Either").Either<C, B>>;
//# sourceMappingURL=aggregateWithinEither.d.ts.map