import { Fiber } from "@effect/core/io/Fiber/definition";
import { Chunk } from "@tsplus/stdlib/collections/Chunk/definition";
import { Duration } from "@tsplus/stdlib/data/Duration";
import { Stream } from "@effect/core/stream/Stream/definition";
import { Channel } from "@effect/core/stream/Channel/definition/base";
/**
 * Delays the emission of values by holding new values for a set duration. If
 * no new values arrive during that time the value is emitted, however if a
 * new value is received during the holding period the previous value is
 * discarded and the process is repeated with the new value.
 *
 * This operator is useful if you have a stream of "bursty" events which
 * eventually settle down and you only need the final event of the burst.
 * @example A search engine may only want to initiate a search after a user has
paused typing so as to not prematurely recommend results.
 * @tsplus static effect/core/stream/Stream.Aspects debounce
 * @tsplus pipeable effect/core/stream/Stream debounce
 * @tsplus location "@effect/core/stream/Stream/operations/debounce"
 */
export declare function debounce<R, E, A>(duration: Duration): (self: Stream<R, E, A>) => Stream<R, E, A>;
//# sourceMappingURL=debounce.d.ts.map