import { Stream } from "@effect/core/stream/Stream/definition";
import { Cause } from "@effect/core/io/Cause/definition";
import { LogLevel } from "@effect/core/io/LogLevel/definition";
/**
 * Logs the specified message at the current log level.
 * @tsplus static effect/core/stream/Stream.Ops log
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function log(message: string): Stream<never, never, void>;
/**
 * Logs the specified message at the debug log level.
 * @tsplus static effect/core/stream/Stream.Ops logDebug
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logDebug(message: string): Stream<never, never, void>;
/**
 * Logs the specified message at the error log level.
 * @tsplus static effect/core/stream/Stream.Ops logError
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logError(message: string): Stream<never, never, void>;
/**
 * Logs the specified cause as an error.
 * @tsplus static effect/core/stream/Stream.Ops logErrorCause
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logErrorCause<E>(cause: Cause<E>): Stream<never, never, void>;
/**
 * Logs the specified message at the fatal log level.
 * @tsplus static effect/core/stream/Stream.Ops logFatal
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logFatal(message: string): Stream<never, never, void>;
/**
 * Logs the specified message at the informational log level.
 * @tsplus static effect/core/stream/Stream.Ops logInfo
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logInfo(message: string): Stream<never, never, void>;
/**
 * Logs the specified message at the warning log level.
 * @tsplus static effect/core/stream/Stream.Ops logWarning
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logWarning(message: string): Stream<never, never, void>;
/**
 * Logs the specified message at the trace log level.
 * @tsplus static effect/core/stream/Stream.Ops logTrace
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logTrace(message: string): Stream<never, never, void>;
/**
 * Sets the log level for streams composed after this.
 * @tsplus static effect/core/stream/Stream.Ops logLevel
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logLevel(level: LogLevel): Stream<never, never, void>;
/**
 * Adjusts the label for the logging span for streams composed after this.
 * @tsplus static effect/core/stream/Stream.Ops logSpan
 * @tsplus location "@effect/core/stream/Stream/operations/logging"
 */
export declare function logSpan(label: string): Stream<never, never, void>;
//# sourceMappingURL=logging.d.ts.map