"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sliding = sliding;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Cause/errors"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Stream/operations/dieSync"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/pipeTo"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Stream/operations/sync"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Stream/operations/flatMap"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/zipWithIndex"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/collect"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/write"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/flatMap"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/failCause"));

var tsplus_module_12 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/unit"));

var tsplus_module_13 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Channel/operations/readWithCause"));

var tsplus_module_14 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/isEmpty"));

var tsplus_module_15 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/empty"));

var tsplus_module_16 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/single"));

var tsplus_module_17 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/takeRight"));

var _StreamInternal = /*#__PURE__*/require("@effect/core/stream/Stream/operations/_internal/StreamInternal");

var _RingBufferNew = /*#__PURE__*/require("@effect/core/support/RingBufferNew");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Emits a sliding window of `n` elements.
 *
 * @tsplus static effect/core/stream/Stream.Aspects sliding
 * @tsplus pipeable effect/core/stream/Stream sliding
 */
function sliding(chunkSize, stepSize = 1) {
  return self => {
    if (chunkSize <= 0 || stepSize <= 0) {
      return tsplus_module_2.dieSync(() => new tsplus_module_1.IllegalArgumentException("Invalid bounds - `chunkSize` and `stepSize` must be greater than 0"));
    }

    return tsplus_module_5.flatMap(queue => {
      void 0;
      return new _StreamInternal.StreamInternal(tsplus_module_3.pipeTo(reader(chunkSize, stepSize, queue, 0))(self.channel));
    })(tsplus_module_4.sync(() => new _RingBufferNew.RingBufferNew(chunkSize)));
  };
}

function reader(chunkSize, stepSize, queue, queueSize) {
  return tsplus_module_13.readWithCause(input => tsplus_module_10.flatMap(() => reader(chunkSize, stepSize, queue, queueSize + input.length))(tsplus_module_9.write(tsplus_module_8.collect(([a, index]) => {
    queue.put(a);
    const currentIndex = queueSize + index + 1;
    return currentIndex < chunkSize || (currentIndex - chunkSize) % stepSize > 0 ? tsplus_module_6.none : tsplus_module_6.some(queue.toChunk());
  })(tsplus_module_7.zipWithIndex(input)))), cause => emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, tsplus_module_11.failCause(cause)), () => emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, tsplus_module_12.unit));
}

function emitOnStreamEnd(chunkSize, stepSize, queue, queueSize, channelEnd) {
  if (queueSize < chunkSize) {
    const items = queue.toChunk();
    const result = tsplus_module_14.isEmpty(items) ? tsplus_module_15.empty() : tsplus_module_16.single(items);
    return tsplus_module_10.flatMap(() => channelEnd)(tsplus_module_9.write(result));
  }

  const lastEmitIndex = queueSize - (queueSize - chunkSize) % stepSize;

  if (lastEmitIndex === queueSize) {
    return channelEnd;
  }

  const leftovers = queueSize - (lastEmitIndex - chunkSize + stepSize);
  const lastItems = tsplus_module_17.takeRight(leftovers)(queue.toChunk());
  const result = tsplus_module_14.isEmpty(lastItems) ? tsplus_module_15.empty() : tsplus_module_16.single(lastItems);
  return tsplus_module_10.flatMap(() => channelEnd)(tsplus_module_9.write(result));
}
//# sourceMappingURL=sliding.js.map