"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.zipAllWith = zipAllWith;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Stream/operations/combineChunks"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/isEmpty"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Either/fold"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/fail"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/map"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/succeed"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/foldEffect"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/unsome"));

var tsplus_module_11 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipPar"));

var tsplus_module_12 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/fold"));

var _zipChunks = /*#__PURE__*/require("@effect/core/stream/Stream/operations/_internal/zipChunks");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class DrainLeft {
  constructor() {
    this._tag = "DrainLeft";
  }

}

class DrainRight {
  constructor() {
    this._tag = "DrainRight";
  }

}

class PullBoth {
  constructor() {
    this._tag = "PullBoth";
  }

}

class PullLeft {
  constructor(rightChunk) {
    this.rightChunk = rightChunk;
    this._tag = "PullLeft";
  }

}

class PullRight {
  constructor(leftChunk) {
    this.leftChunk = leftChunk;
    this._tag = "PullRight";
  }

}
/**
 * Zips this stream with another point-wise. The provided functions will be
 * used to create elements for the composed stream.
 *
 * The functions `left` and `right` will be used if the streams have different
 * lengths and one of the streams has ended before the other.
 *
 * @tsplus static effect/core/stream/Stream.Aspects zipAllWith
 * @tsplus pipeable effect/core/stream/Stream zipAllWith
 */


function zipAllWith(that, left, right, both) {
  return self => tsplus_module_1.combineChunks(that, new PullBoth(), pull(left, right, both))(self);
}

function zipWithChunks(leftChunk, rightChunk, f) {
  const [out, either] = (0, _zipChunks.zipChunks)(leftChunk, rightChunk, f);
  return tsplus_module_3.fold(leftChunk => tsplus_module_2.isEmpty(leftChunk) ? [out, new PullBoth()] : [out, new PullRight(leftChunk)], rightChunk => tsplus_module_2.isEmpty(rightChunk) ? [out, new PullBoth()] : [out, new PullLeft(rightChunk)])(either);
}

function pull(left, right, both) {
  return (state, pullLeft, pullRight) => {
    switch (state._tag) {
      case "DrainLeft":
        {
          return tsplus_module_8.foldEffect(err => tsplus_module_5.succeed(tsplus_module_4.fail(err)), leftChunk => tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(left)(leftChunk), new DrainLeft()])))(pullLeft);
        }

      case "DrainRight":
        {
          return tsplus_module_8.foldEffect(err => tsplus_module_5.succeed(tsplus_module_4.fail(err)), rightChunk => tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(right)(rightChunk), new DrainRight()])))(pullRight);
        }

      case "PullBoth":
        {
          return tsplus_module_8.foldEffect(err => tsplus_module_5.succeed(tsplus_module_4.fail(tsplus_module_9.some(err))), ([l, r]) => {
            if (l._tag === "Some" && r._tag === "Some") {
              const leftChunk = l.value;
              const rightChunk = r.value;

              if (tsplus_module_2.isEmpty(leftChunk) && tsplus_module_2.isEmpty(rightChunk)) {
                return pull(left, right, both)(new PullBoth(), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(leftChunk)) {
                return pull(left, right, both)(new PullLeft(rightChunk), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(rightChunk)) {
                return pull(left, right, both)(new PullRight(leftChunk), pullLeft, pullRight);
              } else {
                return tsplus_module_5.succeed(tsplus_module_7.succeed(zipWithChunks(leftChunk, rightChunk, both)));
              }
            } else if (l._tag === "Some" && r._tag === "None") {
              return tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(left)(l.value), new DrainLeft()]));
            } else if (l._tag === "None" && r._tag === "Some") {
              return tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(right)(r.value), new DrainRight()]));
            } else {
              return tsplus_module_5.succeed(tsplus_module_4.fail(tsplus_module_9.none));
            }
          })(tsplus_module_11.zipPar(tsplus_module_10.unsome(pullRight))(tsplus_module_10.unsome(pullLeft)));
        }

      case "PullLeft":
        {
          return tsplus_module_8.foldEffect(option => tsplus_module_12.fold(() => tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(right)(state.rightChunk), new DrainRight()])), err => tsplus_module_5.succeed(tsplus_module_4.fail(tsplus_module_9.some(err))))(option), leftChunk => tsplus_module_2.isEmpty(leftChunk) ? pull(left, right, both)(new PullLeft(state.rightChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.rightChunk) ? pull(left, right, both)(new PullRight(leftChunk), pullLeft, pullRight) : tsplus_module_5.succeed(tsplus_module_7.succeed(zipWithChunks(leftChunk, state.rightChunk, both))))(pullLeft);
        }

      case "PullRight":
        {
          return tsplus_module_8.foldEffect(option => tsplus_module_12.fold(() => tsplus_module_5.succeed(tsplus_module_7.succeed([tsplus_module_6.map(left)(state.leftChunk), new DrainLeft()])), err => tsplus_module_5.succeed(tsplus_module_4.fail(tsplus_module_9.some(err))))(option), rightChunk => tsplus_module_2.isEmpty(rightChunk) ? pull(left, right, both)(new PullRight(state.leftChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.leftChunk) ? pull(left, right, both)(new PullLeft(rightChunk), pullLeft, pullRight) : tsplus_module_5.succeed(tsplus_module_7.succeed(zipWithChunks(state.leftChunk, rightChunk, both))))(pullRight);
        }
    }
  };
}
//# sourceMappingURL=zipAllWith.js.map