"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.zipWithChunks = zipWithChunks;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/stream/Stream/operations/combineChunks"));

var tsplus_module_2 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/collections/Chunk/operations/isEmpty"));

var tsplus_module_3 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Either/fold"));

var tsplus_module_4 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/Maybe/definition"));

var tsplus_module_5 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/fail"));

var tsplus_module_6 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/succeed"));

var tsplus_module_7 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Exit/operations/succeed"));

var tsplus_module_8 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/unsome"));

var tsplus_module_9 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/zipPar"));

var tsplus_module_10 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/core/io/Effect/operations/foldEffect"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class PullBoth {
  constructor() {
    this._tag = "PullBoth";
  }

}

class PullLeft {
  constructor(rightChunk) {
    this.rightChunk = rightChunk;
    this._tag = "PullLeft";
  }

}

class PullRight {
  constructor(leftChunk) {
    this.leftChunk = leftChunk;
    this._tag = "PullRight";
  }

}
/**
 * Zips this stream with another point-wise and applies the function to the
 * paired elements.
 *
 * The new stream will end when one of the sides ends.
 *
 * @tsplus static effect/core/stream/Stream.Aspects zipWithChunks
 * @tsplus pipeable effect/core/stream/Stream zipWithChunks
 */


function zipWithChunks(that, f) {
  return self => tsplus_module_1.combineChunks(that, new PullBoth(), pull(f))(self);
}

function zipWithChunksInternal(leftChunk, rightChunk, f) {
  const [out, either] = f(leftChunk, rightChunk);
  return tsplus_module_3.fold(leftChunk => tsplus_module_2.isEmpty(leftChunk) ? [out, new PullBoth()] : [out, new PullRight(leftChunk)], rightChunk => tsplus_module_2.isEmpty(rightChunk) ? [out, new PullBoth()] : [out, new PullLeft(rightChunk)])(either);
}

function pull(f) {
  return (state, pullLeft, pullRight) => {
    switch (state._tag) {
      case "PullBoth":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(tsplus_module_4.some(err))), ([left, right]) => {
            if (tsplus_module_4.isSome(left) && tsplus_module_4.isSome(right)) {
              const leftChunk = left.value;
              const rightChunk = right.value;

              if (tsplus_module_2.isEmpty(leftChunk) && tsplus_module_2.isEmpty(rightChunk)) {
                return pull(f)(new PullBoth(), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(leftChunk)) {
                return pull(f)(new PullLeft(rightChunk), pullLeft, pullRight);
              } else if (tsplus_module_2.isEmpty(rightChunk)) {
                return pull(f)(new PullRight(leftChunk), pullLeft, pullRight);
              } else {
                return tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(leftChunk, rightChunk, f)));
              }
            }

            return tsplus_module_6.succeed(tsplus_module_5.fail(tsplus_module_4.none));
          })(tsplus_module_9.zipPar(tsplus_module_8.unsome(pullRight))(tsplus_module_8.unsome(pullLeft)));
        }

      case "PullLeft":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(err)), leftChunk => tsplus_module_2.isEmpty(leftChunk) ? pull(f)(new PullLeft(state.rightChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.rightChunk) ? pull(f)(new PullRight(leftChunk), pullLeft, pullRight) : tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(leftChunk, state.rightChunk, f))))(pullLeft);
        }

      case "PullRight":
        {
          return tsplus_module_10.foldEffect(err => tsplus_module_6.succeed(tsplus_module_5.fail(err)), rightChunk => tsplus_module_2.isEmpty(rightChunk) ? pull(f)(new PullRight(state.leftChunk), pullLeft, pullRight) : tsplus_module_2.isEmpty(state.leftChunk) ? pull(f)(new PullLeft(rightChunk), pullLeft, pullRight) : tsplus_module_6.succeed(tsplus_module_7.succeed(zipWithChunksInternal(state.leftChunk, rightChunk, f))))(pullRight);
        }
    }
  };
}
//# sourceMappingURL=zipWithChunks.js.map