"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultScheduler = exports.StagedScheduler = exports.DefaultScheduler = void 0;

var tsplus_module_1 = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@tsplus/stdlib/data/AtomicReference"));

var _DoublyLinkedList = /*#__PURE__*/require("@tsplus/stdlib/collections/mutable/DoublyLinkedList");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class DefaultScheduler {
  constructor() {
    this.running = new tsplus_module_1.AtomicReference(false);
    this.tasks = new tsplus_module_1.AtomicReference(new _DoublyLinkedList.DoublyLinkedList());
    this.promise = Promise.resolve(void 0);
  }

  starveInternal(depth) {
    const toRun = this.tasks.get;
    this.tasks.set(new _DoublyLinkedList.DoublyLinkedList());
    toRun.forEach(task => {
      task();
    });

    if (this.tasks.get.isEmpty) {
      this.running.set(false);
    } else {
      this.starve(depth);
    }
  }

  starve(depth = 0) {
    if (depth >= 2048) {
      setTimeout(() => this.starveInternal(0), 0);
    } else {
      this.promise.then(() => this.starveInternal(depth + 1));
    }
  }

  scheduleTask(task) {
    this.tasks.get.add(task);

    if (!this.running.get) {
      this.running.set(true);
      this.starve();
    }
  }

}

exports.DefaultScheduler = DefaultScheduler;
const defaultScheduler = /*#__PURE__*/new DefaultScheduler();
exports.defaultScheduler = defaultScheduler;

class StagedScheduler {
  constructor() {
    this.tasks = new _DoublyLinkedList.DoublyLinkedList();
    this.deferred = new tsplus_module_1.AtomicReference(false);
  }

  scheduleTask(task) {
    if (this.deferred.get) {
      defaultScheduler.scheduleTask(task);
    } else {
      this.tasks.add(task);
    }
  }

  flush() {
    while (!this.tasks.isEmpty) {
      this.tasks.shift()();
    }

    this.deferred.set(true);
  }

}

exports.StagedScheduler = StagedScheduler;
//# sourceMappingURL=Scheduler.js.map