import { Gen } from "@effect/core/testing/Gen/definition";
import type { NumberConstraints } from "@effect/core/testing/Gen";
/**
 * A generator of alpha characters.
 * @tsplus static effect/core/testing/Gen.Ops alphaChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const alphaChar: Gen<never, string>;
/**
 * A generator of alphanumeric characters. Shrinks toward '0'.
 * @tsplus static effect/core/testing/Gen.Ops alphaNumericChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const alphaNumericChar: Gen<never, string>;
/**
 * A generator of US-ASCII characters. Shrinks toward '0'.
 * @tsplus static effect/core/testing/Gen.Ops asciiChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const asciiChar: Gen<never, string>;
/**
 * A generator of base-64 characters. Shrinks towards '0'.
 * @tsplus static effect/core/testing/Gen.Ops base64Char
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const base64Char: Gen<never, string>;
/**
 * A generator of character values inside the specified range: [start, end].
 * The shrinker will shrink toward the lower end of the range ("smallest").
 * @tsplus static effect/core/testing/Gen.Ops char
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare function char(constraints: Required<NumberConstraints>): Gen<never, string>;
/**
 * A generator of UTF-16 characters. Shrinks towards '\x00'.
 * @tsplus static effect/core/testing/Gen.Ops char16
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const char16: Gen<never, string>;
/**
 * @tsplus static effect/core/testing/Gen.Ops fullUnicodeChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const fullUnicodeChar: Gen<never, string>;
/**
 * A generator of numeric characters. Shrinks toward '0'.
 * @tsplus static effect/core/testing/Gen.Ops numericChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const numericChar: Gen<never, string>;
/**
 * A generator of hex chars(0-9,a-f,A-F).
 * @tsplus static effect/core/testing/Gen.Ops hexChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const hexChar: Gen<never, string>;
/**
 * A generator of lower hex chars(0-9, a-f).
 * @tsplus static effect/core/testing/Gen.Ops hexCharLower
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const hexCharLower: Gen<never, string>;
/**
 * A generator of upper hex chars(0-9, A-F).
 * @tsplus static effect/core/testing/Gen.Ops hexCharUpper
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const hexCharUpper: Gen<never, string>;
/**
 * A generator of printable characters. Shrinks toward '!'.
 * @tsplus static effect/core/testing/Gen.Ops printableChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const printableChar: Gen<never, string>;
/**
 * A generator of Unicode characters. Shrinks toward '0'.
 * @tsplus static effect/core/testing/Gen.Ops unicodeChar
 * @tsplus location "@effect/core/testing/Gen/operations/char"
 */
export declare const unicodeChar: Gen<never, string>;
//# sourceMappingURL=char.d.ts.map