import { Tag } from "@tsplus/stdlib/service/Tag";
import { Layer } from "@effect/core/io/Layer/definition";
import { Effect } from "@effect/core/io/Effect/definition";
/**
 * The `TestConfig` service provides access to default configuration settings
 * used by tests, including the number of times to repeat tests to ensure
 * they are stable, the number of times to retry flaky tests, the sufficient
 * number of samples to check from a random variable, and the maximum number of
 * shrinkings to minimize large failures.
 *
 * @tsplus type effect/core/testing/TestConfig
 */
export interface TestConfig {
    /**
     * The number of times to repeat tests to ensure they are stable.
     */
    readonly repeats: number;
    /**
     * The number of times to retry flaky tests.
     */
    readonly retries: number;
    /**
     * The number of sufficient samples to check for a random variable.
     */
    readonly samples: number;
    /**
     * The maximum number of shrinkings to minimize large failures
     */
    readonly shrinks: number;
}
/**
 * @tsplus type effect/core/testing/TestConfig.Ops
 */
export interface TestConfigOps {
    (params: {
        readonly repeats: number;
        readonly retries: number;
        readonly samples: number;
        readonly shrinks: number;
    }): TestConfig;
    readonly $: TestConfigAspects;
    readonly Tag: Tag<TestConfig>;
}
export declare const TestConfig: TestConfigOps;
/**
 * @tsplus type effect/core/testing/TestConfig.Aspects
 */
export interface TestConfigAspects {
}
/**
 * Constructs a new `TestConfig` service with the specified settings.
 * @tsplus static effect/core/testing/TestConfig.Ops live
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare function live(params: {
    readonly repeats: number;
    readonly retries: number;
    readonly samples: number;
    readonly shrinks: number;
}): Layer<never, never, TestConfig>;
/**
 * Constructs a new `TestConfig` with the default settings.
 * @tsplus static effect/core/testing/TestConfig.Ops default
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare const defaultTestConfig: Layer<never, never, TestConfig>;
/**
 * The number of times to repeat tests to ensure they are stable.
 * @tsplus static effect/core/testing/TestConfig.Ops repeats
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare const repeats: Effect<TestConfig, never, number>;
/**
 * The number of times to retry flaky tests.
 * @tsplus static effect/core/testing/TestConfig.Ops retries
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare const retries: Effect<TestConfig, never, number>;
/**
 * The number of sufficient samples to check for a random variable.
 * @tsplus static effect/core/testing/TestConfig.Ops samples
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare const samples: Effect<TestConfig, never, number>;
/**
 * The maximum number of shrinkings to minimize large failures.
 * @tsplus static effect/core/testing/TestConfig.Ops shrinks
 * @tsplus location "@effect/core/testing/TestConfig"
 */
export declare const shrinks: Effect<TestConfig, never, number>;
//# sourceMappingURL=TestConfig.d.ts.map